@@ -1,7 +1,7 @@
-/* $Id: about.cpp,v 1.113 2005/04/09 21:59:00 ettin Exp $ */
+/* $Id: about.cpp,v 1.133 2005/06/27 02:54:46 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -30,26 +30,12 @@ namespace about
 
 std::vector<std::string> get_text() {
 	static const char *credits[] = {
-		" ",
-		"- ",
-		" ",
-		"- ",
-		" ",
-		"- ",
-		" ",
-		"- ",
-		" ",
-		"- ",
-		" ",
-		"- ",
-
 		"_" N_("+Core Developers"),
 		"_" N_("-   Main Developer"),
 		"   David White (Sirp)",
 		"- ",
 		"_" N_("-   Artwork and graphics designer"),
 		"   Francisco Muñoz (fmunoz)",
-		"+ ",
 
 		"_" N_("+Developers"),
 		"-   Alfredo Beaumont (ziberpunk)",
@@ -59,19 +45,19 @@ std::vector<std::string> get_text() {
 		"-   Isaac Clerencia",
 		"-   Jan Zvánovec (jaz)",
 		"-   John B. Messerly",
+		"-   Jon Daniel (forcemstr)",
 		"-   Justin Zaun (jzaun)",
 		"-   J.R. Blain (Cowboy)",
 		"-   Kristoffer Erlandsson (erl)",
 		"-   Maksim Orlovich (SadEagle)",
+		"-   ott",
 		"-   Philippe Plantier (Ayin)",
 		"-   Yann Dirson",
 		"-   Zas",
-		"+ ",
 
 		"_" N_("+General Purpose Administrators"),
 		"-   Crossbow/Miyo",
 		"-   Isaac Clerencia",
-		"+ ",
 
 		"_" N_("+Artwork and graphics designers"),
 		"-   Andrew James Patterson (Kamahawk)",
@@ -82,6 +68,7 @@ std::vector<std::string> get_text() {
 		"-   Gareth Miller (Gafgarion)",
 		"-   Hogne Håskjold (frame)",
 		"-   James Barton (Sangel)",
+		"-   Jason Lutes",
 		"-   Jimmy Olsson (Azlan)",
 		"-   Johanna Manninen (lohari)",
 		"-   John Muccigrosso (Eponymous Archon)",
@@ -95,15 +82,14 @@ std::vector<std::string> get_text() {
 		"-   Peter Geinitz (Shadow)",
 		"-   Richard Kettering (Jetryl)",
 		"-   Slainte",
+		"-   Stephen Stone (Disto)",
 		"-   Svetac",
-		"+ ",
 
 		"_" N_("+Music"),
 		"-   Aleksi",
 		"-   Fredrik Lindroth",
 		"-   Joseph Toscano (zhaymusic.com)",
 		"-   Pau Congost",
-		"+ ",
 
 		"_" N_("+Scenario Designers"),
 		"-   Benjamin Drieu",
@@ -113,12 +99,11 @@ std::vector<std::string> get_text() {
 		"-   James Spencer (Shade)",
 		"-   Joseph Simmons (Turin)",
 		"-   Justin Zaun (jzaun)",
-		"+ ",
 
 		"_" N_("+Multiplayer Maps"),
 		"-   Peter Groen (pg)",
 		"-   Tom Chance (telex4)",
-		"+ ",
+		"-   Mike Quinones (Doc Paterson)",
 
 		"_" N_("+Packagers"),
 		"-   Cyril Bouthors (CyrilB)",
@@ -129,41 +114,41 @@ std::vector<std::string> get_text() {
 		"-   Marcin Konicki (ahwayakchih)",
 		"-   Marcus Phillips (Sithrandel)",
 		"-   Mark Michelsen (skovbaer)",
-		"+ ",
 
 		"_" N_("+Miscellaneous"),
 		"-   Francesco Gigli (Jaramir)",
 		"-   Jordà Polo (ettin)",
-		"+ ",
+		"-   Ruben Philipp Wickenhäuser (The Very Uhu)",
 
 		"_" N_("+Internationalization Managers"),
 		"-   Cédric Duval",
-		"-   Isaac Clerencia",
+		"-   David Philippi (Torangan)",
 		"-   Mark Michelsen (skovbaer)",
+		"-   Nils Kneuper (Ivanovic)",
 		"-   Susanna Björverud (sanna)",
-		"+ ",
+
+		"_" N_("+Afrikaans Translation"),
+		"-   Nico Oliver",
+		"-   ott",
 
 		"_" N_("+Basque Translation"),
 		"-   Alfredo Beaumont (ziberpunk)",
 		"-   Julen Landa (genars)",
 		"-   Mikel Olasagasti (Hey_neken)",
-		"+ ",
 
 		"_" N_("+Bulgarian Translation"),
 		"-   Anton Tsigularov (Atilla)",
-		"+ ",
 
 		"_" N_("+Catalan Translation"),
 		"-   Carles Company (brrr)",
 		"-   Dan Rosàs Garcia (focks)",
+		"-   Jonatan Alamà (tin)",
 		"-   Jordà Polo (ettin)",
 		"-   Mark Recasens",
 		"-   Pau Rul·lan Ferragut",
-		"+ ",
 
 		"_" N_("+Chinese Translation"),
-		"-   Lim Choon Kiat",
-		"+ ",
+		"-   林俊杰 - Lim Choon Kiat",
 
 		"_" N_("+Czech Translation"),
 		"-   David Nečas (Yeti)",
@@ -172,45 +157,41 @@ std::vector<std::string> get_text() {
 		"-   Petr Kovár (Juans)",
 		"-   Sofronius",
 		"-   Vít Krčál",
-		"+ ",
 
 		"_" N_("+Danish Translation"),
 		"-   Mark Michelsen (skovbaer)",
 		"-   Mathias Bundgaard Svensson (freaken)",
-		"+ ",
 
 		"_" N_("+Dutch Translation"),
+		"-   Arne Deprez",
 		"-   Lala",
 		"-   Maarten Albrecht",
 		"-   Pieter Vermeylen (Onne)",
-		"+ ",
+		"-   Tobe Deprez",
 
 		"_" N_("+English (GB) Translation"),
 		"-   ott",
-		"+ ",
-		
+
 		"_" N_("+Estonian Translation"),
 		"-   Mart Tõnso",
-		"+ ",
 
 		"_" N_("+Finnish Translation"),
 		"-   Ankka",
+		"-   Matias Parmala",
 		"-   paxed",
-		"+ ",
 
 		"_" N_("+French Translation"),
 		"-   Benoit Astruc",
 		"-   Cédric Duval",
 		"-   DaringTremayne",
 		"-   Guillaume Duwelz-Rebert",
-		"-   Guillaume Massart",
+		"-   Guillaume Massart (Piou2fois)",
 		"-   Guillaume Melquiond (silene)",
 		"-   Jean-Luc Richard (Le Gnome)",
 		"-   Philippe Plantier (Ayin)",
 		"-   Tout",
 		"-   Yann Dirson",
 		"-   Zas",
-		"+ ",
 
 		"_" N_("+German Translation"),
 		"-   Andre Schmidt (schmidta)",
@@ -222,57 +203,54 @@ std::vector<std::string> get_text() {
 		"-   Nils Kneuper (Ivanovic)",
 		"-   Ruben Philipp Wickenhäuser (The Very Uhu)",
 		"-   Stephan Grochtmann (Schattenstephan)",
-		"+ ",
 
 		"_" N_("+Greek Translation"),
 		"-   Katerina Sykioti",
 		"-   Konstantinos Karasavvas",
-		"+ ",
 
 		"_" N_("+Hungarian Translation"),
 		"-   adson",
 		"-   dentro",
 		"-   Kékkői László (BlackEvil)",
+		"-   Kertész Csaba",
 		"-   Khiraly",
+		"-   Kovács Dániel",
 		"-   krix",
+		"-   ott",
 		"-   Széll Tamás (TomJoad)",
-		"+ ",
 
 		"_" N_("+Italian Translation"),
 		"-   Alessio D'Ascanio (otaku)",
 		"-   Americo Iacovizzi (DarkAmex)",
 		"-   crys0000",
+		"-   Eugenio Favalli (ElvenProgrammer)",
 		"-   Federico Tomassetti",
 		"-   isazi",
 		"-   RokStar",
-		"+ ",
-		
+
 		"_" N_("+Japanese Translation"),
 		"-   Naoki Iimura",
 		"-   Nobuhito Okada",
 		"-   Yuji Matsumoto",
-		"+ ",
 
 		"_" N_("+Latin Translation"),
 		"-   Mark Polo (mpolo)",
-		"+ ",
 
 		"_" N_("+Norwegian Translation"),
 		"-   Hallvard Norheim Bø (Lysander)",
 		"-   Håvard Korsvoll",
 		"-   Erik J. Mesoy (Circon)",
 		"-   Susanne Mesoy (Rarlgland)",
-		"+ ",
 
 		"_" N_("+Polish Translation"),
 		"-   Artur R. Czechowski",
+		"-   Bartek Waresiak (Dragonking)",
 		"-   BOrsuk",
-                "-   Karol Nowak (grzywacz)",
+		"-   Karol Nowak (grzywacz)",
 		"-   methinks",
 		"-   Michał Jedynak (Artanis)",
 		"-   Paweł Stradomski",
 		"-   Paweł Tomak",
-		"+ ",
 
 		"_" N_("+Portuguese (Brazil) Translation"),
 		"-   Ambra Viviani Loos",
@@ -281,21 +259,20 @@ std::vector<std::string> get_text() {
 		"-   Michel Loos",
 		"-   Renato Cunha",
 		"-   Sérgio de Miranda Costa",
-		"+ ",
 
 		"_" N_("+Russian Translation"),
 		"-   Alexandr Menovchicov",
 		"-   Azamat Hackimov",
 		"-   Roman Tuchin (Sankt)",
-		"+ ",
+
+		"_" N_("+Serbian Translation"),
+		"-   Srecko Toroman (FreeCraft)",
 
 		"_" N_("+Slovak Translation"),
 		"-   Viliam Bur",
-		"+ ",
-		
+
 		"_" N_("+Slovenian Translation"),
 		"-   Jaka Kranjc (lynx)",
-		"+ ",
 
 		"_" N_("+Spanish Translation"),
 		"-   David Martínez",
@@ -304,22 +281,22 @@ std::vector<std::string> get_text() {
 		"-   Gabriel Rodríguez (Chewie)",
 		"-   Jose Gordillo (kilder)",
 		"-   Jose Manuel Gomez (joseg)",
-		"+ ",
 
 		"_" N_("+Swedish Translation"),
 		"-   Alexander Kjäll (capitol)",
 		"-   Stefan Bergström (tephlon)",
 		"-   Susanna Björverud (sanna)",
 		"-   wint3r",
-		"+ ",
-		
+
+		"_" N_("+Turkish Translation"),
+		"-   İhsan Akın",
+		"-   Selim Farsakoğlu",
+
 		"_" N_("+Contributors"),
 		"-   Frédéric Wagner",
-		"+ ",
 
 		"_" N_("+Bots"),
 		"-   wesbot",
-		"+ "
 	};
 
 	std::vector< std::string > res;
@@ -359,15 +336,21 @@ void show_about(display &disp)
 
 
 	//substitute in the correct control characters for '+' and '-'
-	for(std::vector<std::string>::iterator itor = text.begin(); itor != text.end(); ++itor) {
-		if(itor->empty() == false) {
-			if((*itor)[0] == '-') {
-				(*itor)[0] = font::SMALL_TEXT;
-			} else if((*itor)[0] == '+') {
-				(*itor)[0] = font::LARGE_TEXT;
-			}
+	std::string before_header(2, ' ');
+	before_header[0] = font::LARGE_TEXT;
+	for(unsigned i = 0; i < text.size(); ++i) {
+		std::string &s = text[i];
+		if (s.empty()) continue;
+		char &first = s[0];
+		if (first == '-')
+			first = font::SMALL_TEXT;
+		else if (first == '+') {
+			first = font::LARGE_TEXT;
+			text.insert(text.begin() + i, before_header);
+			++i;
 		}
 	}
+	text.insert(text.begin(), 10, before_header);
 
 	int startline = 0;
 
