@@ -1,7 +1,7 @@
-/* $Id: actions.cpp,v 1.204 2005/04/09 22:22:19 silene Exp $ */
+/* $Id: actions.cpp,v 1.230 2005/06/12 21:39:49 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -14,6 +14,7 @@
 #include "global.hpp"
 
 #include "actions.hpp"
+#include "checksum.hpp"
 #include "display.hpp"
 #include "events.hpp"
 #include "game_config.hpp"
@@ -38,6 +39,8 @@
 #include "wassert.hpp"
 #include "wml_separators.hpp"
 #include "serialization/string_utils.hpp"
+#include "serialization/binary_wml.hpp"
+#include "serialization/parser.hpp"
 #include "widgets/menu.hpp"
 
 #include <cmath>
@@ -69,15 +72,15 @@ private:
 // check_victory, but shouldn't be passed to that function as parameters,
 // since it is called from a variety of places.
 namespace victory_conditions
-{ 
+{
 	bool when_enemies_defeated = true;
 
-	void set_victory_when_enemies_defeated(bool on) 
+	void set_victory_when_enemies_defeated(bool on)
 	{
 		when_enemies_defeated = on;
 	}
 
-	bool victory_when_enemies_defeated() 
+	bool victory_when_enemies_defeated()
 	{
 		return when_enemies_defeated;
 	}
@@ -93,7 +96,7 @@ std::string recruit_unit(const gamemap& map, int side,
 	typedef std::map<gamemap::location,unit> units_map;
 
 	//find the unit that can recruit
-	units_map::const_iterator u;
+	units_map::const_iterator u = units.end();
 
 	for(u = units.begin(); u != units.end(); ++u) {
 		if(u->second.can_recruit() && u->second.side() == side) {
@@ -101,25 +104,31 @@ std::string recruit_unit(const gamemap& map, int side,
 		}
 	}
 
-	if(u == units.end())
+	if(u == units.end() && (need_castle || !map.on_board(recruit_location))) {
 		return _("You don't have a leader to recruit with.");
+	}
 
-	if(map.is_keep(u->first) == false) {
-		LOG_NG << "Leader not on start: leader is on " << (u->first.x+1) << "," << (u->first.y+1) << "\n";
+	wassert(u != units.end() || !need_castle);
+
+	if(need_castle && map.is_keep(u->first) == false) {
+		LOG_NG << "Leader not on start: leader is on " << u->first << '\n';
 		return _("You must have your leader on a keep to recruit or recall units.");
 	}
 
 	if(need_castle && map.on_board(recruit_location)) {
 		castle_cost_calculator calc(map);
-		const paths::route& rt = a_star_search(u->first, recruit_location, 100.0, &calc, map.x(), map.y());		
+		const paths::route& rt = a_star_search(u->first, recruit_location, 100.0, &calc, map.x(), map.y());
 		if(rt.steps.empty() || units.find(recruit_location) != units.end() ||
-		   !map.is_castle(recruit_location))
+		   !map.is_castle(recruit_location)) {
 			recruit_location = gamemap::location();
+		}
 	}
 
 	if(!map.on_board(recruit_location)) {
 		recruit_location = find_vacant_tile(map,units,u->first,
 		                                    need_castle ? VACANT_CASTLE : VACANT_ANY);
+	} else if(units.count(recruit_location) == 1) {
+		recruit_location = find_vacant_tile(map,units,recruit_location,VACANT_ANY);
 	}
 
 	if(!map.on_board(recruit_location)) {
@@ -155,9 +164,41 @@ std::string recruit_unit(const gamemap& map, int side,
 		}
 	}
 
+	checksumstream cs;
+	config cfg_unit;
+	new_unit.write(cfg_unit);
+	::write_compressed(cs,cfg_unit);
+
+	const config* ran_results = get_random_results();
+	if(ran_results != NULL) {
+		unsigned long rc = lexical_cast_default<unsigned long>
+			((*ran_results)["checksum"], 0);
+		if((*ran_results)["checksum"].empty() || rc != cs.checksum()) {
+			ERR_NW << "SYNC: In recruit " << new_unit.type().id() <<
+				": has checksum " << cs.checksum() <<
+				" while datasource has checksum " <<
+				rc << "\n";
+
+			::write(std::cerr, cfg_unit);
+			// FIXME: this was not playtested, so I will disable it
+			// for release.
+			//if (!game_config::ignore_replay_errors) throw replay::error();
+		}
+
+	} else {
+		config cfg;
+		cfg["checksum"] = lexical_cast<std::string>(cs.checksum());
+		set_random_results(cfg);
+	}
+
 	return std::string();
 }
 
+void validate_recruit_unit()
+{
+
+}
+
 gamemap::location under_leadership(const std::map<gamemap::location,unit>& units,
                                    const gamemap::location& loc, int* bonus)
 {
@@ -199,6 +240,55 @@ static int compute_damage(int base_damage, int bonus, int divisor)
 	return maximum<int>(1, (base_damage * bonus + rounding) / divisor);
 }
 
+double pr_atleast(int m, double p, int n, int d)
+{
+	// calculate Pr[A does damage in [m,...)], where
+	// p probability to hit, n swings, d damage/swing
+	double P = 0.0;
+	// 0 damage can happen when unit has no attack of this type
+	if(d == 0)
+		return (m <= 0) ? 1.0 : P;
+	for(int k = (m + d - 1)/d; k <= n; ++k) {
+		double r = 1.0;
+		const int k2 = (k > n - k) ? (n - k) : k;
+		for(int i = 0; i < k2; ++i) { r *= (n-i); r /= (k2-i); }
+		P += r * pow(p, k) * pow(1-p, n-k);
+	}
+	return P;
+}
+
+double pr_between(int mn, int mx, double p, int n, int d)
+{
+	// calculate Pr[A does damage in [mn,mx)], where
+	// p probability to hit, n swings, d damage/swing
+	return pr_atleast(mn, p, n, d) - pr_atleast(mx, p, n, d);
+}
+
+int reduce(int i, int u, int v)
+{
+	// i-th swingpair, but reduce since u < v
+	if(i == 0)
+	    return i;
+	else
+	    return ((i-1) / v)*u + minimum<int>(u, ((i-1) % v) + 1);
+}
+
+double pr_kills_during(const int hpa, const int dmga, const double pa,
+	const int swa, const int hpb, const int dmgb, const double pb,
+	const int swb, const int n, const bool second)
+{
+	if ((swa < swb) && (swa < (n-1) % swb + 1)) // not our move
+		return 0.0;
+	// A kills B during swing n, and is it second?
+	// take into account where one unit doesn't get all n swings
+	const double t1 = pr_between(hpb - dmga, hpb, pa,
+		(swa<swb) ? reduce(n-1, swa, swb) : (n-1), dmga);
+	const int n2 = second ? n : (n - 1);
+	const double t2 = 1.0 - pr_atleast(hpa, pb,
+		(swa>swb) ? reduce(n2, swb, swa) : n2, dmgb);
+	return t1 * pa * t2;
+}
+
 battle_stats evaluate_battle_stats(const gamemap& map,
                                    const gamemap::location& attacker,
                                    const gamemap::location& defender,
@@ -265,9 +355,9 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	}
 
 	static const std::string plague_string("plague");
-	res.attacker_plague = !d->second.type().not_living() && 
-	  (res.attacker_special.substr(0,6) == plague_string) && 
-	  strcmp(d->second.type().undead_variation().c_str(),"null") && 
+	res.attacker_plague = !d->second.type().not_living() &&
+	  (res.attacker_special.substr(0,6) == plague_string) &&
+	  strcmp(d->second.type().undead_variation().c_str(),"null") &&
 	  !map.is_village(defender);
 	if(res.attacker_special.size()>8){ //plague(type) used
 	  res.attacker_plague_type=res.attacker_special.substr(7,res.attacker_special.size()-8);
@@ -322,6 +412,10 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 		const attack_type& defend = defender_attacks[defend_with];
 		res.defender_special = defend.special();
 
+		if(res.defender_special == to_the_death_string) {
+			res.to_the_death = true;
+		}
+
 		//magical attacks always have a 70% chance to hit
 		if (res.defender_special == magical_string) {
 			res.chance_to_hit_attacker = 70;
@@ -332,7 +426,7 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 
 		const int base_damage = defend.damage();
 		const int resistance_modifier = a->second.damage_against(defend);
-		
+
 		if (strings) {
 			std::stringstream str_base;
 			str_base << _("base damage") << COLUMN_SEPARATOR << base_damage;
@@ -410,7 +504,7 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 			res.amount_defender_drains = 0;
 		}
 
-		res.defender_plague = !a->second.type().not_living() && 
+		res.defender_plague = !a->second.type().not_living() &&
 		  (res.defender_special.substr(0,6) == plague_string) &&
 		  strcmp(a->second.type().undead_variation().c_str(),"null") &&
 		  !map.is_village(attacker);
@@ -439,7 +533,16 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	int divisor = 100;
 
 	const int base_damage = attack.damage();
-	const int resistance_modifier = d->second.damage_against(attack);
+	int resistance_modifier = d->second.damage_against(attack);
+
+	//steadfast doubles resistance, but is capped at increasing resistance to 50%
+	if (steadfast && resistance_modifier < 100 && resistance_modifier > 50) {
+		const int diff = 100 - resistance_modifier;
+		resistance_modifier -= diff;
+		if(resistance_modifier < 50) {
+			resistance_modifier = 50;
+		}
+	}
 
 	if (strings) {
 		std::stringstream str_base;
@@ -461,7 +564,7 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	int leader_bonus = 0;
 	if (under_leadership(units,attacker,&leader_bonus).valid()) {
 		bonus += leader_bonus;
-		
+
 		if (strings) {
 			std::stringstream str;
 			str << _("leadership") << EMPTY_COLUMN << '+' << leader_bonus << '%';
@@ -487,22 +590,19 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 		}
 	}
 
-	if (steadfast) {
-		divisor *= 2;
-		if (strings) {
-			std::stringstream str;
-			str << _("steadfast") << EMPTY_COLUMN << _("Halved");
-			strings->attack_calculations.push_back(str.str());
-		}
-	}
-
 	if (strings && resistance_modifier != 100) {
 		const int resist = resistance_modifier - 100;
 		std::stringstream str_resist;
 		str_resist << gettext(resist < 0 ? N_("defender resistance vs") : N_("defender vulnerability vs"))
-		           << ' ' << gettext(attack.type().c_str()) << EMPTY_COLUMN
+		           << ' ' << gettext(attack.type().c_str());
+		if(steadfast && resistance_modifier < 100) {
+			str_resist << ' ' << _(" (+steadfast)");
+		}
+		
+		str_resist << EMPTY_COLUMN
 		           << (resist > 0 ? "+" : "") << resist << '%';
 		strings->attack_calculations.push_back(str_resist.str());
+
 	}
 
 	bonus *= resistance_modifier;
@@ -534,6 +634,40 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	if(d->second.has_flag(slowed_string) && res.ndefends > 1)
 		--res.ndefends;
 
+	// FIXME: doesn't take into account berserk+slow or drain
+	if (strings && res.amount_attacker_drains == 0 &&
+		res.amount_defender_drains == 0 &&
+		!(res.to_the_death &&
+			(res.attacker_slows || res.defender_slows)))
+	{
+		const int maxrounds = (res.to_the_death ? 30 : 1);
+		const int hpa = a->second.hitpoints();
+		const int hpb = d->second.hitpoints();
+		const int dmga = res.damage_defender_takes;
+		const int dmgb = res.damage_attacker_takes;
+		const double pa = res.chance_to_hit_defender/100.0;
+		const double pb = res.chance_to_hit_attacker/100.0;
+		const int swa = res.nattacks;
+		const int swb = res.ndefends;
+		double P1 = 0;
+
+		for(int n = 1; n <= maxrounds*maximum<int>(swa,swb); ++n)
+			P1 += pr_kills_during(hpa, dmga, pa, swa,
+			hpb, dmgb, pb, swb, n, res.defender_strikes_first);
+		const double P3 = (1.0 - pr_atleast(hpb,pa,maxrounds*swa,dmga))
+			* (1.0 - pr_atleast(hpa,pb,maxrounds*swb,dmgb));
+		std::stringstream str;
+		if (P3 > 0.99) {
+			str << _("(both should survive)") << EMPTY_COLUMN;
+		} else {
+			str << _("% Pr[kills/killed by/both survive]")
+			    << EMPTY_COLUMN << (int)(P1*100+0.5)
+			    << '/' << (int)((1-P1-P3)*100+0.5)
+			    << '/' << (int)(P3*100+0.5);
+		}
+		strings->attack_calculations.push_back(str.str());
+	}
+
 	return res;
 }
 
@@ -544,7 +678,7 @@ static std::string unit_dump(std::pair< gamemap::location, unit > const &u)
 	return s.str();
 }
 
-void attack(display& gui, const gamemap& map, 
+void attack(display& gui, const gamemap& map,
             std::vector<team>& teams,
             gamemap::location attacker,
             gamemap::location defender,
@@ -580,8 +714,8 @@ void attack(display& gui, const gamemap& map,
 
 	statistics::attack_context attack_stats(a->second,d->second,stats);
 
-	const int orig_attacks = stats.nattacks;
-	const int orig_defends = stats.ndefends;
+	int orig_attacks = stats.nattacks;
+	int orig_defends = stats.ndefends;
 	int to_the_death = stats.to_the_death ? 30 : 0;
 
 	static const std::string poison_string("poison");
@@ -590,6 +724,7 @@ void attack(display& gui, const gamemap& map,
 		LOG_NG << "start of attack loop...\n";
 
 		if(stats.nattacks > 0 && stats.defender_strikes_first == false) {
+			add_random_separator();
 			const int ran_num = get_random();
 			bool hits = (ran_num%100) < stats.chance_to_hit_defender;
 
@@ -696,15 +831,17 @@ void attack(display& gui, const gamemap& map,
 
 				//plague units make new units on the target hex
 				if(stats.attacker_plague) {
-				        a = units.find(attacker_loc); 
+				        a = units.find(attacker_loc);
 				        game_data::unit_type_map::const_iterator reanimitor;
 				        LOG_NG<<"trying to reanimate "<<stats.attacker_plague_type<<std::endl;
 				        reanimitor = info.unit_types.find(stats.attacker_plague_type);
 				        LOG_NG<<"found unit type:"<<reanimitor->second.id()<<std::endl;
-					
+
 					if(reanimitor != info.unit_types.end()) {
-					       unit newunit=unit(&reanimitor->second,a->second.side(),false,true);
-					       //apply variation					       
+					       unit newunit=unit(&reanimitor->second,a->second.side(),true,true);
+					       newunit.set_attacked();
+
+					       //apply variation
 					       if(strcmp(undead_variation.c_str(),"null")){
 						 config mod;
 						 config& variation=mod.add_child("effect");
@@ -734,8 +871,10 @@ void attack(display& gui, const gamemap& map,
 				if(stats.attacker_slows && d->second.has_flag("slowed") == false) {
 					gui.float_label(d->first,_("slowed"),255,0,0);
 					d->second.set_flag("slowed");
-					if(stats.ndefends > 1)
-						--stats.ndefends;
+					if (orig_defends > 1) {
+						if (stats.ndefends > 0) --stats.ndefends;
+						--orig_defends;
+					}
 				}
 
 				//if the defender is turned to stone, the fight stops immediately
@@ -757,6 +896,7 @@ void attack(display& gui, const gamemap& map,
 
 		if(stats.ndefends > 0) {
 			LOG_NG << "doing defender attack...\n";
+			add_random_separator();
 
 			const int ran_num = get_random();
 			bool hits = (ran_num%100) < stats.chance_to_hit_attacker;
@@ -833,7 +973,7 @@ void attack(display& gui, const gamemap& map,
 					d->second.heal(stats.amount_defender_drains);
 				}
 			}
-				
+
 			if(dies) {//defender kills attacker
 				defenderxp = game_config::kill_experience*a->second.type().level();
 				if(a->second.type().level() == 0)
@@ -859,14 +999,14 @@ void attack(display& gui, const gamemap& map,
 
 				//plague units make new units on the target hex.
 				if(stats.defender_plague) {
-				        d = units.find(defender_loc); 
+				        d = units.find(defender_loc);
 				        game_data::unit_type_map::const_iterator reanimitor;
 				        LOG_NG<<"trying to reanimate "<<stats.defender_plague_type<<std::endl;
 				        reanimitor = info.unit_types.find(stats.defender_plague_type);
 				        LOG_NG<<"found unit type:"<<reanimitor->second.id()<<std::endl;
-					
+
 					if(reanimitor != info.unit_types.end()) {
-					       unit newunit=unit(&reanimitor->second,d->second.side(),false,true);
+					       unit newunit=unit(&reanimitor->second,d->second.side(),true,true);
 					       //apply variation
 					       if(strcmp(undead_variation.c_str(),"null")){
 						 config mod;
@@ -897,8 +1037,10 @@ void attack(display& gui, const gamemap& map,
 				if(stats.defender_slows && a->second.has_flag("slowed") == false) {
 					gui.float_label(a->first,_("slowed"),255,0,0);
 					a->second.set_flag("slowed");
-					if(stats.nattacks > 1)
-						--stats.nattacks;
+					if (orig_attacks > 1) {
+						if (stats.nattacks > 0) --stats.nattacks;
+						--orig_attacks;
+					}
 				}
 
 
@@ -916,6 +1058,8 @@ void attack(display& gui, const gamemap& map,
 			--stats.ndefends;
 		}
 
+		// continue the fight to death; if one of the units got stoned,
+		// either nattacks or ndefends is -1
 		if(to_the_death > 0 && stats.ndefends == 0 && stats.nattacks == 0) {
 			stats.nattacks = orig_attacks;
 			stats.ndefends = orig_defends;
@@ -947,7 +1091,6 @@ int village_owner(const gamemap::location& loc, const std::vector<team>& teams)
 	return -1;
 }
 
-
 void get_village(const gamemap::location& loc, std::vector<team>& teams,
                  size_t team_num, const unit_map& units)
 {
@@ -1161,7 +1304,7 @@ void calculate_healing(display& disp, const gamestatus& status, const gamemap& m
 
 	for(i = units.begin(); i != units.end(); ++i) {
 		if(i->second.side() == side) {
-			if(i->second.hitpoints() < i->second.max_hitpoints() || 
+			if(i->second.hitpoints() < i->second.max_hitpoints() ||
 					i->second.poisoned()){
 				if(i->second.is_resting()) {
 					const std::map<gamemap::location,int>::iterator u =
@@ -1213,8 +1356,14 @@ void calculate_healing(display& disp, const gamestatus& status, const gamemap& m
 					healer.set_healing(true);
 
 					if(halo_image.empty() == false) {
-						halo::add(disp.get_location_x(i->second)+disp.hex_size()/2,disp.get_location_y(i->second)+disp.hex_size()/2,
-								halo_image,healer.facing_left() ? halo::NORMAL : halo::REVERSE,1);
+						int height_adjust = healer.is_flying() ? 0 :
+							int(map.get_terrain_info(map.get_terrain(i->second)).
+								unit_height_adjust() * disp.zoom());
+						int d = disp.hex_size();
+						halo::add(disp.get_location_x(i->second) + d / 2,
+						          disp.get_location_y(i->second) + d / 2 - height_adjust,
+						          halo_image, healer.facing_left() ? halo::NORMAL :
+						                                             halo::REVERSE, 1);
 					}
 
 					disp.draw_tile(i->second.x,i->second.y);
@@ -1253,7 +1402,7 @@ void calculate_healing(display& disp, const gamestatus& status, const gamemap& m
 				if(h->second <= 0)
 					continue;
 			}
-			
+
 			if(h->second < 0) {
 				if(show_healing) {
 					sound::play_sound("groan.wav");
@@ -1314,7 +1463,7 @@ void calculate_healing(display& disp, const gamestatus& status, const gamemap& m
 
 				disp.draw_tile(i->second.x,i->second.y);
 			}
-			
+
 			disp.draw_tile(loc.x,loc.y);
 			disp.update_display();
 		}
@@ -1486,7 +1635,7 @@ bool clear_shroud_loc(const gamemap& map, team& tm,
 //returns true iff some shroud is cleared
 //seen_units will return new units that have been seen by this unit
 //if known_units is NULL, seen_units can be NULL and will not be changed
-bool clear_shroud_unit(const gamemap& map, 
+bool clear_shroud_unit(const gamemap& map,
 		                 const gamestatus& status,
 							  const game_data& gamedata,
                        const unit_map& units, const gamemap::location& loc,
@@ -1535,7 +1684,7 @@ bool clear_shroud_unit(const gamemap& map,
 }
 
 void recalculate_fog(const gamemap& map, const gamestatus& status,
-		const game_data& gamedata, const unit_map& units, 
+		const game_data& gamedata, const unit_map& units,
 		std::vector<team>& teams, int team) {
 
 	teams[team].refog();
@@ -1553,7 +1702,7 @@ void recalculate_fog(const gamemap& map, const gamestatus& status,
 	}
 }
 
-bool clear_shroud(display& disp, const gamestatus& status, 
+bool clear_shroud(display& disp, const gamestatus& status,
 		            const gamemap& map, const game_data& gamedata,
                   const unit_map& units, std::vector<team>& teams, int team)
 {
@@ -1582,7 +1731,7 @@ bool clear_shroud(display& disp, const gamestatus& status,
 	return result;
 }
 
-size_t move_unit(display* disp, const game_data& gamedata, 
+size_t move_unit(display* disp, const game_data& gamedata,
                  const gamestatus& status, const gamemap& map,
                  unit_map& units, std::vector<team>& teams,
                  std::vector<gamemap::location> route,
@@ -1609,7 +1758,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	team& team = teams[team_num];
 	const bool check_shroud = should_clear_shroud && team.auto_shroud_updates() &&
 		(team.uses_shroud() || team.uses_fog());
-	
+
 	//if we use shroud/fog of war, count out the units we can currently see
 	std::set<gamemap::location> known_units;
 	if(check_shroud) {
@@ -1631,7 +1780,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		const gamemap::TERRAIN terrain = map[step->x][step->y];
 
 		const unit_map::const_iterator enemy_unit = units.find(*step);
-			
+
 		const int mv = u.movement_cost(map,terrain);
 		if(discovered_unit || continue_move == false && seen_units.empty() == false ||
 		   mv > moves_left || enemy_unit != units.end() && team.is_enemy(enemy_unit->second.side())) {
@@ -1714,18 +1863,26 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	} else {
 		u.set_interrupted_move(gamemap::location());
 	}
-	
+
 	if(steps.size() < 2) {
 		return 0;
 	}
 
-	units.erase(ui);
+	//move the unit on the screen. Hide the unit in its current location, but don't actually
+	//remove it until the move is done, so that while the unit is moving status etc will
+	//still display the correct number of units.
 	if(disp != NULL) {
+		disp->hide_unit(ui->first,true);
 		unit_display::move_unit(*disp,map,steps,u,status.get_time_of_day(),units,teams);
 	}
 
 	u.set_movement(moves_left);
 
+	if(disp != NULL) {
+		disp->hide_unit(gamemap::location());
+	}
+
+	units.erase(ui);
 	ui = units.insert(std::pair<gamemap::location,unit>(steps.back(),u)).first;
 	if(disp != NULL) {
 		disp->invalidate_unit();
@@ -1775,7 +1932,13 @@ size_t move_unit(display* disp, const game_data& gamedata,
 			for(std::set<gamemap::location>::const_iterator i = seen_units.begin(); i != seen_units.end(); ++i) {
 				LOG_NG << "processing unit at " << (i->x+1) << "," << (i->y+1) << "\n";
 				const unit_map::const_iterator u = units.find(*i);
-				wassert(u != units.end());
+
+				//unit may have been removed by an event.
+				if(u == units.end()) {
+					LOG_NG << "was removed\n";
+					continue;
+				}
+
 				if(team.is_enemy(u->second.side())) {
 					++nenemies;
 				} else {
@@ -1813,7 +1976,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 
 			std::stringstream msg;
 			msg << gettext(msg_id);
-			
+
 			if(steps.size() < route.size()) {
 				//see if the "Continue Move" action has an associated hotkey
 				const hotkey::hotkey_item& hk = hotkey::get_hotkey(hotkey::HOTKEY_CONTINUE_MOVE);
@@ -1833,10 +1996,6 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		disp->recalculate_minimap();
 	}
 
-	for(std::set<gamemap::location>::const_iterator seen = seen_units.begin(); seen != seen_units.end(); ++seen) {
-		game_events::fire("sighted",*seen,steps.back());
-	}
-
 	wassert(steps.size() <= route.size());
 
 	return steps.size();
@@ -1847,7 +2006,7 @@ bool unit_can_move(const gamemap::location& loc, const unit_map& units,
 {
 	const unit_map::const_iterator u_it = units.find(loc);
 	wassert(u_it != units.end());
-	
+
 	const unit& u = u_it->second;
 	const team& current_team = teams[u.side()-1];
 
@@ -1870,7 +2029,7 @@ bool unit_can_move(const gamemap::location& loc, const unit_map& units,
 					return true;
 				}
 			}
-			
+
 			if(u.movement_cost(map,map[locs[n].x][locs[n].y]) <= u.movement_left()) {
 				return true;
 			}
@@ -1900,7 +2059,7 @@ void apply_shroud_changes(undo_list& undos, display* disp, const gamestatus& sta
 		//we're not really going to mutate the unit, just temporarily
 		//set its moves to maximum, but then switch them back
 		const unit_movement_resetter move_resetter(const_cast<unit&>(un->affected_unit));
-		
+
 		std::vector<gamemap::location>::const_iterator step;
 		for(step = un->route.begin(); step != un->route.end(); ++step) {
 			//we have to swap out any unit that is already in the hex, so we can put our
