@@ -1,7 +1,7 @@
-/* $Id: ai.cpp,v 1.146 2005/03/28 19:08:58 gruikya Exp $ */
+/* $Id: ai.cpp,v 1.154 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -35,6 +35,8 @@
 #include <iostream>
 
 #define LOG_AI LOG_STREAM(info, ai)
+#define WRN_AI LOG_STREAM(warn, ai)
+#define ERR_AI LOG_STREAM(err, ai)
 
 ///a trivial ai that sits around doing absolutely nothing
 class idle_ai : public ai_interface {
@@ -103,7 +105,7 @@ protected:
 	int choose_weapon(const location& attacker, const location& defender) {
 		const unit_map::const_iterator att = get_info().units.find(attacker);
         wassert(att != get_info().units.end());
-        
+
         const std::vector<attack_type>& attacks = att->second.attacks();
 
         int best_attack_rating = -1;
@@ -212,7 +214,7 @@ bool ai::recruit_usage(const std::string& usage)
 	user_interact();
 
 	const int min_gold = 0;
-	
+
 	log_scope2(ai, "recruiting troops");
 	LOG_AI << "recruiting " << usage << "\n";
 
@@ -273,6 +275,12 @@ bool ai_interface::recruit(const std::string& unit_name, location loc)
 	if(current_team().gold() < u->second.cost()) {
 		return false;
 	}
+	LOG_AI << "trying recruit: team=" << (info_.team_num) <<
+	    " type=" << unit_name <<
+	    " cost=" << u->second.cost() <<
+	    " loc=(" << loc << ')' <<
+	    " gold=" << (current_team().gold()) <<
+	    " (-> " << (current_team().gold()-u->second.cost()) << ")\n";
 
 	unit new_unit(&u->second,info_.team_num,true);
 
@@ -285,8 +293,22 @@ bool ai_interface::recruit(const std::string& unit_name, location loc)
 		replay_guard.confirm_transaction();
 
 		sync_network();
+		const team_data data = calculate_team_data(current_team(),info_.team_num,info_.units);
+		LOG_AI <<
+		"recruit confirmed: team=" << (info_.team_num) <<
+		" units=" << data.units <<
+		" gold=" << data.gold <<
+		((data.net_income < 0) ? "" : "+") <<
+		data.net_income << "\n";
 		return true;
 	} else {
+		const team_data data = calculate_team_data(current_team(),info_.team_num,info_.units);
+		LOG_AI <<
+		"recruit UNconfirmed: team=" << (info_.team_num) <<
+		" units=" << data.units <<
+		" gold=" << data.gold <<
+		((data.net_income < 0) ? "" : "+") <<
+		data.net_income << "\n";
 		return false;
 	}
 }
@@ -361,8 +383,7 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 
 gamemap::location ai_interface::move_unit_partial(location from, location to, std::map<location,paths>& possible_moves)
 {
-	LOG_AI << "ai_interface::move_unit " << (from.x + 1) << "," << (from.y + 1)
-		<< " -> " << (to.x + 1) << "," << (to.y + 1) << "\n";
+	LOG_AI << "ai_interface::move_unit " << from << " -> " << to << '\n';
 	//stop the user from issuing any commands while the unit is moving
 	const command_disabler disable_commands;
 
@@ -374,13 +395,13 @@ gamemap::location ai_interface::move_unit_partial(location from, location to, st
 	log_scope2(ai, "move_unit");
 	unit_map::iterator u_it = info_.units.find(from);
 	if(u_it == info_.units.end()) {
-		LOG_STREAM(err, ai) << "Could not find unit at " << from.x << ", " << from.y << "\n";
+		LOG_STREAM(err, ai) << "Could not find unit at " << from << '\n';
 		wassert(false);
 		return location();
 	}
 
 	if(from == to) {
-		LOG_AI << "moving unit at " << (from.x+1) << "," << (from.y+1) << " on spot. resetting moves\n";
+		LOG_AI << "moving unit at " << from << " on spot. resetting moves\n";
 		return to;
 	}
 
@@ -445,9 +466,12 @@ gamemap::location ai_interface::move_unit_partial(location from, location to, st
 				}
 
 				info_.disp.scroll_to_tiles(from.x,from.y,to.x,to.y);
+
+				info_.disp.hide_unit(u_it->first,true);
+				unit_display::move_unit(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams);
+				info_.disp.hide_unit(gamemap::location());
 				info_.units.erase(u_it);
 				u_it = info_.units.end();
-				unit_display::move_unit(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams);
 			}
 		}
 	}
@@ -485,8 +509,8 @@ bool ai::multistep_move_possible(location from, location to, location via, std::
 	if(i != units_.end()) {
 		if(from != via && to != via && units_.count(via) == 0) {
 			LOG_AI << "when seeing if leader can move from "
-				<< (from.x + 1) << "," << (from.y + 1) << " -> " << (to.x + 1) << "," << (to.y + 1)
-				<< " seeing if can detour to keep at " << (via.x + 1) << "," << (via.y + 1) << "\n";
+				<< from << " -> " << to
+				<< " seeing if can detour to keep at " << via << '\n';
 			const std::map<location,paths>::const_iterator moves = possible_moves.find(from);
 			if(moves != possible_moves.end()) {
 
@@ -506,7 +530,7 @@ bool ai::multistep_move_possible(location from, location to, location via, std::
 					const paths unit_paths(map_,state_,gameinfo_,units_,via,teams_,false,false);
 
 					LOG_AI << "found " << unit_paths.routes.size() << " moves for temp leader\n";
-					
+
 					//see if this leader could make it back to the keep
 					if(unit_paths.routes.count(to) != 0) {
 						LOG_AI << "can make it back to the keep\n";
@@ -585,7 +609,7 @@ bool ai::attack_close(const gamemap::location& loc) const
 			return true;
 		}
 	}
-	
+
 	return false;
 }
 
@@ -613,7 +637,7 @@ void ai_interface::calculate_possible_moves(std::map<location,paths>& res, move_
 		}
 
 		//we can't see where invisible enemy units might move
-		if(enemy && un_it->second.invisible(info_.map.underlying_terrain(info_.map.get_terrain(un_it->first)), 
+		if(enemy && un_it->second.invisible(info_.map.underlying_terrain(info_.map.get_terrain(un_it->first)),
 		   info_.state.get_time_of_day().lawful_bonus,un_it->first,info_.units,info_.teams)) {
 			continue;
 		}
@@ -856,9 +880,6 @@ void ai::do_move()
 			move_leader_after_recruit(enemy_dstsrc);
 		}
 	}
-
-	recorder.end_turn();
-	sync_network();
 }
 
 bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc)
@@ -912,9 +933,8 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 		const location arrived_at = move_unit(from,to,possible_moves);
 		if(arrived_at != to) {
 			LOG_STREAM(warn, ai) << "unit moving to attack has ended up unexpectedly at "
-				<< (arrived_at.x + 1) << "," << (arrived_at.y + 1) << " when moving to "
-				<< (to.x + 1) << "," << (to.y + 1) << " moved from "
-				<< (from.x + 1) << "," << (from.y + 1) << "\n";
+			                     << arrived_at << " when moving to " << to << " moved from "
+			                     << from << '\n';
 			return true;
 		}
 
@@ -1039,7 +1059,7 @@ bool ai::get_villages(std::map<gamemap::location,paths>& possible_moves, const m
 			if(owned && !current_team().is_enemy(n+1)) {
 				want_village = false;
 			}
-			
+
 			if(owned) {
 				break;
 			}
@@ -1156,10 +1176,10 @@ bool ai::get_healing(std::map<gamemap::location,paths>& possible_moves, const mo
 					if(vuln < best_vulnerability || best_loc == it.second) {
 						best_vulnerability = vuln;
 						best_loc = it.first;
-						LOG_AI << "chose village " << (dst.x+1) << "," << (dst.y+1) << "\n";
+						LOG_AI << "chose village " << dst << '\n';
 					}
 				}
-				
+
 				++it.first;
 			}
 
@@ -1269,8 +1289,8 @@ bool ai::retreat_units(std::map<gamemap::location,paths>& possible_moves, const
 				}
 
 				if(best_pos.valid()) {
-					LOG_AI << "retreating '" << i->second.type().id() << "' " << i->first.x << ","
-						<< i->first.y << " -> " << best_pos.x << "," << best_pos.y << "\n";
+					LOG_AI << "retreating '" << i->second.type().id() << "' " << i->first
+					       << " -> " << best_pos << '\n';
 					move_unit(i->first,best_pos,possible_moves);
 					return true;
 				}
@@ -1309,7 +1329,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 			return true;
 		}
 
-		LOG_AI << "move: " << move.first.x << ", " << move.first.y << " - " << move.second.x << ", " << move.second.y << "\n";
+		LOG_AI << "move: " << move.first << " -> " << move.second << '\n';
 
 		//search to see if there are any enemy units next
 		//to the tile which really should be attacked once the move is done.
@@ -1338,7 +1358,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 				}
 			}
 		}
-		
+
 
 		const location arrived_at = move_unit(move.first,move.second,possible_moves);
 
@@ -1353,7 +1373,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 
 		//if we're going to attack someone
 		if(u_it != units_.end() && u_it->second.stone() == false && weapon != -1) {
-			attack_enemy(move.second,target,weapon);			
+			attack_enemy(move.second,target,weapon);
 		}
 
 		//don't allow any other units to move onto the tile our unit
@@ -1516,7 +1536,7 @@ void ai::analyze_potential_recruit_movements()
 	LOG_AI << "targets: " << targets.size() << "\n";
 
 	std::map<std::string,int> best_scores;
-	
+
 	for(std::set<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
 		const game_data::unit_type_map::const_iterator info = gameinfo_.unit_types.find(*i);
 		if(info == gameinfo_.unit_types.end()) {
@@ -1670,15 +1690,19 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 	}
 
 	const gamemap::location dst(*goal);
+	if (!dst.valid()) {
+		ERR_AI << "Invalid goal\n";
+		return;
+	}
 
 	const unit_map::iterator leader = find_leader(units_,team_num_);
 	if(leader == units_.end() || leader->second.incapacitated()) {
-		LOG_AI << "leader not found\n";
+		WRN_AI << "Leader not found\n";
 		return;
 	}
 
 	LOG_AI << "Doing recruitment before goals\n";
-	
+
 	do_recruitment();
 
 	shortest_path_calculator calc(leader->second, current_team(), units_, teams_, map_, state_);
@@ -1692,7 +1716,7 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 
 	std::map<gamemap::location,paths> possible_moves;
 	possible_moves.insert(std::pair<gamemap::location,paths>(leader->first,leader_paths));
-	
+
 	gamemap::location loc;
 	for(std::vector<gamemap::location>::const_iterator itor = route.steps.begin(); itor != route.steps.end(); ++itor) {
 		if(leader_paths.routes.count(*itor) == 1 && power_projection(*itor,enemy_srcdst,enemy_dstsrc) < double(leader->second.hitpoints()/2)) {
@@ -1775,7 +1799,7 @@ void ai::move_leader_after_recruit(const move_map& enemy_dstsrc)
 			//which we might be able to reach in two turns.
 			if(map_.is_village(i->first) && current_team().owns_village(i->first) &&
 				int(distance_between(i->first,leader->first)) <= leader->second.total_movement()*2) {
-				
+
 				int current_distance = distance_between(i->first,leader->first);
 				location current_loc;
 
@@ -1861,7 +1885,7 @@ bool ai::leader_can_reach_keep() const
 
 	//find where the leader can move
 	const paths leader_paths(map_,state_,gameinfo_,units_,leader->first,teams_,false,false);
-	
+
 
 	return leader_paths.routes.count(start_pos) > 0;
 }
