@@ -1,7 +1,7 @@
-/* $Id: config.hpp,v 1.60 2005/03/27 23:06:16 gruikya Exp $ */
+/* $Id: config.hpp,v 1.63 2005/06/11 02:41:37 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -124,6 +124,13 @@ public:
 	//latter config object will clobber attributes in this one.
 	void append(const config& cfg);
 
+	//all children with the given key will be merged into the first element
+	//with that key
+	void merge_children(const std::string& key);
+
+	//resets the translated values of all strings contained in this object
+	void reset_translation() const;
+
 	//all the attributes of this node.
 	string_map values;
 
