@@ -1,7 +1,7 @@
-/* $Id: dialogs.cpp,v 1.98 2005/04/09 09:46:12 gruikya Exp $ */
+/* $Id: dialogs.cpp,v 1.105 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -70,6 +70,7 @@ void advance_unit(const game_data& info,
 		sample_units.push_back(::get_advanced_unit(info,units,loc,*op));
 		const unit_type& type = sample_units.back().type();
 		lang_options.push_back(IMAGE_PREFIX + type.image() + COLUMN_SEPARATOR + type.language_name());
+		preferences::encountered_units().insert(*op);
 	}
 
 	const config::child_list& mod_options = u->second.get_modification_advances();
@@ -110,7 +111,7 @@ void advance_unit(const game_data& info,
 bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::location loc, display& gui, size_t choice)
 {
 	const command_disabler cmd_disabler;
-	
+
 	std::map<gamemap::location,unit>::iterator u = units.find(loc);
 	if(u == units.end() || u->second.advances() == false) {
 		return false;
@@ -122,10 +123,10 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 	if(choice >= options.size() + mod_options.size()) {
 		return false;
 	}
-	
+
 	//when the unit advances, it fades to white, and then switches to the
 	//new unit, then fades back to the normal colour
-	
+
 	if(!gui.update_locked()) {
 		for(double intensity = 1.0; intensity >= 0.0; intensity -= 0.05) {
 			gui.set_advancing_unit(loc,intensity);
@@ -169,9 +170,9 @@ void show_objectives(display& disp, const config& level, const std::string& obje
 	static const std::string no_objectives(_("No objectives available"));
 	const std::string& name = level["name"];
 
-	gui::show_dialog(disp, NULL, "", font::LARGE_TEXT + name + "\n" +
-	         (objectives.empty() ? no_objectives : objectives), gui::OK_ONLY);
-
+	gui::show_dialog(disp, NULL, "", "*~" + name + "\n" +
+	                 (objectives.empty() ? no_objectives : objectives),
+	                 gui::OK_ONLY);
 }
 
 int get_save_name(display & disp,const std::string& caption, const std::string& message,
@@ -253,7 +254,8 @@ public:
 	                  const std::vector<save_info>& info, const std::vector<config*>& summaries)
 		: gui::preview_pane(video), game_config_(&game_config), map_(map), data_(&data), info_(&info), summaries_(&summaries), index_(0)
 	{
-		set_measurements(200, 400);
+		set_measurements(minimum<int>(200,video.getx()/4),
+				 minimum<int>(400,video.gety() * 4/5));
 	}
 
 	void draw_contents();
@@ -327,9 +329,14 @@ void save_preview_pane::draw_contents()
 			map_surf = itor->second;
 		} else if(map_ != NULL) {
 			try {
+#ifdef USE_TINY_GUI
+				const int minimap_size = 60;
+#else
+				const int minimap_size = 100;
+#endif
 				map_->read(map_data);
 
-				map_surf = image::getMinimap(100, 100, *map_);
+				map_surf = image::getMinimap(minimap_size, minimap_size, *map_);
 				if(map_surf != NULL) {
 					map_cache_.insert(std::pair<std::string,surface>(map_data,surface(map_surf)));
 				}
@@ -361,7 +368,7 @@ void save_preview_pane::draw_contents()
 		str << "\n" << _("#(Invalid)");
 	} else if (!campaign_type.empty()) {
 		str << "\n";
-			
+
 		if(campaign_type == "scenario") {
 			str << _("Campaign");
 		} else if(campaign_type == "multiplayer") {
@@ -373,7 +380,7 @@ void save_preview_pane::draw_contents()
 		}
 
 		str << "\n";
-			
+
 		if(summary["snapshot"] == "no" && summary["replay"] == "yes") {
 			str << _("replay");
 		} else if (!summary["turn"].empty()) {
@@ -391,6 +398,51 @@ void save_preview_pane::draw_contents()
 	font::draw_text(&video(), area, font::SIZE_SMALL, font::NORMAL_COLOUR, str.str(), area.x, ypos, true);
 }
 
+std::string format_time_summary(time_t t)
+{
+	time_t curtime = time(NULL);
+	const struct tm* timeptr = localtime(&curtime);
+	if(timeptr == NULL) {
+		return "";
+	}
+
+	const struct tm current_time = *timeptr;
+
+	timeptr = localtime(&t);
+	if(timeptr == NULL) {
+		return "";
+	}
+
+	const struct tm save_time = *timeptr;
+
+	const char* format_string = _("%b %d %y");
+
+	if(current_time.tm_year == save_time.tm_year) {
+		const int days_apart = current_time.tm_yday - save_time.tm_yday;
+		if(days_apart == 0) {
+			//save is from today
+			format_string = _("%H:%M");
+		} else if(days_apart > 0 && days_apart <= current_time.tm_wday) {
+			//save is from this week
+			format_string = _("%A, %H:%M");
+		} else {
+			//save is from current year
+			format_string = _("%b %d");
+		}
+	} else {
+		//save is from a different year
+		format_string = _("%b %d %y");
+	}
+
+	char buf[40];
+	const size_t res = strftime(buf,sizeof(buf),format_string,&save_time);
+	if(res == 0) {
+		buf[0] = 0;
+	}
+
+	return buf;
+}
+
 } //end anon namespace
 
 std::string load_game_dialog(display& disp, const config& game_config, const game_data& data, bool* show_replay)
@@ -494,13 +546,23 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	}
 
 	std::vector<std::string> items;
+	std::ostringstream heading;
+	heading << HEADING_PREFIX << _("Name") << COLUMN_SEPARATOR << _("Date");
+	items.push_back(heading.str());
+
 	for(i = games.begin(); i != games.end(); ++i) {
 		std::string name = i->name;
 		name.resize(minimum<size_t>(name.size(),40));
 
-		items.push_back(name);
+		std::ostringstream str;
+		str << name << COLUMN_SEPARATOR << format_time_summary(i->time_modified);
+
+		items.push_back(str.str());
 	}
 
+	gui::menu::basic_sorter sorter;
+	sorter.set_alpha_sort(0).set_id_sort(1);
+
 	gamemap map_obj(game_config,"");
 
 	std::vector<gui::preview_pane*> preview_panes;
@@ -516,7 +578,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	const int res = gui::show_dialog(disp,NULL,
 					 _("Load Game"),
 					 _("Choose the game to load"),
-			         gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,&options,-1,-1,NULL,&buttons);
+			         gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,&options,-1,-1,NULL,&buttons,"",&sorter);
 
 	if(res == -1)
 		return "";
@@ -666,7 +728,7 @@ void unit_preview_pane::draw_contents()
 
 	details << _("HP: ") << u.hitpoints()
 			<< "/" << u.max_hitpoints() << "\n";
-	
+
 	if(u.can_advance() == false) {
 		details << _("XP: ") << u.experience() << "/-";
 	} else {
@@ -678,7 +740,7 @@ void unit_preview_pane::draw_contents()
 
 		details << _("XP: ") << u.experience() << "/" << u.max_experience();
 	}
-	
+
 	if(weapons_) {
 		details << "\n"
 				<< _("Moves: ") << u.movement_left() << "/"
@@ -699,9 +761,9 @@ void unit_preview_pane::draw_contents()
 			        << (at_it->range() == attack_type::SHORT_RANGE ? _("melee") : _("ranged"));
 		}
 	}
-	
+
 	const std::string text = details.str();
-	
+
 	const std::vector<std::string> lines = utils::split(text, '\n');
 
 	SDL_Rect cur_area = area;
@@ -742,7 +804,11 @@ namespace {
 
 campaign_preview_pane::campaign_preview_pane(CVideo &video,std::vector<std::pair<std::string,std::string> >* desc) : gui::preview_pane(video),descriptions_(desc),index_(0)
 {
-	set_measurements(font::relative_size(350), font::relative_size(400));
+#ifdef USE_TINY_GUI
+	set_measurements(160, 200);
+#else
+	set_measurements(350, 400);
+#endif
 }
 
 bool campaign_preview_pane::show_above() const { return false; }
