@@ -1,7 +1,7 @@
-/* $Id: filesystem.cpp,v 1.67 2005/04/05 21:47:27 Sirp Exp $ */
+/* $Id: filesystem.cpp,v 1.74 2005/06/24 22:41:49 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -63,7 +63,8 @@ BPath be_path;
 #include "scoped_resource.hpp"
 #include "util.hpp"
 
-#define LOG_G LOG_STREAM(info, general)
+#define LOG_FS LOG_STREAM(info, filesystem)
+#define ERR_FS LOG_STREAM(err, filesystem)
 
 #ifdef USE_ZIPIOS
 #include <sstream>
@@ -89,13 +90,13 @@ namespace {
 				    (0 == fname.compare(fname.size() - suffix.size(), suffix.size(), suffix))) {
 					zipios::ZipFile zip(game_config::path + "/" + fname);
 					the_collection->addCollection(zip);
-					LOG_G << "zip collection " << fname << 
-						" has " << zip.size() << " elements\n";
+					LOG_FS << "zip collection " << fname
+					       << " has " << zip.size() << " elements\n";
 				}
 			}
-			else LOG_G << "skipping invalid entry\n";
+			else LOG_FS << "skipping invalid entry\n";
 		}
-		
+
 	}
 }
 #endif
@@ -111,16 +112,16 @@ bool filesystem_init()
 	the_collection = new xzipios::XCColl;
 
 	if (!get_user_data_dir().empty()) {
-		LOG_G << "looking at user dir " << get_user_data_dir() << "\n";
+		LOG_FS << "looking at user dir " << get_user_data_dir() << '\n';
 		zipios::DirectoryCollection dir(get_user_data_dir());
-		LOG_G << "user collection has " << dir.size() << " elements\n";
+		LOG_FS << "user collection has " << dir.size() << " elements\n";
 		the_collection->addCollection(dir);
 		register_zipdir(get_user_data_dir());
 	}
 	if (!game_config::path.empty()) {
-		LOG_G << "looking at system dir " << game_config::path << "\n";
+		LOG_FS << "looking at system dir " << game_config::path << '\n';
 		zipios::DirectoryCollection dir(game_config::path);
-		LOG_G << "system collection has " << dir.size() << " elements\n";
+		LOG_FS << "system collection has " << dir.size() << " elements\n";
 		the_collection->addCollection(dir);
 		register_zipdir(game_config::path);
 	}
@@ -150,7 +151,7 @@ void get_files_in_dir(const std::string& directory,
 	else
 #endif
 	{
-		
+
 		//if we have a path to find directories in, then convert relative
 		//pathnames to be rooted on the wesnoth path
 		if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
@@ -262,20 +263,25 @@ std::string get_intl_dir()
 	return get_cwd() + "/po";
 #endif
 
-#ifdef __APPLE__
-	return get_cwd() + "/messages";
+#ifdef USE_INTERNAL_DATA
+	return get_cwd() + "/" LOCALEDIR;
 #endif
 
-	std::string res;
 #if HAS_RELATIVE_LOCALEDIR
-	res = game_config::path + "/" LOCALEDIR;
+	std::string res = game_config::path + "/" LOCALEDIR;
 #else
-	res = LOCALEDIR;
+	std::string res = LOCALEDIR;
 #endif
 
 	return res;
 }
 
+std::string get_screenshot_dir()
+{
+	const std::string dir_path = get_user_data_dir() + "/screenshots";
+	return get_dir(dir_path);
+}
+
 std::string get_dir(const std::string& dir_path)
 {
 #ifdef _WIN32
@@ -288,8 +294,7 @@ std::string get_dir(const std::string& dir_path)
 		if(res == 0) {
 			dir = opendir(dir_path.c_str());
 		} else {
-			std::cerr << "Could not open or create directory: '" << dir_path
-			          << "'\n";
+			ERR_FS << "could not open or create directory: " << dir_path << '\n';
 		}
 	}
 
@@ -389,8 +394,7 @@ std::string get_user_data_dir()
 		if(res == 0) {
 			dir = opendir(dir_path.c_str());
 		} else {
-			std::cerr << "Could not open or create directory: '" << dir_path
-			          << "'\n";
+			ERR_FS << "could not open or create directory: " << dir_path << '\n';
 		}
 	}
 
@@ -403,44 +407,6 @@ std::string get_user_data_dir()
 #endif
 }
 
-namespace {
-void read_file_internal(const std::string& fname, std::string& res)
-{
-	const int size = file_size(fname);
-	if(size < 0) {
-		return;
-	}
-
-	std::vector<char> v;
-	v.reserve(size);
-
-	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"rb"));
-	const util::scoped_FILE file(fopen(fname.c_str(),"rb"));
-
-	if(file == NULL) {
-		return;
-	}
-
-	const int block_size = 65536;
-
-	while(v.size() < size) {
-		const size_t expected = minimum<size_t>(block_size,size - v.size());
-
-		if(expected > 0) {
-			v.resize(v.size() + expected);
-			const size_t nbytes = fread(&v[v.size() - expected],1,expected,file);
-			if(nbytes < expected) {
-				v.resize(v.size() - (expected - nbytes));
-				break;
-			}
-		}
-	}
-
-	res.resize(v.size());
-	std::copy(v.begin(),v.end(),res.begin());
-}
-} //end anon namespace
-
 std::string read_stream(std::istream& s)
 {
 	std::stringstream ss;
@@ -453,47 +419,11 @@ std::string read_stdin()
 	return read_stream(std::cin);
 }
 
-std::string read_file(const std::string& fname)
-{
-	//if we have a path to the data,
-	//convert any filepath which is relative
-	LOG_G << "Reading " << fname << "\n";
-#ifdef USE_ZIPIOS
-	if(!fname.empty() && fname[0] != '/') {
-		zipios::ConstEntryPointer p = the_collection->getEntry(fname);
-		if (p != 0) {
-			std::istream* s = the_collection->getInputStream(p);
-			if (s != NULL) {
-				std::string contents = read_stream(*s);
-				delete s;
-				return contents;
-			}
-		}
-	}
-#else
-	if(!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
-		std::string res;
-		read_file_internal(game_config::path + "/" + fname,res);
-		if(!res.empty()) {
-			return res;
-		}
-	}
-#endif
-
-	// FIXME: why do we rely on this even with relative paths ?
-	{
-		// still useful with zipios, for things like cache and prefs
-		std::string res;
-		read_file_internal(fname,res);
-		return res;
-	}
-}
-
 std::istream *istream_file(std::string const &fname)
 {
-	LOG_G << "Streaming " << fname << " for reading.\n";
+	LOG_FS << "streaming " << fname << " for reading.\n";
 #ifdef USE_ZIPIOS
-	if (!fname.empty() && fname[0] != '/') {
+	if (!fname.empty() && fname[0] != '/' && the_collection) {
 		zipios::ConstEntryPointer p = the_collection->getEntry(fname);
 		if (p != 0)
 			if (std::istream *s = the_collection->getInputStream(p))
@@ -513,9 +443,15 @@ std::istream *istream_file(std::string const &fname)
 	return new std::ifstream(fname.c_str(),std::ios_base::binary);
 }
 
+std::string read_file(std::string const &fname)
+{
+	scoped_istream s = istream_file(fname);
+	return read_stream(*s);
+}
+
 std::ostream *ostream_file(std::string const &fname)
 {
-	LOG_G << "Streaming " << fname << " for writing.\n";
+	LOG_FS << "streaming " << fname << " for writing.\n";
 	return new std::ofstream(fname.c_str(), std::ios_base::binary);
 }
 
@@ -690,9 +626,9 @@ const file_tree_checksum& data_tree_checksum()
 	if(checksum.nfiles == 0) {
 		get_file_tree_checksum_internal("data/",checksum);
 		get_file_tree_checksum_internal(get_user_data_dir() + "/data/",checksum);
-		LOG_G << "calculated data tree checksum: "
-		      << checksum.nfiles << " files; "
-		      << checksum.sum_size << " bytes\n";
+		LOG_FS << "calculated data tree checksum: "
+		       << checksum.nfiles << " files; "
+		       << checksum.sum_size << " bytes\n";
 	}
 
 	return checksum;
@@ -802,12 +738,12 @@ const std::vector<std::string>& get_binary_paths(const std::string& type)
 	}
 
 	std::vector<std::string>& res = binary_paths_cache[type];
-	
+
 	init_binary_paths();
 
 	for(std::set<std::string>::const_iterator i = binary_paths.begin(); i != binary_paths.end(); ++i) {
 		res.push_back(get_user_data_dir() + "/" + *i + type + "/");
-		
+
 		if(!game_config::path.empty()) {
 			res.push_back(game_config::path + "/" + *i + type + "/");
 		}
@@ -823,7 +759,7 @@ std::string get_binary_file_location(const std::string& type, const std::string&
 	const std::vector<std::string>& paths = get_binary_paths(type);
 	for(std::vector<std::string>::const_iterator i = paths.begin(); i != paths.end(); ++i) {
 		const std::string file = *i + filename;
-		if(file_exists(file)) {
+		if(file_exists(file) || is_directory(file)) {
 			return file;
 		}
 	}
