@@ -1,7 +1,7 @@
-/* $Id: filesystem.hpp,v 1.35 2005/03/26 14:07:16 silene Exp $ */
+/* $Id: filesystem.hpp,v 1.37 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -50,6 +50,7 @@ std::string get_save_index_file();
 std::string get_saves_dir();
 std::string get_cache_dir();
 std::string get_intl_dir();
+std::string get_screenshot_dir();
 std::string get_user_data_dir();
 
 std::string get_cwd();
