@@ -1,8 +1,8 @@
-/* $Id: font.cpp,v 1.133 2005/04/09 20:52:22 gruikya Exp $ */
+/* $Id: font.cpp,v 1.145 2005/06/04 19:16:05 ott Exp $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -83,10 +83,10 @@ struct text_chunk
 	text_chunk(subset_id subset, std::string const & text) : subset(subset), text(text) {}
 	text_chunk(subset_id subset, ucs2_string const & ucs2_text) : subset(subset), ucs2_text(ucs2_text) {}
 	text_chunk(subset_id subset, std::string const & text, ucs2_string const & ucs2_text) : subset(subset), text(text), ucs2_text(ucs2_text) {}
-	
+
 	bool operator==(text_chunk const & t) const { return subset == t.subset && ucs2_text == t.ucs2_text; }
 	bool operator!=(text_chunk const & t) const { return !operator==(t); }
-	
+
 	subset_id subset;
 	//FIXME if we don't need the utf8 here remove it
 	std::string text;
@@ -95,6 +95,12 @@ struct text_chunk
 
 std::vector<subset_id> font_map;
 
+//cache sizes of small text
+typedef std::map<std::string,SDL_Rect> line_size_cache_map;
+
+//map of styles -> sizes -> cache
+std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
+
 //Splits the UTF-8 text into text_chunks using the same font.
 std::vector<text_chunk> split_text(std::string const & utf8_text) {
 	text_chunk current_chunk(0);
@@ -109,8 +115,8 @@ std::vector<text_chunk> split_text(std::string const & utf8_text) {
 			current_chunk.subset = font_map[size_t(*ch)];
 		}
 		for(utils::utf8_iterator end = utils::utf8_iterator::end(utf8_text); ch != end; ++ch) {
-			if(size_t(*ch) < font_map.size() && 
-					font_map[size_t(*ch)] >= 0 && 
+			if(size_t(*ch) < font_map.size() &&
+					font_map[size_t(*ch)] >= 0 &&
 					font_map[size_t(*ch)] != current_chunk.subset) {
 				//null-terminate ucs2_text so we can pass it to SDL_ttf later
 				current_chunk.ucs2_text.push_back(0);
@@ -214,6 +220,7 @@ void clear_fonts()
 	font_table.clear();
 	font_names.clear();
 	font_map.clear();
+	line_size_cache.clear();
 }
 
 struct font_style_setter
@@ -233,7 +240,7 @@ struct font_style_setter
 		//according to the SDL_ttf documentation, combinations of
 		//styles may cause SDL_ttf to segfault. We work around this
 		//here by disallowing combinations of styles
-		
+
 		if((style&TTF_STYLE_UNDERLINE) != 0) {
 			//style = TTF_STYLE_NORMAL; //TTF_STYLE_UNDERLINE;
 			style = TTF_STYLE_UNDERLINE;
@@ -242,7 +249,7 @@ struct font_style_setter
 		} else if((style&TTF_STYLE_ITALIC) != 0) {
 			//style = TTF_STYLE_NORMAL; //TTF_STYLE_ITALIC;
 			style = TTF_STYLE_ITALIC;
-		} 
+		}
 #endif
 
 		TTF_SetFontStyle(font_, style);
@@ -296,7 +303,7 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist)
 		font_names.push_back(itor->name);
 
 		std::vector<std::pair<size_t,size_t> >::const_iterator cp_range;
-		for(cp_range = itor->present_codepoints.begin(); 
+		for(cp_range = itor->present_codepoints.begin();
 				cp_range != itor->present_codepoints.end(); ++cp_range) {
 
 			size_t cp_max = maximum<size_t>(cp_range->first, cp_range->second);
@@ -304,7 +311,7 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist)
 				font_map.resize(cp_max+1, -1);
 			}
 			for(size_t cp = cp_range->first; cp <= cp_range->second; ++cp) {
-				if(font_map[cp] < 0) 
+				if(font_map[cp] < 0)
 					font_map[cp] = subset;
 			}
 		}
@@ -312,12 +319,14 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist)
 }
 
 const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
+                LOBBY_COLOUR  = {0xBB,0xBB,0xBB,0},
                 GOOD_COLOUR   = {0x00,0xFF,0x00,0},
                 BAD_COLOUR    = {0xFF,0x00,0x00,0},
                 BLACK_COLOUR  = {0x00,0x00,0x00,0},
                 DARK_COLOUR   = {0x00,0x00,0x66,0},
                 YELLOW_COLOUR = {0xFF,0xFF,0x00,0},
                 BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
+                STONED_COLOUR = {0xA0,0xA0,0xA0,0},
                 TITLE_COLOUR  = {0xBC,0xB0,0x88,0};
 
 const char LARGE_TEXT='*', SMALL_TEXT='`', GOOD_TEXT='@', BAD_TEXT='#',
@@ -340,7 +349,7 @@ public:
 	std::vector<surface> const & get_surfaces() const;
 
 	bool operator==(text_surface const &t) const {
-		return hash_ == t.hash_ && font_size_ == t.font_size_ 
+		return hash_ == t.hash_ && font_size_ == t.font_size_
 			&& color_ == t.color_ && style_ == t.style_ && str_ == t.str_;
 	}
 	bool operator!=(text_surface const &t) const { return !operator==(t); }
@@ -390,8 +399,8 @@ void text_surface::measure() const
 {
 	w_ = 0;
 	h_ = 0;
-	
-	for(std::vector<text_chunk>::iterator itor = chunks_.begin(); 
+
+	for(std::vector<text_chunk>::iterator itor = chunks_.begin();
 			itor != chunks_.end(); ++itor) {
 
 		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
@@ -405,7 +414,7 @@ void text_surface::measure() const
 		if(itor->ucs2_text.back() != 0) {
 			itor->ucs2_text.push_back(0);
 		}
-		
+
 		TTF_SizeUNICODE(ttfont, (Uint16 const *)&(itor->ucs2_text.front()), &w, &h);
 		w_ += w;
 		h_ = maximum<int>(h_, h);
@@ -425,7 +434,7 @@ size_t text_surface::width() const
 size_t text_surface::height() const
 {
 	if (h_ == -1) {
-		if(chunks_.empty()) 
+		if(chunks_.empty())
 			chunks_ = split_text(str_);
 		measure();
 	}
@@ -444,13 +453,13 @@ std::vector<surface> const &text_surface::get_surfaces() const
 	if(width() > max_text_line_width)
 		return surfs_;
 
-	for(std::vector<text_chunk>::const_iterator itor = chunks_.begin(); 
+	for(std::vector<text_chunk>::const_iterator itor = chunks_.begin();
 			itor != chunks_.end(); ++itor) {
 		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
 		if (ttfont == NULL)
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
-		
+
 		surface s = surface(TTF_RenderUNICODE_Blended(ttfont, (Uint16 const *)&(itor->ucs2_text.front()), color_));
 		if(!s.null())
 			surfs_.push_back(s);
@@ -490,6 +499,8 @@ text_surface &text_cache::find(text_surface const &t)
 	return cache_.front();
 }
 
+}
+
 surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style)
 {
 	const std::vector<std::string> lines = utils::split(text, '\n', utils::REMOVE_EMPTY);
@@ -545,6 +556,8 @@ surface render_text(const std::string& text, int fontsize, const SDL_Color& colo
 	}
 }
 
+namespace {
+
 //function which will parse the markup tags at the front of a string
 std::string::const_iterator parse_markup(std::string::const_iterator i1, std::string::const_iterator i2,
 					 int* font_size, SDL_Color* colour, int* style)
@@ -582,12 +595,25 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 			break;
 		case NULL_MARKUP:
 			return i1+1;
-		default:
-			if(*i1 >= 1 && *i1 <= 9) {
-				*colour = team::get_side_colour(*i1);
-				break;
+		// semi ANSI colour escape sequences at the start of the line for now only
+		case '\033':
+			if(i2 - i1 >= 4) {
+				++i1;
+				if(*i1 == '[') {
+					++i1;
+					if(*i1 == '3') {
+						++i1;
+						if(*i1 >= '0' && *i1 <= '9' && *(i1 + 1) == 'm')
+						{
+							if(*i1 != '0')
+								*colour = team::get_side_colour(lexical_cast<int, char>(*i1));
+							++i1;
+						}
+					}
+				}
 			}
-
+			break;
+		default:
 			return i1;
 		}
 
@@ -613,7 +639,7 @@ surface get_rendered_text(const std::string& str, int size, const SDL_Color& col
 //length restriction and small font size.
 SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::const_iterator last, int font_size, int style) {
 	wassert(last - first >= 0);
-	
+
 	SDL_Rect rect;
 	rect.w = 0;
 	rect.h = 0;
@@ -628,7 +654,7 @@ SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::
 	if(*first < font_map.size() && font_map[*first] >= 0) {
 		current_font = font_map[*first];
 	}
-	
+
 	for(;first != last; ++first) {
 		if(*first < font_map.size() && font_map[*first] >= 0 && font_map[*first] != current_font) {
 			TTF_Font* ttfont = get_font(font_id(current_font, font_size));
@@ -639,16 +665,16 @@ SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::
 				continue;
 			}
 			*(chunk_itor++) = 0;
-			
+
 			font_style_setter const style_setter(ttfont, style);
 
 			TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), (int*)&rect.x, (int*)&rect.y);
 
 			rect.w += rect.x;
 			rect.h = maximum<Sint16>(rect.h, rect.y);
-			
+
 			chunk_itor = chunk.begin();
-			
+
 			current_font = font_map[*first];
 		}
 		*(chunk_itor++) = *first;
@@ -710,7 +736,7 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 	if(line_width(text, size) > area.w) {
 		tooltips::add_tooltip(dest,text);
 	}
-	
+
 	if(dest.x + dest.w > area.x + area.w) {
 		dest.w = area.x + area.w - dest.x;
 	}
@@ -738,13 +764,13 @@ SDL_Rect draw_text_line(CVideo* gui, const SDL_Rect& area, int size,
                         int x, int y, bool use_tooltips, int style)
 {
 	surface surface;
-	
+
 	if(gui == NULL) {
 		surface = NULL;
 	} else {
 		surface = gui->getSurface();
 	}
-	
+
 	return draw_text_line(surface, area, size, colour, text, x, y, use_tooltips, style);
 }
 
@@ -814,9 +840,20 @@ int get_max_height(int size)
 
 bool is_format_char(char c)
 {
+	//side coloring
+	if(c > 0 && c <= 10) {
+		return true;
+	}
+
 	switch(c) {
+	case LARGE_TEXT:
+	case SMALL_TEXT:
 	case GOOD_TEXT:
 	case BAD_TEXT:
+	case NORMAL_TEXT:
+	case BLACK_TEXT:
+	case BOLD_TEXT:
+	case NULL_MARKUP:
 		return true;
 	default:
 		return false;
@@ -829,20 +866,27 @@ std::string remove_first_space(const std::string& text)
   if (text.length() > 0 && text[0] == ' ') {
     return text.substr(1);
   }
-  
+
   return text;
 }
 
 int line_width(const std::string& line, int font_size, int style)
 {
-	const SDL_Color col = { 0, 0, 0, 0 };
-	text_surface s(line, font_size, col, style);
-
-	return s.width();
+	return line_size(line,font_size,style).w;
 }
 
 SDL_Rect line_size(const std::string& line, int font_size, int style)
 {
+	const size_t max_cache_size = 12;
+	line_size_cache_map& cache = line_size_cache[style][font_size];
+
+	if(line.size() < max_cache_size) {
+		const line_size_cache_map::const_iterator i = cache.find(line);
+		if(i != cache.end()) {
+			return i->second;
+		}
+	}
+
 	SDL_Rect res;
 
 	const SDL_Color col = { 0, 0, 0, 0 };
@@ -851,6 +895,10 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 	res.w = s.width();
 	res.h = s.height();
 
+	if(line.size() < max_cache_size) {
+		cache.insert(std::pair<std::string,SDL_Rect>(line,res));
+	}
+
 	return res;
 }
 
@@ -911,7 +959,7 @@ inline bool no_break_before(wchar_t ch)
 		ch == 0x30e3 || ch == 0x30e5 || ch == 0x30e7 || ch == 0x30ee || ch == 0x30f5 ||
 		ch == 0x30f6 || ch == 0x30fb || ch == 0x30fc || ch == 0x30fd || ch == 0x30fe ||
 		ch == 0xff01 || ch == 0xff09 || ch == 0xff0c || ch == 0xff0e || ch == 0xff1a ||
-		ch == 0xff1b || ch == 0xff1f || ch == 0xff3d || ch == 0xff5d; 
+		ch == 0xff1b || ch == 0xff1f || ch == 0xff3d || ch == 0xff5d;
 }
 
 inline bool break_before(wchar_t ch)
@@ -919,7 +967,7 @@ inline bool break_before(wchar_t ch)
 	if(no_break_before(ch))
 		return false;
 
-	return ch == ' ' || 
+	return ch == ' ' ||
 		// CKJ characters
 		(ch >= 0x3000 && ch < 0xa000) ||
 		(ch >= 0xf900 && ch < 0xfb00) ||
@@ -931,7 +979,7 @@ inline bool break_after(wchar_t ch)
 	if(no_break_after(ch))
 		return false;
 
-	return ch == ' ' || 
+	return ch == ' ' ||
 		// CKJ characters
 		(ch >= 0x3000 && ch < 0xa000) ||
 		(ch >= 0xf900 && ch < 0xfb00) ||
@@ -939,19 +987,36 @@ inline bool break_after(wchar_t ch)
 }
 }
 
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height)
+std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height, int max_lines)
 {
+	assert(max_width > 0);
+
 	utils::utf8_iterator ch(unwrapped_text);
 	std::string current_word;
 	std::string current_line;
+	size_t line_width = 0;
 	size_t current_height = 0;
 	bool line_break = false;
 	bool first = true;
+	bool start_of_line = true;
 	std::string wrapped_text;
-	
+	std::string format_string;
+	utils::utf8_iterator end = utils::utf8_iterator::end(unwrapped_text);
+
 	while(1) {
+		if(start_of_line) {
+			line_width = 0;
+			format_string = "";
+			while(ch != end && *ch < 0x100U && is_format_char(*ch)) {
+				format_string.append(ch.substr().first, ch.substr().second);
+				++ch;
+			}
+			current_line = format_string;
+			start_of_line = false;
+		}
+
 		// If there is no current word, get one
-		if(current_word.empty() && ch == utils::utf8_iterator::end(unwrapped_text)) {
+		if(current_word.empty() && ch == end) {
 			break;
 		} else if(current_word.empty()) {
 			if(*ch == ' ' || *ch == '\n') {
@@ -963,7 +1028,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 						*ch != ' ' && *ch != '\n'; ++ch) {
 
 					if(!current_word.empty() &&
-							break_before(*ch) && 
+							break_before(*ch) &&
 							!no_break_after(previous))
 						break;
 
@@ -982,12 +1047,17 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 		if(current_word == "\n") {
 			line_break = true;
 			current_word = "";
+			start_of_line = true;
 		} else {
-			SDL_Rect size = line_size(current_line + current_word, font_size);
 
-			if(size.w > max_width) {
-				SDL_Rect wsize = line_size(current_word, font_size);
-				if(wsize.w > max_width) {
+			const std::string word = format_string + current_word;
+
+			const size_t word_width = line_size(word,font_size).w;
+
+			line_width += word_width;
+
+			if(line_width > max_width) {
+				if(word_width > max_width) {
 					cut_word(current_line, current_word, font_size, max_width);
 				}
 				if(current_word == " ")
@@ -999,18 +1069,26 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 			}
 		}
 
-		if(line_break || current_word.empty() && ch == utils::utf8_iterator::end(unwrapped_text)) {
+		if(line_break || current_word.empty() && ch == end) {
 			SDL_Rect size = line_size(current_line, font_size);
-			if(max_height > 0 && current_height + size.h >= size_t(max_height))
+			if(max_height > 0 && current_height + size.h >= size_t(max_height)) {
 				return wrapped_text;
+			}
 
-			if(!first)
+			if(!first) {
 				wrapped_text += '\n';
+			}
+
 			wrapped_text += current_line;
-			current_line = "";
+			current_line = format_string;
+			line_width = 0;
 			current_height += size.h;
 			line_break = false;
 			first = false;
+
+			if(--max_lines == 0) {
+				return wrapped_text;
+			}
 		}
 	}
 	return wrapped_text;
@@ -1019,12 +1097,12 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 std::string make_text_ellipsis(const std::string &text, int font_size, int max_width)
 {
 	static const std::string ellipsis = "...";
-	
+
 	if(line_width(text, font_size) <= max_width)
 		return text;
 	if(line_width(ellipsis, font_size) > max_width)
 		return "";
-	
+
 	std::string current_substring;
 
 	utils::utf8_iterator itor(text);
@@ -1037,7 +1115,7 @@ std::string make_text_ellipsis(const std::string &text, int font_size, int max_w
 		if (line_width(tmp, font_size) > max_width) {
 			return current_substring + ellipsis;
 		}
-		
+
 		current_substring.append(itor.substr().first, itor.substr().second);
 	}
 
@@ -1394,12 +1472,12 @@ void undraw_floating_labels(surface screen)
 }
 
 namespace {
-	bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name) 
+	bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name)
 	{
 		config* font = fonts_config->find_child("font", "name", name);
 		if(font == NULL)
 			return false;
-		
+
 		fontlist.push_back(font::subset_descriptor());
 		fontlist.back().name = name;
 		std::vector<std::string> ranges = utils::split((*font)["codepoints"]);
@@ -1433,8 +1511,9 @@ bool load_font_config()
 	try {
 		scoped_istream stream = preprocess_file("data/fonts.cfg");
 		read(cfg, *stream);
-	} catch(config::error&) {
-		std::cerr << "Could not read fonts.cfg\n";
+	} catch(config::error &e) {
+		ERR_FT << "could not read fonts.cfg:\n"
+		       << e.message << '\n';
 		return false;
 	}
 
