@@ -1,7 +1,7 @@
-/* $Id: font.hpp,v 1.54 2005/04/02 14:06:35 gruikya Exp $ */
+/* $Id: font.hpp,v 1.59 2005/06/22 23:30:18 ydirson Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -44,8 +44,9 @@ struct subset_descriptor
 void set_font_list(const std::vector<subset_descriptor>& fontlist);
 
 //various standard colours
-extern const SDL_Color NORMAL_COLOUR, GOOD_COLOUR, BAD_COLOUR, BLACK_COLOUR,
-                       DARK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR, TITLE_COLOUR;
+extern const SDL_Color NORMAL_COLOUR, LOBBY_COLOUR, GOOD_COLOUR, BAD_COLOUR,
+                       BLACK_COLOUR, DARK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR,
+                       STONED_COLOUR, TITLE_COLOUR;
 
 //standard markups
 extern const char LARGE_TEXT, SMALL_TEXT, GOOD_TEXT, BAD_TEXT, NORMAL_TEXT, BLACK_TEXT, BOLD_TEXT, IMAGE, NULL_MARKUP;
@@ -53,7 +54,7 @@ extern const char LARGE_TEXT, SMALL_TEXT, GOOD_TEXT, BAD_TEXT, NORMAL_TEXT, BLAC
 // font sizes, to be made theme parameters
 #ifdef USE_TINY_GUI
 // this is not meant for normal play, just for checking other dimensions get adapted accordingly
-const int SIZE_NORMAL = 8;
+const int SIZE_NORMAL = 9;
 #else
 const int SIZE_NORMAL = 14;
 #endif
@@ -70,7 +71,7 @@ const int
 	SIZE_PLUS       = relative_size(16),
 	SIZE_LARGE      = relative_size(18),
 	SIZE_XLARGE     = relative_size(24)
-  ;  
+  ;
 //function to draw text on the screen. The text will be clipped to area.
 //If the text runs outside of area horizontally, an ellipsis will be displayed
 //at the end of it. If use_tooltips is true, then text with an ellipsis will
@@ -103,7 +104,7 @@ SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::
 
 // Returns the maximum height of a font, in pixels
 int get_max_height(int size);
-	
+
 bool is_format_char(char c);
 
 ///
@@ -123,13 +124,13 @@ SDL_Rect line_size(const std::string& line, int font_size, int style=TTF_STYLE_N
 /// If the is not possible, e.g. the word is too big to fit, wrap it on a
 /// char basis.
 ///
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height=-1);
+std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height=-1, int max_lines=-1);
 
 ///
 /// If the text excedes the specified max width, end it with an ellipsis (...)
 ///
 std::string make_text_ellipsis(const std::string& text, int font_size, int max_width);
-	
+
 
 ///
 /// Draw text on the screen. This method makes sure that the text
@@ -139,7 +140,7 @@ std::string make_text_ellipsis(const std::string& text, int font_size, int max_w
 /// but it doesn't support special markup or tooltips.
 ///
 /// @return a bounding rectangle of the text.
-/// 
+///
 SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
 			     int x, int y, int max_width);
@@ -170,7 +171,7 @@ enum LABEL_SCROLL_MODE { ANCHOR_LABEL_SCREEN, ANCHOR_LABEL_MAP };
 int add_floating_label(const std::string& text, int font_size, const SDL_Color& colour,
 		double xpos, double ypos, double xmove, double ymove, int lifetime,
 		const SDL_Rect& clip_rect, ALIGN alignment=CENTER_ALIGN,
-		const SDL_Color* bg_colour=NULL, int border_size=0, 
+		const SDL_Color* bg_colour=NULL, int border_size=0,
 		LABEL_SCROLL_MODE scroll_mode=ANCHOR_LABEL_SCREEN);
 
 /// moves the floating label given by 'handle' by (xmove,ymove)
