@@ -1,7 +1,7 @@
-/* $Id: game_config.cpp,v 1.94 2005/03/10 00:00:06 ydirson Exp $ */
+/* $Id: game_config.cpp,v 1.99 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -11,11 +11,12 @@
    See the COPYING file for more details.
 */
 
-#include "config.hpp"
 #include "global.hpp"
 
+#include "config.hpp"
 #include "game_config.hpp"
 #include "wesconfig.h"
+#include "serialization/string_utils.hpp"
 
 #include <cstdlib>
 
@@ -55,11 +56,11 @@ namespace game_config
 
 	std::string dot_image = "misc/dot.png";
 	std::string cross_image = "misc/cross.png";
-	
-	std::string foot_left_nw, foot_left_n, foot_right_nw, foot_right_n;
+
+	std::vector<std::string> foot_left_nw, foot_left_n, foot_right_nw, foot_right_n;
 
 	std::string observer_image;
-	
+
 	std::string unchecked_menu_image = "buttons/checkbox.png";
 	std::string checked_menu_image = "buttons/checkbox-pressed.png";
 
@@ -126,16 +127,16 @@ namespace game_config
 		cross_image = v["cross_image"];
 		dot_image = v["dot_image"];
 
-		foot_left_nw = v["footprint_left_nw"];
-		foot_left_n = v["footprint_left_n"];
-		foot_right_nw = v["footprint_right_nw"];
-		foot_right_n = v["footprint_right_n"];
+		foot_left_nw = utils::split(v["footprint_left_nw"]);
+		foot_left_n = utils::split(v["footprint_left_n"]);
+		foot_right_nw = utils::split(v["footprint_right_nw"]);
+		foot_right_n = utils::split(v["footprint_right_n"]);
 
 		missile_n_image = v["missile_n_image"];
 		missile_ne_image = v["missile_ne_image"];
 
 		terrain_mask_image = v["terrain_mask_image"];
-		
+
 		observer_image = v["observer_image"];
 
 		download_campaign_image = v["download_campaign_image"];
