@@ -1,7 +1,7 @@
-/* $Id: game_config.hpp,v 1.27 2005/03/10 00:00:06 ydirson Exp $ */
+/* $Id: game_config.hpp,v 1.30 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -16,6 +16,7 @@
 class config;
 
 #include <string>
+#include <vector>
 
 //basic game configuration information is here.
 namespace game_config
@@ -40,10 +41,11 @@ namespace game_config
 		               moved_energy_image, unmoved_energy_image, partmoved_energy_image,
 					   enemy_energy_image,ally_energy_image,flag_image,
 					   dot_image,cross_image,
-					   foot_left_nw,foot_left_n,foot_right_nw,foot_right_n,
 					   missile_n_image,missile_ne_image,terrain_mask_image,observer_image,download_campaign_image,
 					   checked_menu_image,unchecked_menu_image;
 
+	extern std::vector<std::string> foot_left_nw,foot_left_n,foot_right_nw,foot_right_n;
+
 	extern int title_logo_x, title_logo_y, title_buttons_x, title_buttons_y, title_buttons_padding, title_tip_x, title_tip_y, title_tip_width, title_tip_padding;
 
 	namespace sounds {
