@@ -1,7 +1,7 @@
-/* $Id: game_events.cpp,v 1.141 2005/04/01 13:41:10 j_daniel Exp $ */
+/* $Id: game_events.cpp,v 1.150 2005/06/09 21:20:13 silene Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -43,6 +43,7 @@
 #define WRN_NG LOG_STREAM(warn, engine)
 #define ERR_NG LOG_STREAM(err, engine)
 #define LOG_DP LOG_STREAM(info, display)
+#define ERR_CF LOG_STREAM(err, config)
 
 namespace {
 
@@ -64,7 +65,7 @@ bool conditional_passed(const std::map<gamemap::location,unit>* units,
 	//an 'or' statement means that if the contained statements are true,
 	//then it automatically passes
 	const vconfig::child_list& or_statements = cond.get_children("or");
-	for(vconfig::child_list::const_iterator or_it = or_statements.begin(); 
+	for(vconfig::child_list::const_iterator or_it = or_statements.begin();
 			or_it != or_statements.end(); ++or_it) {
 		if(conditional_passed(units,*or_it)) {
 			return true;
@@ -173,7 +174,7 @@ std::deque<queued_event> events_queue;
 class event_handler
 {
 public:
-	event_handler(const config& cfg) : 
+	event_handler(const config& cfg) :
 		name_(cfg["name"]),
 		first_time_only_(cfg["first_time_only"] != "no"),
 		disabled_(false),
@@ -200,12 +201,12 @@ public:
 		return cfg_.get_children("filter");
 	}
 
-	const vconfig::child_list second_arg_filters() 
+	const vconfig::child_list second_arg_filters()
 	{
 		return cfg_.get_children("filter_second");
 	}
 
-	bool handle_event(const queued_event& event_info, 
+	bool handle_event(const queued_event& event_info,
 			const vconfig cfg = vconfig());
 
 private:
@@ -234,12 +235,16 @@ std::multimap<std::string,event_handler> events_map;
 
 //this function handles all the different types of actions that can be triggered
 //by an event.
-bool event_handler::handle_event_command(const queued_event& event_info, 
+bool event_handler::handle_event_command(const queued_event& event_info,
 		const std::string& cmd, const vconfig cfg, bool& mutated)
 {
 	log_scope2(engine, "handle_event_command");
 	LOG_NG << "handling command: '" << cmd << "'\n";
 
+	//Ensures the context for the synchronization [result] of this even is
+	//not the same one as the previous event's.
+	add_random_separator();
+
 	bool rval = true;
 	//sub commands that need to be handled in a guaranteed ordering
 	if(cmd == "command") {
@@ -265,19 +270,19 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 				}
 			}
 		}
-		
+
 		screen->invalidate_all();
 	}
 
 
 	//teleport a unit from one location to another
 	else if(cmd == "teleport") {
-		
+
 		unit_map::iterator u = units->find(event_info.loc1);
 
 		//search for a valid unit filter, and if we have one, look for the matching unit
 		const vconfig filter = cfg.child("filter");
-		if(!filter.null()) {	
+		if(!filter.null()) {
 			for(u = units->begin(); u != units->end(); ++u){
 				if(game_events::unit_matches_filter(u, filter))
 					break;
@@ -413,6 +418,12 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		screen->draw(true,true);
 	}
 
+	else if(cmd == "scroll_to") {
+		const int xpos = atoi(cfg["x"].c_str());
+		const int ypos = atoi(cfg["y"].c_str());
+		screen->scroll_to_tile(xpos,ypos);
+	}
+
 	else if(cmd == "scroll_to_unit") {
 		unit_map::const_iterator u;
 		for(u = units->begin(); u != units->end(); ++u){
@@ -503,6 +514,10 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 				if(i==0){
 					src.x = atoi(xvals[i].c_str())-1;
 					src.y = atoi(yvals[i].c_str())-1;
+					if (!game_map->on_board(src)) {
+						ERR_CF << "invalid move_unit_fake source: " << src << '\n';
+						break;
+					}
 					continue;
 				}
 				shortest_path_calculator calc(dummy_unit,
@@ -514,16 +529,21 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 
 				dst.x = atoi(xvals[i].c_str())-1;
 				dst.y = atoi(yvals[i].c_str())-1;
+				if (!game_map->on_board(dst)) {
+					ERR_CF << "invalid move_unit_fake destination: " << dst << '\n';
+					break;
+				}
 
-				paths::route route = a_star_search(src, dst, 10000, &calc, game_map->x(), game_map->y());
-				unit_display::move_unit(*screen, *game_map, route.steps,
-						dummy_unit,status_ptr->get_time_of_day(), *units, *teams);
+				paths::route route = a_star_search(src, dst, 10000, &calc,
+				                                   game_map->x(), game_map->y());
+				unit_display::move_unit(*screen, *game_map, route.steps, dummy_unit,
+				                        status_ptr->get_time_of_day(), *units, *teams);
 
 				src = dst;
 			}
 		}
 	}
- 
+
 	//provide a means of specifying win/loss conditions:
 	// [event]
 	// name=prestart
@@ -561,6 +581,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		const std::string lose_str = "#";
 
 		const t_string& summary = cfg["summary"];
+		const t_string& note = cfg["note"];
 		const size_t side = lexical_cast_default<size_t>(cfg["side"], 0);
 		bool silent = cfg["silent"] == "yes";
 
@@ -570,17 +591,17 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		}
 
 		t_string win_string = cfg["victory_string"];
-		if(win_string.empty()) 
-			win_string = _("Victory:");
+		if(win_string.empty())
+			win_string = t_string(N_("Victory:"), "wesnoth");
 		t_string lose_string = cfg["defeat_string"];
 		if(lose_string.empty())
-			lose_string = _("Defeat:");
+			lose_string = t_string(N_("Defeat:"), "wesnoth");
 
 		t_string win_objectives;
 		t_string lose_objectives;
 
 		const vconfig::child_list objectives = cfg.get_children("objective");
-		for(vconfig::child_list::const_iterator obj_it = objectives.begin(); 
+		for(vconfig::child_list::const_iterator obj_it = objectives.begin();
 				obj_it != objectives.end(); ++obj_it) {
 
 			const t_string description = (*obj_it)["description"];
@@ -603,13 +624,15 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		if(!summary.empty())
 			objs += "*" + summary + "\n";
 		if(!win_objectives.empty()) {
-			objs += win_string + "\n";
+			objs += "*" + win_string + "\n";
 			objs += win_objectives + "\n";
 		}
 		if(!lose_objectives.empty()) {
-			objs += lose_string + "\n";
+			objs += "*" + lose_string + "\n";
 			objs += lose_objectives + "\n";
 		}
+		if(!note.empty())
+			objs += note + "\n";
 
 		if(side == 0) {
 			for(std::vector<team>::iterator itor = teams->begin();
@@ -668,7 +691,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 
 		// random generation works as follows:
 		// random=[comma delimited list]
-		// Each element in the list will be considered a separate choice, 
+		// Each element in the list will be considered a separate choice,
 		// unless it contains "..". In this case, it must be a numerical
 		// range. (i.e. -1..-10, 0..100, -10..10, etc)
 		const std::string& random = cfg["random"];
@@ -677,20 +700,20 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			std::vector<std::string> words;
 			std::vector<std::pair<size_t,size_t> > ranges;
 			int num_choices = 0;
-			std::string::size_type pos = 0, pos2 = std::string::npos, tmp; 
+			std::string::size_type pos = 0, pos2 = std::string::npos, tmp;
 			std::stringstream ss(std::stringstream::in|std::stringstream::out);
 			while (pos2 != random.length()) {
 				pos = pos2+1;
 				pos2 = random.find(",", pos);
 
-				if (pos2 == std::string::npos) 
+				if (pos2 == std::string::npos)
 					pos2 = random.length();
 
 				word = random.substr(pos, pos2-pos);
 				words.push_back(word);
 				tmp = word.find("..");
-				
-				
+
+
 				if (tmp == std::string::npos) {
 					// treat this element as a string
 					ranges.push_back(std::pair<int, int>(0,0));
@@ -719,7 +742,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			}
 
 			size_t choice = get_random() % num_choices;
-			tmp = 0;	
+			tmp = 0;
 			for(size_t i = 0; i < ranges.size(); i++) {
 				tmp += (ranges[i].second - ranges[i].first) + 1;
 				if (tmp > choice) {
@@ -895,6 +918,16 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			const std::string& terrain_type = cfg["letter"];
 			preferences::encountered_terrains().insert(terrain_type);
 			if(terrain_type.size() > 0) {
+				const bool old_village = game_map->is_village(*loc);
+				const bool new_village = game_map->is_village(terrain_type[0]);
+
+				if(old_village && !new_village) {
+					int owner = village_owner(*loc, *teams);
+					if(owner != -1) {
+						(*teams)[owner].lose_village(*loc);
+					}
+				}
+
 				game_map->set_terrain(*loc,terrain_type[0]);
 			}
 		}
@@ -943,12 +976,6 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			}
 
 			screen->invalidate(loc);
-			// need to redraw here because unit might be in a prestart event
-			for(std::vector<team>::iterator t = teams->begin(); t != teams->end(); ++t) {
-				clear_shroud(*screen, *status_ptr, *game_map, *game_data_ptr, *units, *teams, (t - teams->begin()));
-			}
-			screen->draw();
-			screen->recalculate_minimap();
 		} else {
 			player_info* const player = state_of_game->get_player((*teams)[new_unit.side()-1].save_id());
 
@@ -1085,7 +1112,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 					break;
 			}
 		}
-	
+
 		if(speaker == units->end() && cfg["speaker"] != "narrator") {
 			//no matching unit found, so the dialog can't come up
 			//continue onto the next message
@@ -1144,7 +1171,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		int option_chosen = -1;
 
 		LOG_DP << "showing dialog...\n";
-		
+
 		//if we're not replaying, or if we are replaying and there is no choice
 		//to be made, show the dialog.
 		if(get_replay_source().at_end() || options.empty()) {
@@ -1183,7 +1210,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		//implement the consequences of the choice
 		if(options.empty() == false) {
 			wassert(size_t(option_chosen) < menu_items.size());
-			
+
 			vconfig::child_list events = option_events[option_chosen];
 			for(vconfig::child_list::const_iterator itor = events.begin();
 					itor != events.end(); ++itor) {
@@ -1442,7 +1469,7 @@ bool event_handler::handle_event(const queued_event& event_info, const vconfig c
 
 		const std::pair<const std::string*,const config*> item = *i;
 
-		// If the user pressed escape, we skip any message that doesn't 
+		// If the user pressed escape, we skip any message that doesn't
 		// require them to make a choice.
 		if ((skip_messages) && (*item.first == "message")) {
 			if ((item.second)->get_children("option").size() == 0) {
@@ -1453,7 +1480,7 @@ bool event_handler::handle_event(const queued_event& event_info, const vconfig c
 		if (!handle_event_command(event_info, *item.first, vconfig(item.second), mutated)) {
 			skip_messages = true;
 		}
-		else { 
+		else {
 			skip_messages = false;
 		}
 	}
