@@ -1,7 +1,6 @@
 #include "global.hpp"
 
 #include "gettext.hpp"
-#include "serialization/string_utils.hpp"
 
 #include <cstring>
 
@@ -36,10 +35,3 @@ const char* dsgettext (const char * domainname, const char *msgid)
 	}
 	return msgval;
 }
-
-std::string vgettext (const char *msgid, const utils::string_map& symbols)
-{
-	const std::string orig(gettext(msgid));
-	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
-	return msg;
-}
