@@ -1,6 +1,6 @@
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -22,6 +22,7 @@
 #include "help.hpp"
 #include "image.hpp"
 #include "language.hpp"
+#include "log.hpp"
 #include "preferences.hpp"
 #include "sdl_utils.hpp"
 #include "show_dialog.hpp"
@@ -57,7 +58,7 @@ struct section;
 typedef std::vector<section *> section_list;
 
 /// Generate a topic text on the fly.
-class topic_generator 
+class topic_generator
 {
 	unsigned count;
 	friend class topic_text;
@@ -76,7 +77,7 @@ public:
 
 /// The text displayed in a topic. It is generated on the fly with the information
 /// contained in generator_.
-class topic_text 
+class topic_text
 {
 	mutable std::vector< std::string > parsed_text_;
 	mutable topic_generator *generator_;
@@ -123,10 +124,10 @@ struct section {
 	bool operator==(const section &) const;
 	/// Comparison on the ID.
 	bool operator<(const section &) const;
-	
+
 	/// Allocate memory for and add the section.
 	void add_section(const section &s);
-	
+
 	void clear();
 	std::string title, id;
 	topic_list topics;
@@ -136,7 +137,7 @@ struct section {
 
 /// To be used as a function object to locate sections and topics
 /// with a specified ID.
-class has_id 
+class has_id
 {
 public:
 	has_id(const std::string &id) : id_(id) {}
@@ -148,26 +149,26 @@ private:
 };
 
 /// To be used as a function object when sorting topic lists on the title.
-class title_less 
+class title_less
 {
 public:
 	bool operator()(const topic &t1, const topic &t2) { return t1.title < t2.title; }
 };
 
-struct delete_section 
+struct delete_section
 {
 	void operator()(section *s) { delete s; }
 };
 
-struct create_section 
+struct create_section
 {
 	section *operator()(const section *s) { return new section(*s); }
 	section *operator()(const section &s) { return new section(s); }
 };
-		
+
 /// The menu to the left in the help browser, where topics can be
 /// navigated through and chosen.
-class help_menu : public gui::menu 
+class help_menu : public gui::menu
 {
 public:
 	help_menu(CVideo &video, const section &toplevel, int max_height=-1);
@@ -200,7 +201,7 @@ private:
 	/// Regenerate what items are visible by checking what sections are
 	/// expanded.
 	void update_visible_items(const section &top_level, unsigned starting_level=0);
-	
+
 	/// Return true if the section is expanded.
 	bool expanded(const section &sec);
 
@@ -221,14 +222,14 @@ private:
 
 	/// Draw the currently visible items.
 	void display_visible_items();
-	
+
 	/// Internal recursive thingie. did_expand will be true if any
 	/// section was expanded, otherwise untouched.
 	bool select_topic_internal(const topic &t, const section &sec);
 
 	std::vector<visible_item> visible_items_;
 	const section &toplevel_;
-	std::set<const section*> expanded_; 
+	std::set<const section*> expanded_;
 	surface_restorer restorer_;
 	SDL_Rect rect_;
 	topic const *chosen_topic_;
@@ -236,14 +237,14 @@ private:
 };
 
 /// Thrown when the help system fails to parse something.
-struct parse_error 
+struct parse_error
 {
 	parse_error(const std::string& msg) : message(msg) {}
 	std::string message;
 };
 
 /// The area where the content is shown in the help browser.
-class help_text_area : public gui::scrollarea 
+class help_text_area : public gui::scrollarea
 {
 public:
 	help_text_area(CVideo &video, const section &toplevel);
@@ -282,11 +283,11 @@ private:
 		surface surf;
 
 		// If this item contains text, this will contain that text.
-		std::string text; 
+		std::string text;
 
 		// If this item contains a cross-reference, this is the id
 		// of the referenced topic.
-		std::string ref_to; 
+		std::string ref_to;
 
 		// If this item is floating, that is, if things should be filled
 		// around it.
@@ -294,7 +295,7 @@ private:
 		bool box;
 		ALIGNMENT align;
 	};
-	
+
 	/// Function object to find an item at the specified coordinates.
 	class item_at {
 	public:
@@ -343,7 +344,7 @@ private:
 
 	/// Return the width that remain on the line the current input point is at.
 	int get_remaining_width();
-	
+
 	/// Return the least x coordinate at which something of the
 	/// specified height can be drawn at the specified y coordinate
 	/// without interfering with floating images.
@@ -436,7 +437,7 @@ std::vector<topic> generate_terrains_topics();
 
 /// Parse a help config, return the top level section. Return an empty
 /// section if cfg is NULL.
-section parse_config(const config *cfg); 
+section parse_config(const config *cfg);
 /// Recursive function used by parse_config.
 void parse_config_internal(const config *help_cfg, const config *section_cfg,
 						   section &sec, int level=0);
@@ -483,9 +484,6 @@ std::string remove_first_space(const std::string& text);
 /// Return a lowercase copy of s.
 std::string to_lower(const std::string &s);
 
-/// Return a copy of s with the first letter capitalized.
-std::string cap(const std::string &s);
-
 /// Prepend all chars with meaning inside attributes with a backslash.
 std::string escape(const std::string &s);
 
@@ -500,13 +498,13 @@ namespace {
 	const game_data *game_info = NULL;
 	gamemap *map = NULL;
 	// The default toplevel.
-	help::section toplevel; 
+	help::section toplevel;
 	// All sections and topics not referenced from the default toplevel.
-	help::section hidden_sections; 
+	help::section hidden_sections;
 
 	int last_num_encountered_units = -1;
 	int last_num_encountered_terrains = -1;
-								   
+
 	config dummy_cfg;
 	std::vector<std::string> empty_string_vector;
 	const int max_section_level = 15;
@@ -520,8 +518,8 @@ namespace {
 	const std::string closed_section_img = "help/closed_section.png";
 	const std::string open_section_img = "help/open_section.png";
 	// The topic to open by default when opening the help dialog.
-	const std::string default_show_topic = "introduction_topic"; 
-	
+	const std::string default_show_topic = "introduction_topic";
+
 	/// Return true if the id is valid for user defined topics and
 	/// sections. Some IDs are special, such as toplevel and may not be
 	/// be defined in the config.
@@ -549,42 +547,15 @@ namespace {
 	/// for the help dialog.
 	class about_text_formatter {
 	public:
-		about_text_formatter() : text_started_(false) {}
 		std::string operator()(const std::string &s) {
-			std::string res = s;
-			if (res.size() > 0) {
-				bool header = false;
-				// Format + as headers, and the rest as normal text.
-				if (res[0] == '+') {
-					header = true;
-					res.erase(res.begin());
-				}
-				else if (res[0] == '-') {
-					res.erase(res.begin());
-				}
-				// There is a bunch of empty rows in the start in about.cpp,
-				// we do not want to show these here. Thus, if we still
-				// encounter one of those, return an empty string that will
-				// be removed totally at a later stage.
-				if (!text_started_ && res.find_first_not_of(' ') != std::string::npos) {
-					text_started_ = true;
-				}
-				if (text_started_) {
-					std::stringstream ss;
-					if (header) {
-						ss << "<header>text='" << help::escape(res) << "'</header>";
-						res = ss.str();
-					}
-					text_started_ = false;
-				}
-				else {
-					res = " ";
-				}
-			}
-			return res;
+			if (s.empty()) return s;
+			// Format + as headers, and the rest as normal text.
+			if (s[0] == '+')
+				return " \n<header>text='" + help::escape(s.substr(1)) + "'</header>";
+			if (s[0] == '-')
+				return s.substr(1);
+			return s;
 		}
-	private:
-		bool text_started_;
 	};
 
 	// Helpers for making generation of topics easier.
@@ -601,7 +572,7 @@ namespace {
 		ss << "<jump>amount=" << amount << "</jump>";
 		return ss.str();
 	}
-	
+
 	std::string bold(const std::string &s)
 	{
 		std::stringstream ss;
@@ -614,7 +585,7 @@ namespace {
 	// that create a table. The table spec contains a vector with
 	// vectors with pairs. The pairs are the markup string that should
 	// be in a cell, and the width of that cell.
-  std::string generate_table(const table_spec &tab, const unsigned int spacing=font::relative_size(20)) 
+  std::string generate_table(const table_spec &tab, const unsigned int spacing=font::relative_size(20))
   {
 		table_spec::const_iterator row_it;
 		std::vector<std::pair<std::string, unsigned> >::const_iterator col_it;
@@ -657,7 +628,7 @@ namespace {
 	}
 
 	// Return the width for the image with filename.
-	unsigned image_width(const std::string &filename) 
+	unsigned image_width(const std::string &filename)
 	{
 		image::locator loc(filename);
 		surface surf(image::get_image(loc, image::UNSCALED));
@@ -804,7 +775,7 @@ bool topic_is_referenced(const std::string &topic_id, const config &cfg)
 	}
 	return false;
 }
-	
+
 void parse_config_internal(const config *help_cfg, const config *section_cfg,
 						   section &sec, int level)
 {
@@ -872,9 +843,9 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 		std::copy(generated_topics.begin(), generated_topics.end(),
 				  std::back_inserter(sec.topics));
 	}
-}	
+}
 
-section parse_config(const config *cfg) 
+section parse_config(const config *cfg)
 {
 	section sec;
 	if (cfg != NULL) {
@@ -885,7 +856,7 @@ section parse_config(const config *cfg)
 }
 
 
-std::vector<section> generate_sections(const std::string &generator) 
+std::vector<section> generate_sections(const std::string &generator)
 {
 	std::vector<section> empty_vec;
 	if (generator == "") {
@@ -929,7 +900,7 @@ std::string generate_topic_text(const std::string &generator)
 	return empty_string;
 }
 
-topic_text::~topic_text() 
+topic_text::~topic_text()
 {
 	if (generator_ && --generator_->count == 0)
 		delete generator_;
@@ -949,7 +920,7 @@ topic_text &topic_text::operator=(topic_generator *g)
 	return *this;
 }
 
-topic_text::operator std::vector< std::string > const &() const 
+topic_text::operator std::vector< std::string > const &() const
 {
 	if (generator_) {
 		parsed_text_ = parse_text((*generator_)());
@@ -976,17 +947,21 @@ std::vector<topic> generate_weapon_special_topics()
 			std::vector<attack_type> attacks = type.attacks();
 			for (std::vector<attack_type>::const_iterator it = attacks.begin();
 				 it != attacks.end(); it++) {
-				const std::string special = (*it).special();
+				std::string special = (*it).special();
+
+				//some abilities like plague can be in the form ability(argument)
+				//make sure we cut off the argument
+				special.erase(std::find(special.begin(),special.end(),'('),special.end());
 				if (special != "") {
 					if (checked_specials.find(special) == checked_specials.end()) {
 						std::string lang_special = gettext(special.c_str());
-						lang_special = cap(lang_special);
+						lang_special = utils::capitalize(lang_special);
 						std::string description
 							= string_table["weapon_special_" + special + "_description"];
 						const size_t colon_pos = description.find(':');
 						if (colon_pos != std::string::npos) {
 							// Remove the first colon and the following newline.
-							description.erase(0, colon_pos + 2); 
+							description.erase(0, colon_pos + 2);
 						}
 						topic t(lang_special, "weaponspecial_" + special, description);
 						topics.push_back(t);
@@ -1018,12 +993,12 @@ std::vector<topic> generate_ability_topics()
 				 it != type.abilities().end(); it++) {
 				if (checked_abilities.find(*it) == checked_abilities.end()) {
 					const std::string id = "ability_" + *it;
-					std::string lang_ability = cap(string_table[id]);
+					std::string lang_ability = utils::capitalize(string_table[id]);
 					std::string description = string_table[*it + "_description"];
 					const size_t colon_pos = description.find(':');
 					if (colon_pos != std::string::npos) {
 						// Remove the first colon and the following newline.
-						description.erase(0, colon_pos + 2); 
+						description.erase(0, colon_pos + 2);
 					}
 					topic t(lang_ability, id, description);
 					topics.push_back(t);
@@ -1031,7 +1006,7 @@ std::vector<topic> generate_ability_topics()
 				}
 			}
 		}
-		
+
 	}
 	return topics;
 }
@@ -1039,6 +1014,10 @@ std::vector<topic> generate_ability_topics()
 class unit_topic_generator: public topic_generator
 {
 	unit_type type_;
+	typedef std::pair< std::string, unsigned > item;
+	void push_header(std::vector< item > &row, char const *name) const {
+		row.push_back(item(bold(name), font::line_width(name, normal_font_size, TTF_STYLE_BOLD)));
+	}
 public:
 	unit_topic_generator(unit_type const &t): type_(t) {}
 	virtual std::string operator()() const {
@@ -1053,8 +1032,22 @@ public:
 
 		if (&female_type != &male_type)
 			ss << "<img>src='" << female_type.image() << "'</img> ";
+
 		ss << "<format>font_size=" << font::relative_size(11) << " text=' " << escape(_("level"))
-		   << " " << type_.level() << "'</format>\n";
+		   << " " << type_.level() << "'</format>";
+
+		const std::string& male_portrait = male_type.image_profile();
+		const std::string& female_portrait = female_type.image_profile();
+
+		if (male_portrait.empty() == false && male_portrait != male_type.image()) {
+			ss << "<img>src='" << male_portrait << "' align='right'</img> ";
+		}
+
+		if (female_portrait.empty() == false && female_portrait != male_portrait && female_portrait != female_type.image()) {
+			ss << "<img>src='" << female_portrait << "' align='right'</img> ";
+		}
+
+		ss << "\n";
 
 		// Print the units this unit can advance to. Cross reference
 		// to the topics containing information about those units.
@@ -1066,7 +1059,7 @@ public:
 				 advance_it != advance_end; ++advance_it) {
 				std::string unit_id = *advance_it;
 				std::map<std::string,unit_type>::const_iterator new_type = game_info->unit_types.find(unit_id);
-			 	if(new_type != game_info->unit_types.end()) {
+				if(new_type != game_info->unit_types.end()) {
 					std::string lang_unit = new_type->second.language_name();
 					std::string ref_id = std::string("unit_") + new_type->second.id();
 					ss << "<ref>dst='" << escape(ref_id) << "' text='" << escape(lang_unit)
@@ -1109,43 +1102,22 @@ public:
 		// Print the detailed description about the unit.
 		ss << "\n\n" << detailed_description;
 
-		typedef std::pair<std::string,unsigned int> item;
-
 		// Print the different attacks a unit has, if it has any.
 		std::vector<attack_type> attacks = type_.attacks();
 		if (!attacks.empty()) {
 			// Print headers for the table.
-			ss << "\n\n<header>text='" << escape(cap(_("attacks")))
+			ss << "\n\n<header>text='" << escape(utils::capitalize(_("attacks")))
 			   << "'</header>\n\n";
 			table_spec table;
 
 			std::vector<item> first_row;
 			// Dummy element, icons are below.
 			first_row.push_back(item("", 0));
-			first_row.push_back(item(bold(_("Name")),
-						 font::line_width(cap(_("Name")),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
-			first_row.push_back(item(bold(_("Type")),
-						 font::line_width(_("Type"),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
-			first_row.push_back(item(bold(_("Dmg")),
-						 font::line_width(_("Dmg"),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
-			first_row.push_back(item(bold(_("Strikes")),
-						 font::line_width(_("Strikes"),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
-			first_row.push_back(item(bold(_("Range")),
-						 font::line_width(_("Range"),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
-			first_row.push_back(item(bold(_("Special")),
-						 font::line_width(_("Special"),
-								  normal_font_size,
-								  TTF_STYLE_BOLD)));
+			push_header(first_row, _("Name"));
+			push_header(first_row, _("Type"));
+			push_header(first_row, _("Strikes"));
+			push_header(first_row, _("Range"));
+			push_header(first_row, _("Special"));
 			table.push_back(first_row);
 			// Print information about every attack.
 			for(std::vector<attack_type>::const_iterator attack_it = attacks.begin(),
@@ -1158,13 +1130,10 @@ public:
 				attack_ss << "<img>src='" << (*attack_it).icon() << "'</img>";
 				row.push_back(std::make_pair(attack_ss.str(),
 							     image_width(attack_it->icon())));
-				attack_ss.str(clear_stringstream);
 				push_tab_pair(row, lang_weapon);
 				push_tab_pair(row, lang_type);
-				attack_ss << attack_it->damage();
-				push_tab_pair(row, attack_ss.str());
 				attack_ss.str(clear_stringstream);
-				attack_ss << attack_it->num_attacks();
+				attack_ss << attack_it->damage() << '-' << attack_it->num_attacks();
 				push_tab_pair(row, attack_ss.str());
 				attack_ss.str(clear_stringstream);
 				push_tab_pair(row, (*attack_it).range() == attack_type::SHORT_RANGE ?
@@ -1192,14 +1161,8 @@ public:
 		   << "'</header>\n\n";
 		table_spec resistance_table;
 		std::vector<item> first_res_row;
-		first_res_row.push_back(std::make_pair(bold(_("Attack Type")),
-						       font::line_width(_("Attack Type"),
-									normal_font_size,
-									TTF_STYLE_BOLD)));
-		first_res_row.push_back(std::make_pair(bold(_("Resistance")),
-						       font::line_width(_("Resistance"),
-									normal_font_size,
-									TTF_STYLE_BOLD)));
+		push_header(first_res_row, _("Attack Type"));
+		push_header(first_res_row, _("Resistance"));
 		resistance_table.push_back(first_res_row);
 		const unit_movement_type &movement_type = type_.movement_type();
 		string_map dam_tab = movement_type.damage_table();
@@ -1228,18 +1191,10 @@ public:
 			   << "'</header>\n\n";
 			std::vector<item> first_row;
 			table_spec table;
-			first_row.push_back(std::make_pair(bold(_("Terrain")),
-								   font::line_width(_("Terrain"),
-										    normal_font_size,
-										    TTF_STYLE_BOLD)));
-			first_row.push_back(std::make_pair(bold(_("Movement")),
-							   font::line_width(_("Movement"),
-									    normal_font_size,
-									    TTF_STYLE_BOLD)));
-			first_row.push_back(std::make_pair(bold(_("Defense")),
-							   font::line_width(_("Defense"),
-									    normal_font_size,
-									    TTF_STYLE_BOLD)));
+			push_header(first_row, _("Terrain"));
+			push_header(first_row, _("Defense"));
+			push_header(first_row, _("Movement"));
+
 			table.push_back(first_row);
 			for (std::set<std::string>::const_iterator terrain_it =
 					 preferences::encountered_terrains().begin(),
@@ -1257,20 +1212,25 @@ public:
 					std::stringstream str;
 					str << "<ref>text='" << escape(name) << "' dst='"
 					    << escape(std::string("terrain_") + terrain) << "'</ref>";
-					row.push_back(std::make_pair(str.str(), 
+					row.push_back(std::make_pair(str.str(),
 								     font::line_width(name,
 										      normal_font_size)));
+
+					//defense
+					str.str(clear_stringstream);
+					const int defense =
+						100 - movement_type.defense_modifier(*map,terrain);
+					str << defense << "%";
+					push_tab_pair(row, str.str());
+
+					//movement
 					str.str(clear_stringstream);
 					if(moves < 100)
 						str << moves;
 					else
 						str << "--";
 					push_tab_pair(row, str.str());
-					str.str(clear_stringstream);
-					const int defense =
-						100 - movement_type.defense_modifier(*map,terrain);
-					str << defense << "%";
-					push_tab_pair(row, str.str());
+
 					table.push_back(row);
 				}
 			}
@@ -1359,7 +1319,7 @@ struct terrain_topic_generator: topic_generator
 	}
 };
 
-std::vector<topic> generate_terrains_topics() 
+std::vector<topic> generate_terrains_topics()
 {
 	std::vector<topic> res;
 	std::vector<gamemap::TERRAIN> show_info_about;
@@ -1392,7 +1352,7 @@ std::vector<topic> generate_terrains_topics()
 
 std::string generate_traits_text()
 {
-	// Ok, this didn't go as well as I thought since the information
+	// OK, this didn't go as well as I thought since the information
 	// generated from this is rather short and not suitable for the help
 	// system. Hence, this method is not used currently :).
 	std::stringstream ss;
@@ -1429,10 +1389,8 @@ std::string generate_about_text()
 	std::vector<std::string> res_lines;
 	std::transform(about_lines.begin(), about_lines.end(), std::back_inserter(res_lines),
 				   about_text_formatter());
-	std::vector<std::string>::iterator it =
-		std::remove(res_lines.begin(), res_lines.end(), "");
-	std::vector<std::string> res_lines_rem(res_lines.begin(), it);
-	std::string text = utils::join(res_lines_rem, '\n');
+	res_lines.erase(std::remove(res_lines.begin(), res_lines.end(), ""), res_lines.end());
+	std::string text = utils::join(res_lines, '\n');
 	return text;
 }
 
@@ -1460,8 +1418,8 @@ section::~section()
 	std::for_each(sections.begin(), sections.end(), delete_section());
 }
 
-section::section(const section &sec) 
-	: title(sec.title), id(sec.id), topics(sec.topics) 
+section::section(const section &sec)
+	: title(sec.title), id(sec.id), topics(sec.topics)
 {
 	std::transform(sec.sections.begin(), sec.sections.end(),
 				   std::back_inserter(sections), create_section());
@@ -1476,7 +1434,7 @@ section& section::operator=(const section &sec)
 				   std::back_inserter(sections), create_section());
 	return *this;
 }
-	
+
 
 bool section::operator==(const section &sec) const
 {
@@ -1488,7 +1446,7 @@ bool section::operator<(const section &sec) const
 	return id < sec.id;
 }
 
-void section::add_section(const section &s) 
+void section::add_section(const section &s)
 {
 	sections.push_back(new section(s));
 }
@@ -1502,7 +1460,7 @@ void section::clear()
 
 help_menu::help_menu(CVideo &video, section const &toplevel, int max_height)
 	: gui::menu(video, empty_string_vector, true, max_height),
-	  toplevel_(toplevel), chosen_topic_(NULL), selected_item_(&toplevel, "") 
+	  toplevel_(toplevel), chosen_topic_(NULL), selected_item_(&toplevel, "")
 {
 	update_visible_items(toplevel_);
 	display_visible_items();
@@ -1515,19 +1473,19 @@ bool help_menu::expanded(const section &sec)
 	return expanded_.find(&sec) != expanded_.end();
 }
 
-void help_menu::expand(const section &sec) 
+void help_menu::expand(const section &sec)
 {
 	if (sec.id != "toplevel") {
 		expanded_.insert(&sec);
 	}
 }
 
-void help_menu::contract(const section &sec) 
+void help_menu::contract(const section &sec)
 {
 	expanded_.erase(&sec);
 }
 
-void help_menu::update_visible_items(const section &sec, unsigned level) 
+void help_menu::update_visible_items(const section &sec, unsigned level)
 {
 	if (level == 0) {
 		// Clear if this is the top level, otherwise append items.
@@ -1549,7 +1507,7 @@ void help_menu::update_visible_items(const section &sec, unsigned level)
 }
 
 
-std::string help_menu::get_string_to_show(const section &sec, const unsigned level) 
+std::string help_menu::get_string_to_show(const section &sec, const unsigned level)
 {
 	std::stringstream to_show;
 	std::string pad_string;
@@ -1637,7 +1595,7 @@ const topic *help_menu::chosen_topic()
 	chosen_topic_ = NULL;
 	return ret;
 }
-	
+
 void help_menu::display_visible_items()
 {
 	std::vector<std::string> menu_items;
@@ -1662,7 +1620,7 @@ bool help_menu::visible_item::operator==(const section &_sec) const
 	return sec != NULL && *sec == _sec;
 }
 
-bool help_menu::visible_item::operator==(const topic &_t) const 
+bool help_menu::visible_item::operator==(const topic &_t) const
 {
 	return t != NULL && *t == _t;
 }
@@ -1688,14 +1646,14 @@ void help_text_area::set_inner_location(SDL_Rect const &rect)
 		set_items();
 }
 
-void help_text_area::show_topic(const topic &t) 
+void help_text_area::show_topic(const topic &t)
 {
 	shown_topic_ = &t;
 	set_items();
 	set_dirty(true);
 }
 
-	
+
 help_text_area::item::item(surface surface, int x, int y, const std::string _text,
 						   const std::string reference_to, bool _floating,
 						   bool _box, ALIGNMENT alignment)
@@ -1937,7 +1895,7 @@ void help_text_area::handle_format_cfg(const config &cfg)
 
 void help_text_area::add_text_item(const std::string text, const std::string ref_dst,
 								   int _font_size, bool bold, bool italic,
-								   SDL_Color text_color) 
+								   SDL_Color text_color)
 {
 	const int font_size = _font_size < 0 ? normal_font_size : _font_size;
 	if (text.empty())
@@ -1974,20 +1932,11 @@ void help_text_area::add_text_item(const std::string text, const std::string ref
 		if (!surf.null())
 			add_item(item(surf, curr_loc_.first, curr_loc_.second, first_part, ref_dst));
 		if (parts.size() > 1) {
-			// Parts remain, remove the first part from the string and
-			// add the remaining parts.
-			std::string s = text;
-			s.erase(0, first_part.size());
-			if (s.length() < 1) {
-				return;
-			}
+
+			std::string& s = parts.back();
+
 			const std::string first_word_before = get_first_word(s);
 			const std::string first_word_after = get_first_word(remove_first_space(s));
-			//std::cout << "before: '" << first_word_before << "'\n"
-			//		  << "after: '" << first_word_after << "'\n"
-			//		  << "before linewidth: " << font::line_width(first_word_before, font_size)
-			//		  << "\nafter linewidth: " << font::line_width(first_word_after, font_size)
-			//		  << "\nremaining width: " << get_remaining_width() << std::endl;
 			if (get_remaining_width() >= font::line_width(first_word_after, font_size, state)
 				&& get_remaining_width()
 				< font::line_width(first_word_before, font_size, state)) {
@@ -2002,7 +1951,7 @@ void help_text_area::add_text_item(const std::string text, const std::string ref
 				s = remove_first_space(s);
 			}
 			add_text_item(s, ref_dst, _font_size, bold, italic, text_color);
-				
+
 		}
 	}
 }
@@ -2011,15 +1960,11 @@ void help_text_area::add_img_item(const std::string path, const std::string alig
 								  const bool floating, const bool box)
 {
 	surface surf(image::get_image(path, image::UNSCALED));
-	if (surf == NULL) {
-		std::stringstream msg;
-		msg << "Image " << path << " could not be loaded.";
-		std::cerr << msg.str();
+	if (surf.null())
 		return;
-	}
 	ALIGNMENT align = str_to_align(alignment);
 	if (align == HERE && floating) {
-		std::cerr << "Floating image with align HERE, aligning left." << std::endl;
+		LOG_STREAM(warn, display) << "Floating image with align HERE, aligning left.\n";
 		align = LEFT;
 	}
 	const int width = surf->w + (box ? box_width * 2 : 0);
@@ -2121,8 +2066,8 @@ void help_text_area::add_item(const item &itm)
 		contents_height_ = maximum<int>(contents_height_, itm.rect.y + itm.rect.h);
 	}
 }
-	
-	
+
+
 help_text_area::ALIGNMENT help_text_area::str_to_align(const std::string &s)
 {
 	const std::string cmp_str = to_lower(s);
@@ -2139,7 +2084,7 @@ help_text_area::ALIGNMENT help_text_area::str_to_align(const std::string &s)
 	msg << "Invalid alignment string: '" << s << "'";
 	throw parse_error(msg.str());
 }
-	
+
 void help_text_area::down_one_line()
 {
 	adjust_last_row();
@@ -2227,7 +2172,7 @@ help_browser::help_browser(display &disp, const section &toplevel)
 	  text_area_(disp.video(), toplevel), toplevel_(toplevel), ref_cursor_(false),
 	  back_button_(disp.video(), _("< Back"), gui::button::TYPE_PRESS),
 	  forward_button_(disp.video(), _("Forward >"), gui::button::TYPE_PRESS),
-	  shown_topic_(NULL) 
+	  shown_topic_(NULL)
 {
 	// Hide the buttons at first since we do not have any forward or
 	// back topics at this point. They will be unhidden when history
@@ -2238,14 +2183,18 @@ help_browser::help_browser(display &disp, const section &toplevel)
 	set_measurements(font::relative_size(400), font::relative_size(500));
 }
 
-void help_browser::adjust_layout() 
+void help_browser::adjust_layout()
 {
   const int menu_buttons_padding = font::relative_size(10);
 	const int menu_y = location().y;
 	const int menu_x = location().x;
-	const int menu_w = font::relative_size(250);
+#ifdef USE_TINY_GUI
+	const int menu_w = 120;
+#else
+	const int menu_w = 250;
+#endif
 	const int menu_h = height() - back_button_.height() - menu_buttons_padding;
-	
+
 	const int menu_text_area_padding = font::relative_size(10);
 	const int text_area_y = location().y;
 	const int text_area_x = menu_x + menu_w + menu_text_area_padding;
@@ -2305,7 +2254,7 @@ void help_browser::process_event()
 }
 
 void help_browser::move_in_history(std::deque<const topic *> &from,
-		std::deque<const topic *> &to) 
+		std::deque<const topic *> &to)
 {
 	if (!from.empty()) {
 		const topic *to_show = from.back();
@@ -2334,7 +2283,7 @@ void help_browser::handle_event(const SDL_Event &event)
 				const topic *t = find_topic(toplevel_, ref);
 				if (t == NULL) {
 					std::stringstream msg;
-					msg << "Reference to unknown topic: '" << ref << "'.";
+					msg << _("Reference to unknown topic: ") << "'" << ref << "'.";
 					gui::show_dialog(disp_, NULL, "", msg.str(), gui::OK_ONLY);
 					update_cursor();
 				}
@@ -2344,7 +2293,7 @@ void help_browser::handle_event(const SDL_Event &event)
 				}
 			}
 		}
-	}	
+	}
 	else if (event.type == SDL_MOUSEMOTION) {
 		update_cursor();
 	}
@@ -2383,7 +2332,7 @@ const topic *find_topic(const section &sec, const std::string &id)
 	return NULL;
 }
 
-const section *find_section(const section &sec, const std::string &id) 
+const section *find_section(const section &sec, const std::string &id)
 {
 	section_list::const_iterator sit =
 		std::find_if(sec.sections.begin(), sec.sections.end(), has_id(id));
@@ -2414,6 +2363,7 @@ void help_browser::show_topic(const std::string &topic_id)
 
 void help_browser::show_topic(const topic &t, bool save_in_history)
 {
+	log_scope("show_topic");
 	if (save_in_history) {
 		forward_topics_.clear();
 		if (shown_topic_ != NULL) {
@@ -2520,6 +2470,7 @@ std::string convert_to_wml(const std::string &element_name, const std::string &c
 		}
 		else {
 			if (c == '\'' && !last_char_escape) {
+				ss << '"';
 				in_quotes = !in_quotes;
 			}
 			else if ((c == ' ' || c == '\n') && !last_char_escape && !in_quotes) {
@@ -2561,7 +2512,7 @@ bool get_bool(const std::string &s)
 	return false;
 }
 
-SDL_Color string_to_color(const std::string &s) 
+SDL_Color string_to_color(const std::string &s)
 {
 	const std::string cmp_str = to_lower(s);
 	if (cmp_str == "green") {
@@ -2582,9 +2533,14 @@ SDL_Color string_to_color(const std::string &s)
 std::vector<std::string> split_in_width(const std::string &s, const int font_size,
 		const unsigned width)
 {
-	std::string wrapped = font::word_wrap_text(s, font_size, width);
-	std::vector<std::string> parts = utils::split(wrapped, '\n', 0);
-	return parts;
+	std::vector<std::string> res;
+	const std::string& first_line = font::word_wrap_text(s, font_size, width, -1, 1);
+	res.push_back(first_line);
+	if(s.size() > first_line.size()) {
+		res.push_back(s.substr(first_line.size()));
+	}
+
+	return res;
 }
 
 std::string remove_first_space(const std::string& text)
@@ -2603,17 +2559,6 @@ std::string to_lower(const std::string &s)
 	return lower_string;
 }
 
-std::string cap(const std::string &s)
-{
-	if (s.size() > 0) {
-		utils::utf8_iterator itor(s);
-		std::string res = utils::wchar_to_string(towupper(*itor));
-		res.append(itor.substr().second, s.end());
-		return res;
-	}
-	return s;
-}
-	
 std::string escape(const std::string &s)
 {
 	std::string res = s;
@@ -2629,8 +2574,8 @@ std::string escape(const std::string &s)
 	}
 	return res;
 }
-		
-std::string get_first_word(const std::string &s) 
+
+std::string get_first_word(const std::string &s)
 {
 	if (s == "") {
 		return s;
@@ -2707,7 +2652,7 @@ void show_help(display &disp, const section &toplevel_sec, const std::string sho
 	// of the screen.
 	if (yloc <= -1 || xloc <= -1) {
 		xloc = scr->w / 2 - width / 2;
-		yloc = scr->h / 2 - height / 2; 
+		yloc = scr->h / 2 - height / 2;
 	}
 	std::vector<gui::button*> buttons_ptr;
 	gui::button close_button_(disp.video(), _("Close"));
@@ -2760,7 +2705,7 @@ void show_help(display &disp, const section &toplevel_sec, const std::string sho
 	}
 	catch (parse_error e) {
 		std::stringstream msg;
-		msg << "Parse error when parsing help text: '" << e.message << "'";
+		msg << _("Parse error when parsing help text: ") << "'" << e.message << "'";
 		gui::show_dialog(disp, NULL, "", msg.str(), gui::OK_ONLY);
 	}
 }
