@@ -1,7 +1,7 @@
-/* $Id: hotkeys.cpp,v 1.70 2005/03/26 17:10:32 silene Exp $ */
+/* $Id: hotkeys.cpp,v 1.78 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -43,6 +43,8 @@ const struct {
 	bool hidden;
 } hotkey_list_[] = {
 	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next unit"), false },
+	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous unit"), false },
+	{ hotkey::HOTKEY_UNIT_HOLD_POSITION, "holdposition", N_("Hold Position"), false},
 	{ hotkey::HOTKEY_END_UNIT_TURN, "endunitturn", N_("End Unit Turn"), false },
 	{ hotkey::HOTKEY_LEADER, "leader", N_("Leader"), false },
 	{ hotkey::HOTKEY_UNDO, "undo", N_("Undo"), false },
@@ -50,7 +52,8 @@ const struct {
 	{ hotkey::HOTKEY_ZOOM_IN, "zoomin", N_("Zoom In"), false },
 	{ hotkey::HOTKEY_ZOOM_OUT, "zoomout", N_("Zoom Out"), false },
 	{ hotkey::HOTKEY_ZOOM_DEFAULT, "zoomdefault", N_("Default Zoom"), false },
-	{ hotkey::HOTKEY_FULLSCREEN, "fullscreen", N_("Fullscreen"), false },
+	{ hotkey::HOTKEY_FULLSCREEN, "fullscreen", N_("Toggle Full Screen"), false },
+	{ hotkey::HOTKEY_SCREENSHOT, "screenshot", N_("Screenshot"), false },
 	{ hotkey::HOTKEY_ACCELERATED, "accelerated", N_("Accelerated"), false },
 	{ hotkey::HOTKEY_UNIT_DESCRIPTION, "describeunit", N_("Unit Description"), false },
 	{ hotkey::HOTKEY_RENAME_UNIT, "renameunit", N_("Rename Unit"), false },
@@ -114,7 +117,7 @@ hotkey::hotkey_item null_hotkey_;
 
 namespace hotkey {
 
-hotkey_item::hotkey_item(HOTKEY_COMMAND id, const std::string& command, const std::string& description, bool hidden) 
+hotkey_item::hotkey_item(HOTKEY_COMMAND id, const std::string& command, const std::string& description, bool hidden)
 	: id_(id), command_(command), description_(description), keycode_(0),
 	  shift_(false), ctrl_(false), alt_(false), cmd_(false), hidden_(hidden)
 {
@@ -143,10 +146,10 @@ void hotkey_item::load_from_config(const config& cfg)
 	cmd_ = (cfg["cmd"] == "yes");
 }
 
-std::string hotkey_item::get_name() const 
+std::string hotkey_item::get_name() const
 {
 	if (keycode_ != 0) {
-		std::stringstream str;			
+		std::stringstream str;
 		if (alt_)
 			str << "alt+";
 		if (ctrl_)
@@ -177,10 +180,10 @@ void hotkey_item::set_key(int keycode, bool shift, bool ctrl, bool alt, bool cmd
 	cmd_ = cmd;
 }
 
-manager::manager() 
+manager::manager()
 {
 	for (int i = 0; hotkey_list_[i].command; ++i) {
-		hotkeys_.push_back(hotkey_item(hotkey_list_[i].id, hotkey_list_[i].command, 
+		hotkeys_.push_back(hotkey_item(hotkey_list_[i].id, hotkey_list_[i].command,
 				"", hotkey_list_[i].hidden));
 	}
 }
@@ -232,7 +235,7 @@ void save_hotkeys(config& cfg)
 	}
 }
 
-hotkey_item& get_hotkey(HOTKEY_COMMAND id) 
+hotkey_item& get_hotkey(HOTKEY_COMMAND id)
 {
 	std::vector<hotkey_item>::iterator itor;
 
@@ -247,7 +250,7 @@ hotkey_item& get_hotkey(HOTKEY_COMMAND id)
 	return *itor;
 }
 
-hotkey_item& get_hotkey(const std::string& command) 
+hotkey_item& get_hotkey(const std::string& command)
 {
 	std::vector<hotkey_item>::iterator itor;
 
@@ -269,17 +272,17 @@ hotkey_item& get_hotkey(int keycode, bool shift, bool ctrl, bool alt, bool cmd,
 
 	for (itor = hotkeys_.begin(); itor != hotkeys_.end(); ++itor) {
 		if(mods) {
-			if(itor->get_keycode() == keycode 
+			if(itor->get_keycode() == keycode
 					&& (shift == itor->get_shift() || shift == true)
 					&& (ctrl == itor->get_ctrl() || ctrl == true)
 					&& (alt == itor->get_alt() || alt == true)
 					&& (cmd == itor->get_cmd() || cmd == true))
 				break;
 		} else {
-			if(itor->get_keycode() == keycode 
-					&& shift == itor->get_shift() 
-					&& ctrl == itor->get_ctrl() 
-					&& alt == itor->get_alt() 
+			if(itor->get_keycode() == keycode
+					&& shift == itor->get_shift()
+					&& ctrl == itor->get_ctrl()
+					&& alt == itor->get_alt()
 					&& cmd == itor->get_cmd())
 				break;
 		}
@@ -293,12 +296,15 @@ hotkey_item& get_hotkey(int keycode, bool shift, bool ctrl, bool alt, bool cmd,
 
 hotkey_item& get_hotkey(const SDL_KeyboardEvent& event, bool mods)
 {
-	return get_hotkey(event.keysym.sym, 
-			(event.keysym.mod & KMOD_SHIFT) != 0, 
-			(event.keysym.mod & KMOD_CTRL) != 0, 
-			(event.keysym.mod & KMOD_ALT) != 0, 
-			(event.keysym.mod & KMOD_LMETA) != 0, 
-			mods);
+	return get_hotkey(event.keysym.sym,
+			(event.keysym.mod & KMOD_SHIFT) != 0,
+			(event.keysym.mod & KMOD_CTRL) != 0,
+			(event.keysym.mod & KMOD_ALT) != 0,
+			(event.keysym.mod & KMOD_LMETA) != 0
+#ifdef __APPLE__
+			|| (event.keysym.mod & KMOD_RMETA) != 0
+#endif
+			, mods);
 }
 
 hotkey_item& get_visible_hotkey(int index)
@@ -394,6 +400,9 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 		case HOTKEY_FULLSCREEN:
 			preferences::set_fullscreen(!preferences::fullscreen());
 			break;
+		case HOTKEY_SCREENSHOT:
+			disp.screenshot();
+			break;
 		case HOTKEY_ACCELERATED:
 			preferences::set_turbo(!preferences::turbo());
 			break;
@@ -404,10 +413,18 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			if(executor)
 				executor->cycle_units();
 			break;
+		case HOTKEY_CYCLE_BACK_UNITS:
+			if(executor)
+				executor->cycle_back_units();
+			break;
 		case HOTKEY_ENDTURN:
 			if(executor)
 				executor->end_turn();
 			break;
+		case HOTKEY_UNIT_HOLD_POSITION:
+			if(executor)
+                executor->unit_hold_position();
+			break;
 		case HOTKEY_END_UNIT_TURN:
 			if(executor)
 				executor->end_unit_turn();
