@@ -1,7 +1,7 @@
-/* $Id: language.hpp,v 1.34 2005/03/27 23:06:17 gruikya Exp $ */
+/* $Id: language.hpp,v 1.36 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -21,6 +21,7 @@
 //this module controls internationalization.
 
 class t_string;
+class config;
 
 struct language_def
 {
@@ -28,7 +29,7 @@ struct language_def
 	language_def(const std::string& name, const std::string& lang) : localename(name), language(lang)
 	{}
 	std::string localename;
-  	std::string language;
+	std::string language;
 	bool operator== (const language_def&);
 };
 extern language_def known_languages[];
@@ -62,4 +63,7 @@ const language_def& get_language();
 //function which attempts to query and return the locale on the system
 const language_def& get_locale();
 
+/** Initializes the list of textdomains from a configuration object */
+void init_textdomains(const config& cfg);
+
 #endif
