@@ -86,8 +86,8 @@ void default_map_generator::user_config(display& disp)
 	const std::string& players_label = _("Players:");
 	const std::string& width_label = _("Width:");
 	const std::string& height_label = _("Height:");
-	const std::string& iterations_label = _("Iterations:");
-	const std::string& hillsize_label = _("Bump Size:");
+	const std::string& iterations_label = _("Number of Hills:");
+	const std::string& hillsize_label = _("Max Hill Size:");
 	const std::string& villages_label = _("Villages:");
 	const std::string& landform_label = _("Landform:");
 
@@ -99,7 +99,7 @@ void default_map_generator::user_config(display& disp)
 	SDL_Rect villages_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,villages_label,0,0);
 	SDL_Rect landform_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,landform_label,0,0);
 
-	const int horz_margin = 5;
+	const int horz_margin = 15;
 	const int text_right = xpos + horz_margin +
 	        maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(
 		         players_rect.w,width_rect.w),height_rect.w),iterations_rect.w),hillsize_rect.w),villages_rect.w);
@@ -111,7 +111,7 @@ void default_map_generator::user_config(display& disp)
 	hillsize_rect.x = text_right - hillsize_rect.w;
 	villages_rect.x = text_right - villages_rect.w;
 	landform_rect.x = text_right - landform_rect.w;
-	
+
 	const int vertical_margin = 20;
 	players_rect.y = ypos + vertical_margin*2;
 	width_rect.y = players_rect.y + players_rect.h + vertical_margin;
@@ -123,7 +123,7 @@ void default_map_generator::user_config(display& disp)
 
 	const int max_players = 9;
 
-	const int right_space = 100;
+	const int right_space = 150;
 
 	const int slider_left = text_right + 10;
 	const int slider_right = xpos + width - horz_margin - right_space;
@@ -138,7 +138,7 @@ void default_map_generator::user_config(display& disp)
 	const int max_width = 100;
 	const int max_height = 100;
 	const int extra_size_per_player = 2;
-	
+
 	slider_rect.y = width_rect.y;
 	gui::slider width_slider(screen);
 	width_slider.set_location(slider_rect);
@@ -250,7 +250,7 @@ void default_map_generator::user_config(display& disp)
 		height_str << height_;
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,height_str.str(),
 		                slider_right+horz_margin,height_rect.y);
-		
+
 		std::stringstream villages_str;
 		villages_str << nvillages_ << _("/1000 tiles");
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,villages_str.str(),
