@@ -1,4 +1,4 @@
-/* $Id: multiplayer.cpp,v 1.154 2005/04/06 20:46:45 gruikya Exp $ */
+/* $Id: multiplayer.cpp,v 1.160 2005/06/04 19:16:05 ott Exp $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -23,11 +23,13 @@
 #include "multiplayer_lobby.hpp"
 #include "multiplayer_create.hpp"
 #include "network.hpp"
-#include "playlevel.hpp"
+#include "playcampaign.hpp"
 #include "preferences.hpp"
 #include "random.hpp"
 #include "replay.hpp"
 #include "video.hpp"
+#include "statistics.hpp"
+#include "serialization/string_utils.hpp"
 
 #define LOG_NW LOG_STREAM(info, network)
 
@@ -101,14 +103,14 @@ server_type open_connection(display& disp, const std::string& original_host)
 	const int pos = h.find_first_of(":");
 	std::string host;
 	unsigned int port;
- 
- 	if(pos == -1) {
+
+	if(pos == -1) {
 		host = h;
 		port = 15000;
- 	} else {
+	} else {
 		host = h.substr(0, pos);
 		port = lexical_cast_default<unsigned int>(h.substr(pos + 1), 15000);
- 	}
+	}
 
 	// shown_hosts is used to prevent the client being locked in a redirect
 	// loop.
@@ -118,7 +120,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 	config::child_list redirects;
 	config data;
-	sock = network::connect(host, port);
+	sock = gui::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
 
 	do {
 
@@ -134,12 +136,13 @@ server_type open_connection(display& disp, const std::string& original_host)
 		// Backwards-compatibility "version" attribute
 		const std::string& version = data["version"];
 		if(version.empty() == false && version != game_config::version) {
-			// FIXME: make this string translatable
-			throw network::error("The server requires version '" + version
-					+ "' while you are using version '" + 
-					game_config::version + "'");
+			utils::string_map i18n_symbols;
+			i18n_symbols["version1"] = version;
+			i18n_symbols["version2"] = game_config::version;
+			const std::string errorstring = vgettext("The server requires version '$version1' while you are using version '$version2'", i18n_symbols);
+			throw network::error(errorstring);
 		}
-		
+
 		// Check for "redirect" messages
 		if(data.child("redirect")) {
 			config* redirect = data.child("redirect");
@@ -148,14 +151,13 @@ server_type open_connection(display& disp, const std::string& original_host)
 			port = lexical_cast_default<unsigned int>((*redirect)["port"], 15000);
 
 			if(shown_hosts.find(hostpair(host,port)) != shown_hosts.end()) {
-				// FIXME: make this string translatable
-				throw network::error("Server-side redirect loop");
+				throw network::error(_("Server-side redirect loop"));
 			}
 			shown_hosts.insert(hostpair(host, port));
 
 			if(network::nconnections() > 0)
 				network::disconnect();
-			sock = network::connect(host, port);
+			sock = gui::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
 			continue;
 		}
 
@@ -179,7 +181,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 				std::string login = preferences::login();
 
-				if(!first_time) {	
+				if(!first_time) {
 					const int res = gui::show_dialog(disp, NULL, "",
 							_("You must log in to this server"), gui::OK_CANCEL,
 							NULL, NULL, _("Login: "), &login);
@@ -232,15 +234,15 @@ server_type open_connection(display& disp, const std::string& original_host)
 void enter_wait_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, bool observe)
 {
 	mp::ui::result res;
-	config level;
 	game_state state;
 	network_game_manager m;
 
 	gamelist.clear();
+	statistics::fresh_stats();
 
 	{
 		mp::wait ui(disp, game_config, data, chat, gamelist);
-		
+
 		ui.join_game(observe);
 
 		run_lobby_loop(disp, ui);
@@ -248,14 +250,14 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
-			level = ui.get_level();
+
 			state = ui.get_state();
 		}
 	}
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_level(data, game_config, &level, disp.video(), state, std::vector<config*>());
+		play_game(disp, state, game_config, data, disp.video(), IO_CLIENT);
 		recorder.clear();
 
 		break;
@@ -265,20 +267,20 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 	}
 }
 
-void enter_connect_mode(display& disp, const config& game_config, game_data& data, 
+void enter_connect_mode(display& disp, const config& game_config, game_data& data,
 		mp::chat& chat, config& gamelist, const mp::create::parameters& params,
 		mp::controller default_controller, bool is_server)
 {
 	mp::ui::result res;
-	config level;
 	game_state state;
 	const network::manager net_manager;
-	const network::server_manager serv_manager(15000, is_server ? 
+	const network::server_manager serv_manager(15000, is_server ?
 			network::server_manager::TRY_CREATE_SERVER :
 			network::server_manager::NO_SERVER);
 	network_game_manager m;
 
 	gamelist.clear();
+	statistics::fresh_stats();
 
 	{
 		mp::connect ui(disp, game_config, data, chat, gamelist, params, default_controller);
@@ -287,17 +289,16 @@ void enter_connect_mode(display& disp, const config& game_config, game_data& dat
 		res = ui.get_result();
 
 		// start_game() updates the parameters to reflect game start,
-		// so it must be called before get_level() 
+		// so it must be called before get_level()
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
-			level = ui.get_level();
 			state = ui.get_state();
 		}
 	}
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_level(data, game_config, &level, disp.video(), state, std::vector<config*>());
+		play_game(disp, state, game_config, data, disp.video(), IO_SERVER);
 		recorder.clear();
 
 		break;
@@ -329,7 +330,7 @@ void enter_create_mode(display& disp, const config& game_config, game_data& data
 	}
 }
 
-void enter_lobby_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist) 
+void enter_lobby_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist)
 {
 	mp::ui::result res;
 
@@ -363,7 +364,7 @@ void enter_lobby_mode(display& disp, const config& game_config, game_data& data,
 		}
 	}
 }
- 
+
 }
 
 namespace mp {
