@@ -1,6 +1,6 @@
-/* $Id: multiplayer_connect.cpp,v 1.140 2005/04/05 19:42:05 gruikya Exp $ */
+/* $Id: multiplayer_connect.cpp,v 1.151 2005/06/26 20:10:08 ott Exp $ */
 /*
-   Copyright (C) 
+   Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
 #include "log.hpp"
 #include "multiplayer_connect.hpp"
 #include "preferences.hpp"
-#include "replay.hpp"
 #include "statistics.hpp"
 #include "show_dialog.hpp"
 #include "wassert.hpp"
@@ -50,15 +49,15 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	index_(index),
 
 	player_number_(parent.video(), lexical_cast_default<std::string>(index+1, ""),
-			font::SIZE_XLARGE, font::GOOD_COLOUR),
+	               font::SIZE_LARGE, font::LOBBY_COLOUR),
 	combo_controller_(parent.disp(), parent.player_types_),
-	orig_controller_(parent.video(), cfg["description"], font::SIZE_SMALL), 
+	orig_controller_(parent.video(), cfg["description"], font::SIZE_SMALL),
 	combo_faction_(parent.disp(), parent.player_factions_),
 	combo_leader_(parent.disp(), std::vector<std::string>()),
 	combo_team_(parent.disp(), parent.player_teams_),
 	combo_colour_(parent.disp(), parent.player_colours_),
 	slider_gold_(parent.video()),
-	label_gold_(parent.video(), "100", font::SIZE_NORMAL, font::GOOD_COLOUR),
+	label_gold_(parent.video(), "100", font::SIZE_SMALL, font::LOBBY_COLOUR),
 	enabled_(!parent_->params_.saved_game),
 	changed_(false),
 	llm_(parent.era_sides_, &parent.game_data_, enabled_ ? &combo_leader_ : NULL)
@@ -73,12 +72,12 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 			}
 		}
 	}
-	
+
 	slider_gold_.set_min(20);
 	slider_gold_.set_max(1000);
 	slider_gold_.set_increment(25);
 	slider_gold_.set_value(lexical_cast_default<int>(cfg_["gold"], 100));
-	slider_gold_.set_measurements(120, 16);
+	slider_gold_.set_measurements(100, 16);
 
 	combo_faction_.enable(enabled_);
 	combo_leader_.enable(enabled_);
@@ -122,7 +121,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		if(leader_type.empty()) {
 			leader_name_pseudolist.push_back("-");
 		} else {
-			game_data::unit_type_map::const_iterator leader_name = 
+			game_data::unit_type_map::const_iterator leader_name =
 				parent_->game_data_.unit_types.find(leader_type);
 			if(leader_name == parent_->game_data_.unit_types.end()) {
 				leader_name_pseudolist.push_back("-");
@@ -138,8 +137,8 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 }
 
 connect::side::side(const side& a) :
-	parent_(a.parent_), cfg_(a.cfg_), 
-	index_(a.index_), id_(a.id_),  save_id_(a.save_id_), 
+	parent_(a.parent_), cfg_(a.cfg_),
+	index_(a.index_), id_(a.id_),  save_id_(a.save_id_),
 	controller_(a.controller_),
 	faction_(a.faction_), team_(a.team_), colour_(a.colour_),
 	gold_(a.gold_), leader_(a.leader_), /* taken_(a.taken_), */
@@ -147,23 +146,23 @@ connect::side::side(const side& a) :
 	orig_controller_(a.orig_controller_),
 	combo_faction_(a.combo_faction_), combo_leader_(a.combo_leader_),
 	combo_team_(a.combo_team_), combo_colour_(a.combo_colour_),
-	slider_gold_(a.slider_gold_), label_gold_(a.label_gold_), 
+	slider_gold_(a.slider_gold_), label_gold_(a.label_gold_),
 	enabled_(a.enabled_), changed_(a.changed_), llm_(a.llm_)
 {
 	llm_.set_combo(enabled_ ? &combo_leader_ : NULL);
 }
 
 void connect::side::add_widgets_to_scrollpane(gui::scrollpane& pane, int pos)
-{ 
-	pane.add_widget(&player_number_, 10, 3 + pos);
-	pane.add_widget(&combo_controller_, 30, 5 + pos);
-	pane.add_widget(&orig_controller_, 30 + (combo_controller_.width() - orig_controller_.width()) / 2, 35 + pos + (combo_leader_.height() - orig_controller_.height()) / 2);
-	pane.add_widget(&combo_faction_, 145, 5 + pos);
-	pane.add_widget(&combo_leader_, 145, 35 + pos);
-	pane.add_widget(&combo_team_, 260, 5 + pos);
-	pane.add_widget(&combo_colour_, 375, 5 + pos);
-	pane.add_widget(&slider_gold_, 490, 5 + pos);
-	pane.add_widget(&label_gold_, 500 + slider_gold_.width(), 5 + pos);
+{
+	pane.add_widget(&player_number_, 0, 5 + pos);
+	pane.add_widget(&combo_controller_, 20, 5 + pos);
+	pane.add_widget(&orig_controller_, 20 + (combo_controller_.width() - orig_controller_.width()) / 2, 35 + pos + (combo_leader_.height() - orig_controller_.height()) / 2);
+	pane.add_widget(&combo_faction_, 135, 5 + pos);
+	pane.add_widget(&combo_leader_, 135, 35 + pos);
+	pane.add_widget(&combo_team_, 250, 5 + pos);
+	pane.add_widget(&combo_colour_, 365, 5 + pos);
+	pane.add_widget(&slider_gold_, 480, 5 + pos);
+	pane.add_widget(&label_gold_, 490 + slider_gold_.width(), 8 + pos);
 }
 
 void connect::side::process_event()
@@ -174,7 +173,7 @@ void connect::side::process_event()
 		} else if (combo_controller_.selected() < CNTR_LAST) {
 			// If the current side corresponds to an existing user,
 			// we must kick it!
-			
+
 			if(id_ == preferences::login()) {
 				update_controller_ui(); // Cannot kick game creator
 			} else {
@@ -209,7 +208,7 @@ void connect::side::process_event()
 	}
 
 	if(!enabled_)
-		return; 
+		return;
 
 	if (combo_faction_.changed() && combo_faction_.selected() >= 0) {
 		faction_ = combo_faction_.selected();
@@ -293,28 +292,31 @@ config connect::side::get_config() const
 	res["description"] = id_;
 
 	if (id_.empty()) {
+		char const *description;
 		switch(controller_) {
 		case CNTR_NETWORK:
-			res["user_description"] = _("(Vacant slot)");
+			description = N_("(Vacant slot)");
 			break;
 		case CNTR_LOCAL:
 			if(enabled_ && cfg_["save_id"].empty()) {
 				res["save_id"] = "local" + res["side"].str();
 			}
-			res["user_description"] =  _("Anonymous local player");
+			description = N_("Anonymous local player");
 			break;
 		case CNTR_COMPUTER:
 			if(enabled_ && cfg_["save_id"].empty()) {
 				res["save_id"] = "ai" + res["side"].str();
 			}
-			res["user_description"] = _("Computer player");
+			description = N_("Computer player");
 			break;
 		case CNTR_EMPTY:
-			res["user_description"] = _("(Empty slot)");
+			description = N_("(Empty slot)");
 			break;
 		default:
+			wassert(false);
 			break;
 		}
+		res["user_description"] = t_string(description, "wesnoth");
 	} else {
 		if(enabled_ && cfg_["save_id"].empty()) {
 			res["save_id"] = id_;
@@ -384,12 +386,12 @@ void connect::side::update_user_list()
 	update_controller_ui();
 }
 
-const std::string& connect::side::get_id() const 
+const std::string& connect::side::get_id() const
 {
 	return id_;
 }
 
-void connect::side::set_id(const std::string& id) 
+void connect::side::set_id(const std::string& id)
 {
 	connected_user_list::iterator i = parent_->find_player(id);
 	if (i != parent_->users_.end()) {
@@ -399,7 +401,7 @@ void connect::side::set_id(const std::string& id)
 	update_ui();
 }
 
-const std::string& connect::side::get_save_id() const 
+const std::string& connect::side::get_save_id() const
 {
 	return save_id_;
 }
@@ -409,24 +411,27 @@ void connect::side::import_network_user(const config& data)
 	id_ = data["name"];
 	controller_ = CNTR_NETWORK;
 
-	if (enabled_) {
+	if (enabled_ && !parent_->era_sides_.empty()) {
 		faction_ = lexical_cast_default<int>(data["faction"], 0);
 		if (faction_ > int(parent_->era_sides_.size()))
 			faction_ = 0;
 		llm_.update_leader_list(faction_);
 		llm_.set_leader(data["leader"]);
 	}
-	
+
 	update_ui();
 }
 
 void connect::side::reset(mp::controller controller)
 {
 	id_ = "";
-	faction_ = 0;
 	controller_ = controller;
 
-	llm_.update_leader_list(0);
+	if(enabled_ && !parent_->era_sides_.empty()) {
+		faction_ = 0;
+		llm_.update_leader_list(0);
+	}
+
 	update_ui();
 }
 
@@ -439,7 +444,7 @@ void connect::side::resolve_random()
 
 		// Builds the list of sides which aren't random
 		std::vector<int> nonrandom_sides;
-		for(config::child_iterator itor = parent_->era_sides_.begin(); 
+		for(config::child_iterator itor = parent_->era_sides_.begin();
 				itor != parent_->era_sides_.end(); ++itor) {
 			if((**itor)["random_faction"] != "yes") {
 				nonrandom_sides.push_back(itor - parent_->era_sides_.begin());
@@ -454,7 +459,7 @@ void connect::side::resolve_random()
 	}
 
 	if (llm_.get_leader() == "random") {
-		// Choose a random leader type.  
+		// Choose a random leader type.
 		const config& fact = *parent_->era_sides_[faction_];
 		std::vector<std::string> types = utils::split(fact["leader"]);
 		if (!types.empty()) {
@@ -469,9 +474,9 @@ void connect::side::resolve_random()
 }
 
 connect::connect(display& disp, const config& game_config, const game_data& data,
-		chat& c, config& gamelist, const create::parameters& params, 
+		chat& c, config& gamelist, const create::parameters& params,
 		mp::controller default_controller) :
-	mp::ui(disp, game_config, c, gamelist),
+	mp::ui(disp, _("Game Lobby"), game_config, c, gamelist),
 
 	game_data_(data),
 	level_(),
@@ -479,16 +484,16 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 
 	team_prefix_(std::string(_("Team")) + " "),
 
-	waiting_label_(video(), ""),
+	waiting_label_(video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
 	message_full_(false),
 	default_controller_(default_controller),
 
 	scroll_pane_(video()),
-	type_title_label_(video(), _("Player/Type"), font::SIZE_NORMAL, font::GOOD_COLOUR),
-	faction_title_label_(video(), _("Faction"), font::SIZE_NORMAL, font::GOOD_COLOUR),
-	team_title_label_(video(), _("Team"), font::SIZE_NORMAL, font::GOOD_COLOUR),
-	colour_title_label_(video(), _("Color"), font::SIZE_NORMAL, font::GOOD_COLOUR),
-	gold_title_label_(video(), _("Gold"), font::SIZE_NORMAL, font::GOOD_COLOUR),
+	type_title_label_(video(), _("Player/Type"), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	faction_title_label_(video(), _("Faction"), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	team_title_label_(video(), _("Team"), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	colour_title_label_(video(), _("Color"), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	gold_title_label_(video(), _("Gold"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 
 	ai_(video(), _("Computer vs Computer")),
 	launch_(video(), _("I'm Ready")),
@@ -501,12 +506,14 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 	network::send_data(response);
 
 	load_game();
+	if(get_result() == QUIT)
+		return;
 	lists_init();
 
 	// Adds the current user as default user.
 	users_.push_back(connected_user(preferences::login(), CNTR_LOCAL, 0));
 	update_user_combos();
-	if(sides_.empty()) { 
+	if(sides_.empty()) {
 		throw config::error(_("The scenario is invalid because it has no sides."));
 	}
 
@@ -571,11 +578,6 @@ void connect::process_event()
 	}
 }
 
-const config& connect::get_level()
-{
-	return level_;
-}
-
 const game_state& connect::get_state()
 {
 	return state_;
@@ -584,7 +586,7 @@ const game_state& connect::get_state()
 void connect::start_game()
 {
 	// Resolves the "random faction" and "random message"
-	for (side_list::iterator itor = sides_.begin(); itor != sides_.end(); 
+	for (side_list::iterator itor = sides_.begin(); itor != sides_.end();
 			++itor) {
 
 		itor->resolve_random();
@@ -594,23 +596,15 @@ void connect::start_game()
 	lock.add_child("stop_updates");
 	network::send_data(lock);
 
-	// Replays the level
-	recorder.set_save_info(state_);
-	recorder.set_skip(-1);
-	level_.clear_children("replay");
-
 	// Re-sends the whole level
 	update_and_send_diff();
 
-	// This must be done after updating level_
-	state_.snapshot = level_;
-	state_.starting_pos = level_;
-	state_.players.clear();
+	// Build the gamestate object after updating the level
+	level_to_gamestate(level_, state_);
 
 	config cfg;
 	cfg.add_child("start_game");
 	network::send_data(cfg);
-
 }
 
 void connect::hide_children(bool hide)
@@ -787,23 +781,20 @@ void connect::layout_children(const SDL_Rect& rect)
 	size_t bottom = ca.y + ca.h;
 
 	//Buttons
-	right_button->set_location(right - right_button->width() - gui::ButtonHPadding,
-			bottom - right_button->height() - gui::ButtonVPadding);
-	left_button->set_location(right - right_button->width() - left_button->width() - gui::ButtonHPadding*2,
-			bottom - left_button->height()-gui::ButtonVPadding);
+	right_button->set_location(right - right_button->width(),
+	                           bottom - right_button->height());
+	left_button->set_location(right - right_button->width() - left_button->width() - gui::ButtonHPadding,
+	                          bottom - left_button->height());
 
-	ai_.set_location(left+30, bottom-left_button->height()-gui::ButtonVPadding);
-	waiting_label_.set_location(ai_.location().x + ai_.location().w + 10,
-			bottom-left_button->height()-gui::ButtonVPadding);
+	ai_.set_location(left, bottom-left_button->height());
+	waiting_label_.set_location(ai_.location().x + ai_.location().w + 8,
+	                            bottom-left_button->height() + 4);
 
-	// Title and labels
-	gui::draw_dialog_title(left,top,&video(),_("Game Lobby"));
-
-	type_title_label_.set_location((left+30)+(launch_.width()/2)-(type_title_label_.width()/2),top+35);
-	faction_title_label_.set_location((left+145)+(launch_.width()/2)-(faction_title_label_.width()/2),top+35);
-	team_title_label_.set_location((left+260)+(launch_.width()/2)-(team_title_label_.width()/2),top+35);
-	colour_title_label_.set_location((left+375)+(launch_.width()/2)-(colour_title_label_.width()/2),top+35);
-	gold_title_label_.set_location((left+480)+(launch_.width()/2)-(gold_title_label_.width()/2),top+35);
+	type_title_label_.set_location(left+30, top+35);
+	faction_title_label_.set_location((left+145), top+35);
+	team_title_label_.set_location((left+260), top+35);
+	colour_title_label_.set_location((left+375), top+35);
+	gold_title_label_.set_location((left+493), top+35);
 
 	SDL_Rect scroll_pane_rect;
 	scroll_pane_rect.x = ca.x;
@@ -844,9 +835,6 @@ void connect::lists_init()
 		}
 	}
 
-	std::string prefix;
-	prefix.resize(1);
-
 	// Colours
 	for(int i = 0; i < 9; ++i) {
 		player_colours_.push_back(get_colour_string(i));
@@ -877,14 +865,15 @@ void connect::load_game()
 			set_result(QUIT);
 			return;
 		}
-		
-		//state_.players.clear();
+
 		std::string error_log;
 		::load_game(game_data_, game, state_, &error_log);
 		if(!error_log.empty()) {
-			gui::show_error_message(disp(), 
+			gui::show_error_message(disp(),
 					_("The file you have tried to load is corrupt: '") +
 					error_log);
+			set_result(QUIT);
+			return;
 		}
 
 		if(state_.campaign_type != "multiplayer") {
@@ -905,28 +894,23 @@ void connect::load_game()
 		}
 
 		level_ = state_.snapshot;
+
+		// Adds the replay data, and the replay start, to the level, so
+		// clients can receive it.
+
+		//TO-DO: this should be done only if the player has selected for the replay to be viewed
+		if(false) {
+			level_.add_child("replay") = state_.replay_data;
+			if(!state_.starting_pos.empty())
+				level_.add_child("replay_start") = state_.starting_pos;
+		}
+
 		level_.add_child("statistics") = statistics::write_stats();
 
-		recorder = replay(state_.replay_data);
-
-		//if this is a snapshot save, we don't want to use the replay data
-		if(level_["snapshot"] == "yes") {
-			config* const start = level_.child("start");
-			if(start != NULL)
-				start->clear_children("replay");
-			level_.clear_children("replay");
-			recorder.set_to_end();
-		} else {
-			//add the replay data under the level data so clients can
-			//receive it
-			level_.clear_children("replay");
-			level_.add_child("replay") = state_.replay_data;
-		}
 	} else {
 		level_ = params_.scenario_data;
 		level_["turns"] = lexical_cast_default<std::string>(params_.num_turns, "20");
 
-		// Gets the era from the era of the savegame
 		const std::string& era = params_.era;
 
 		// Initialize the list of sides available for the current era.
@@ -937,21 +921,15 @@ void connect::load_game()
 			throw config::error(vgettext("Cannot find era $era", i18n_symbols));
 		}
 		era_sides_ = era_cfg->get_children("multiplayer_side");
+		level_["scenario"] = params_.name;
 		level_.add_child("era", *era_cfg);
+		level_["experience_modifier"] = lexical_cast<std::string>(params_.xp_modifier);
 	}
 
 	//this will force connecting clients to be using the same version number as us.
 	level_["version"] = game_config::version;
 
-	state_.label = level_["name"];
-	state_.players.clear();
-	level_["scenario"] = state_.scenario = params_.name;
-	state_.campaign_type = "multiplayer";
-
-	if(!params_.saved_game) 
-		level_["experience_modifier"] = lexical_cast<std::string>(params_.xp_modifier);
-
-	level_["observers"] = params_.allow_observers ? "yes" : "no";
+	level_["observer"] = params_.allow_observers ? "yes" : "no";
 
 	if(level_["objectives"].empty()) {
 		level_["objectives"] = t_string(N_("Victory\n\
@@ -963,7 +941,7 @@ void connect::update_level()
 {
 	// Import all sides into the level
 	level_.clear_children("side");
-	for(side_list::const_iterator itor = sides_.begin(); itor != sides_.end(); 
+	for(side_list::const_iterator itor = sides_.begin(); itor != sides_.end();
 			++itor) {
 
 		level_.add_child("side", itor->get_config());
