@@ -1,6 +1,6 @@
-/* $Id: multiplayer_lobby.cpp,v 1.70 2005/03/23 19:41:09 j_daniel Exp $ */
+/* $Id: multiplayer_lobby.cpp,v 1.75 2005/06/06 20:12:47 gruikya Exp $ */
 /*
-   Copyright (C) 
+   Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -19,17 +19,97 @@
 #include "wml_separators.hpp"
 #include "game_config.hpp"
 #include "gettext.hpp"
+#include "show_dialog.hpp"
+
+namespace {
+
+std::string games_menu_heading()
+{
+	std::ostringstream str;
+	str << HEADING_PREFIX << _("Map") << COLUMN_SEPARATOR << _("Name")
+		<< COLUMN_SEPARATOR << _("Status");
+	return str.str();
+}
+
+}
 
 namespace mp {
 
+lobby::lobby_sorter::lobby_sorter(const config& cfg) : cfg_(cfg)
+{
+	set_alpha_sort(1);
+}
+
+bool lobby::lobby_sorter::column_sortable(int column) const
+{
+	switch(column)
+	{
+	case MAP_COLUMN:
+	case STATUS_COLUMN:
+		return true;
+	default:
+		return basic_sorter::column_sortable(column);
+	}
+}
+
+bool lobby::lobby_sorter::less(int column, const gui::menu::item& row1, const gui::menu::item& row2) const
+{
+	const config* const list = cfg_.child("gamelist");
+	if(list == NULL) {
+		return false;
+	}
+
+	const config::child_list& games = list->get_children("game");
+	if(row1.id >= games.size() || row2.id >= games.size()) {
+		return false;
+	}
+
+	const config& game1 = *games[row1.id];
+	const config& game2 = *games[row2.id];
+
+	if(column == MAP_COLUMN) {
+		size_t mapsize1 = game1["map_data"].size();
+		if(mapsize1 == 0) {
+			mapsize1 = game1["map"].size();
+		}
+
+		size_t mapsize2 = game2["map_data"].size();
+		if(mapsize2 == 0) {
+			mapsize2 = game2["map"].size();
+		}
+
+		return mapsize1 < mapsize2;
+
+	} else if(column == STATUS_COLUMN) {
+		const int nslots1 = atoi(game1["slots"].c_str());
+		const int nslots2 = atoi(game2["slots"].c_str());
+
+		const int turn1 = atoi(game1["turn"].c_str());
+		const int turn2 = atoi(game2["turn"].c_str());
+
+		if(nslots1 > nslots2) {
+			return true;
+		} else if(nslots1 < nslots2) {
+			return false;
+		} else {
+			return turn1 < turn2;
+		}
+	} else {
+		return basic_sorter::less(column,row1,row2);
+	}
+
+	return false;
+}
+
 lobby::lobby(display& disp, const config& cfg, chat& c, config& gamelist) :
-	mp::ui(disp, cfg, c, gamelist),
+	mp::ui(disp, _("Game Lobby"), cfg, c, gamelist),
 
 	observe_game_(disp.video(), _("Observe Game")),
 	join_game_(disp.video(), _("Join Game")),
 	create_game_(disp.video(), _("Create Game")),
 	quit_game_(disp.video(), _("Quit")),
-	games_menu_(disp.video(), std::vector<std::string>()),
+	sorter_(gamelist),
+	games_menu_(disp.video(), std::vector<std::string>(1,games_menu_heading()),false,-1,-1,&sorter_),
 	current_game_(0)
 {
 	game_config::debug = false;
@@ -56,8 +136,9 @@ void lobby::layout_children(const SDL_Rect& rect)
 	create_game_.set_location(observe_game_.location().x + observe_game_.location().w + 5,yscale(7));
 	quit_game_.set_location(create_game_.location().x + create_game_.location().w + 5,yscale(7));
 
-	games_menu_.set_width(xscale(832));
-	games_menu_.set_location(xscale(12),yscale(42));
+	games_menu_.set_location(client_area().x, client_area().y + title().height());
+	games_menu_.set_measurements(client_area().w, client_area().h 
+			- title().height() - gui::ButtonVPadding);
 }
 
 void lobby::gamelist_updated(bool silent)
@@ -114,7 +195,7 @@ void lobby::gamelist_updated(bool silent)
 		if(turn != "") {
 			str << COLUMN_SEPARATOR << _("Turn") << " " << turn;
 		} else if(slots != "") {
-			str << COLUMN_SEPARATOR << slots << " " << 
+			str << COLUMN_SEPARATOR << slots << " " <<
 				ngettext(_("Vacant Slot"), _("Vacant Slots"), nslots);
 		}
 
@@ -128,7 +209,8 @@ void lobby::gamelist_updated(bool silent)
 		game_strings.push_back(_("<no games open>"));
 	}
 
-	games_menu_.set_items(game_strings);
+	//set the items, retaining the menu positioning if possible
+	games_menu_.set_items(game_strings,true,true);
 
 	if(games_menu_.selection() >= 0 && games_menu_.selection() < int(game_vacant_slots_.size())) {
 		wassert(game_vacant_slots_.size() == game_observers_.size());
@@ -182,7 +264,7 @@ void lobby::process_event()
 		}
 		return;
 	}
-			
+
 	if(create_game_.pressed()) {
 		set_result(CREATE);
 		return;
