@@ -1,6 +1,6 @@
-/* $Id: multiplayer_wait.cpp,v 1.23 2005/03/30 19:48:17 gruikya Exp $ */
+/* $Id: multiplayer_wait.cpp,v 1.28 2005/06/06 20:12:47 gruikya Exp $ */
 /*
-   Copyright (C) 
+   Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 #include "log.hpp"
 #include "multiplayer_wait.hpp"
 #include "preferences.hpp"
-#include "replay.hpp"
 #include "statistics.hpp"
 #include "util.hpp"
 #include "video.hpp"
@@ -40,7 +39,7 @@ wait::leader_preview_pane::leader_preview_pane(display& disp, const game_data* d
 		const config::child_list& side_list) :
 	gui::preview_pane(disp.video()),
 	side_list_(side_list),
-	leader_combo_(disp, std::vector<std::string>()), 
+	leader_combo_(disp, std::vector<std::string>()),
 	leaders_(side_list, data, &leader_combo_),
 	selection_(0), data_(data)
 {
@@ -102,7 +101,7 @@ void wait::leader_preview_pane::draw_contents()
 		SDL_Rect image_rect = {area.x,area.y,0,0};
 
 		surface unit_image(image::get_image(image, image::UNSCALED));
-	
+
 		if(!unit_image.null()) {
 			image_rect.w = unit_image->w;
 			image_rect.h = unit_image->h;
@@ -143,10 +142,10 @@ std::string wait::leader_preview_pane::get_selected_leader()
 }
 
 wait::wait(display& disp, const config& cfg, const game_data& data, mp::chat& c, config& gamelist) :
-	ui(disp, cfg, c, gamelist),
+	ui(disp, _("Game Lobby"), cfg, c, gamelist),
 
 	cancel_button_(disp.video(), _("Cancel")),
-	start_label_(disp.video(), _("Waiting for game to start...")),
+	start_label_(disp.video(), _("Waiting for game to start..."), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	game_menu_(disp.video(), std::vector<std::string>()),
 
 	game_data_(data),
@@ -164,12 +163,12 @@ void wait::process_event()
 void wait::join_game(bool observe)
 {
 	for(;;) {
-		network::connection data_res = gui::network_data_dialog(disp(), 
+		network::connection data_res = gui::network_data_dialog(disp(),
 				_("Getting game data..."), level_);
 		check_response(data_res, level_);
 
 		//if we have got valid side data
-		if(level_.child("gamelist") == NULL) 
+		if(level_.child("gamelist") == NULL)
 			break;
 	}
 
@@ -206,7 +205,7 @@ void wait::join_game(bool observe)
 			const config* era = level_.child("era");
 			if(era == NULL)
 				throw network::error(_("Era not available"));
-			const config::child_list& possible_sides = 
+			const config::child_list& possible_sides =
 				era->get_children("multiplayer_side");
 			if(possible_sides.empty()) {
 				set_result(QUIT);
@@ -216,13 +215,13 @@ void wait::join_game(bool observe)
 
 			std::vector<std::string> choices;
 			for(config::child_list::const_iterator side =
-					possible_sides.begin(); side != 
+					possible_sides.begin(); side !=
 					possible_sides.end(); ++side) {
 				choices.push_back((**side)["name"]);
 			}
 
 			std::vector<gui::preview_pane* > preview_panes;
-			leader_preview_pane leader_selector(disp(), &game_data_, 
+			leader_preview_pane leader_selector(disp(), &game_data_,
 					possible_sides);
 			preview_panes.push_back(&leader_selector);
 
@@ -251,53 +250,17 @@ const game_state& wait::get_state()
 	return state_;
 }
 
-const config& wait::get_level()
+void wait::start_game()
 {
-	return level_;
-}
-
-void wait::start_game() 
-{
-	const config::child_list& sides_list = level_.get_children("side");
-	for(config::child_list::const_iterator side = sides_list.begin(); 
-			side != sides_list.end(); ++side) {
-		if((**side)["controller"] == "network" && (**side)["description"] == preferences::login()) {
-			(**side)["controller"] = preferences::client_type();
-		} else if((**side)["controller"] != "null") {
-			(**side)["controller"] = "network";
-		}
-	}
-
-	//any replay data is only temporary and should be removed from
-	//the level data in case we want to save the game later
-	config* const replay_data = level_.child("replay");
-	config replay_data_store;
-	if(replay_data != NULL) {
-		replay_data_store = *replay_data;
-		LOG_NW << "setting replay\n";
-		recorder = replay(replay_data_store);
-		if(!recorder.empty()) {
-			recorder.set_skip(-1);
-		}
-
-		level_.clear_children("replay");
-	}
-
 	config const * const stats = level_.child("statistics");
 	if(stats != NULL) {
 		statistics::fresh_stats();
 		statistics::read_stats(*stats);
 	}
 
-	LOG_NW << "starting game\n";
+	level_to_gamestate(level_, state_);
 
-	state_.campaign_type = "multiplayer";
-	state_.label = level_["name"];
-	state_.scenario = level_["id"];
-	state_.starting_pos = level_;
-	state_.snapshot = level_;
-	state_.players.clear();
-	recorder.set_save_info(state_);
+	LOG_NW << "starting game\n";
 }
 
 void wait::layout_children(const SDL_Rect& rect)
@@ -305,12 +268,13 @@ void wait::layout_children(const SDL_Rect& rect)
 	ui::layout_children(rect);
 
 	const SDL_Rect ca = client_area();
-	int y = ca.y + ca.h - cancel_button_.height() - gui::ButtonVPadding;
+	int y = ca.y + ca.h - cancel_button_.height();
 
-	game_menu_.set_location(ca.x, ca.y);
-	game_menu_.set_measurements(ca.x + ca.w, y - ca.y - gui::ButtonVPadding);
-	cancel_button_.set_location(ca.x + ca.w - cancel_button_.width() - gui::ButtonHPadding, y);
-	start_label_.set_location(ca.x + gui::ButtonHPadding, y + 4);
+	game_menu_.set_location(ca.x, ca.y + title().height());
+	game_menu_.set_measurements(ca.w, y - ca.y - title().height()
+			- gui::ButtonVPadding);
+	cancel_button_.set_location(ca.x + ca.w - cancel_button_.width(), y);
+	start_label_.set_location(ca.x, y + 4);
 }
 
 void wait::hide_children(bool hide)
@@ -347,7 +311,7 @@ void wait::process_network_data(const config& data, const network::connection so
 			<< level_.get_children("side").size() << ","
 			<< data.get_children("side").size() << "\n";
 		generate_menu();
-	} 
+	}
 }
 
 void wait::generate_menu()
@@ -362,7 +326,7 @@ void wait::generate_menu()
 	for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
 		const config& sd = **s;
 
-		std::string description = sd["description"];
+		std::string description = sd["user_description"];
 		const std::string faction_id = sd["id"];
 
 		t_string side_name = sd["name"];
@@ -395,7 +359,7 @@ void wait::generate_menu()
 			// Dumps the "image" part of the faction name, if any,
 			// to replace it by a picture of the actual leader
 			if(side_name.str()[0] == font::IMAGE) {
-				std::string::size_type p = 
+				std::string::size_type p =
 					side_name.str().find_first_of(COLUMN_SEPARATOR);
 				if(p != std::string::npos && p < side_name.size()) {
 					side_name = IMAGE_PREFIX + leader_image + COLUMN_SEPARATOR + side_name.str().substr(p+1);
