@@ -7,6 +7,11 @@ class config;
 
 #include <string>
 
+namespace threading
+{
+	class waiter;
+}
+
 //this module wraps the network interface.
 
 namespace network {
@@ -19,7 +24,7 @@ struct manager {
 
 private:
 	bool free_;
-	
+
 	manager(const manager&);
 	void operator=(const manager&);
 };
@@ -29,7 +34,7 @@ private:
 struct server_manager {
 
 	//parameter to pass to the constructor.
-	
+
 	enum CREATE_SERVER { MUST_CREATE_SERVER, //will throw exception on failure
 	                     TRY_CREATE_SERVER, //will swallow failure
 	                     NO_SERVER }; //won't try to create a server at all
@@ -59,6 +64,8 @@ bool is_server();
 //throws error.
 connection connect(const std::string& host, int port=15000);
 
+connection connect(const std::string& host, int port, threading::waiter& waiter);
+
 //function to accept a connection from a remote host. If no
 //host is attempting to connect, it will return 0 immediately.
 //otherwise returns the new connection.
