@@ -1,7 +1,8 @@
-/* $Id: pathfind.cpp,v 1.65 2005/03/26 17:10:32 silene Exp $ */
+/* $Id: pathfind.cpp,v 1.71 2005/06/05 10:56:41 silene Exp $ */
 /*
-Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
@@ -29,30 +30,26 @@ class gamestatus;
 
 typedef std::vector<gamemap::location> vector_location;
 typedef std::vector<a_star_node*> vector_a_star_node;
-typedef std::set<gamemap::location>	set_location;	
+typedef std::set<gamemap::location> set_location;
 
-bool compare_strict_sup_a_star_node(const a_star_node* node1, const a_star_node* node2) {
-	return node1->g + node1->h > node2->g + node2->h;		
+// heaps give the biggest element for free, so we want the biggest element to
+// have the smallest cost
+static bool compare_lt_a_star_node(const a_star_node* node1, const a_star_node* node2) {
+	return node1->g + node1->h > node2->g + node2->h;
 }
 
-bool compare_sup_equal_a_star_node(const a_star_node* node1, const a_star_node* node2) {
-	return node1->g + node1->h >= node2->g + node2->h;		
-}
-
-void a_star_init(gamemap::location const &src, gamemap::location const &dst,
-								 vector_a_star_node& openList, a_star_world& aStarGameWorld,
-								 const size_t parWidth, const size_t parHeight,
-								 vector_location& vectLocation, std::set<gamemap::location> const *teleports,
-								 size_t& parNbTeleport)
+static void a_star_init(gamemap::location const &src, gamemap::location const &dst,
+                        vector_a_star_node &openList, a_star_world &aStarGameWorld,
+                        const size_t parWidth, const size_t parHeight,
+                        vector_location &vectLocation, std::set<gamemap::location> const *teleports,
+                        size_t &parNbTeleport)
 {
-	a_star_node*		locStartNode = NULL;
-	bool						locIsCreated = false;
-	
+	bool locIsCreated;
+
 	aStarGameWorld.resize_IFN(parWidth, parHeight);
-	wassert(aStarGameWorld.empty());
-	locStartNode = aStarGameWorld.getNodeFromLocation(src, locIsCreated);
+	a_star_node *locStartNode = aStarGameWorld.getNodeFromLocation(src, locIsCreated);
 	wassert(locIsCreated);
-	locStartNode->initNode(src, dst, 0.0, NULL, teleports);	
+	locStartNode->initNode(src, dst, 0.0, NULL, teleports);
 
 	const size_t locValueH = size_t(locStartNode->h);
 	size_t locAllocSize;
@@ -61,113 +58,49 @@ void a_star_init(gamemap::location const &src, gamemap::location const &dst,
 		locAllocSize = 16;
 	else if (locValueH > 128)
 		locAllocSize = 128;
-	else
-		locAllocSize = locValueH;	
-	openList.reserve(locAllocSize);
-	openList.push_back(locStartNode);	
-
-	if (locValueH < 32)
-		locAllocSize = 32;
-	else if (locValueH > 256)
-		locAllocSize = 256;
 	else
 		locAllocSize = locValueH;
-	
-	if (teleports != NULL) 
+	openList.reserve(locAllocSize);
+	openList.push_back(locStartNode);
+
+	if (teleports != NULL)
 		parNbTeleport = teleports->size();
 	else
 		parNbTeleport = 0;
 
 	vectLocation.reserve(parNbTeleport + 6);
-	vectLocation.resize(parNbTeleport + 6);	
+	vectLocation.resize(parNbTeleport + 6);
 
-	if (parNbTeleport > 0) 
-	{
-		gamemap::location* locLocation;
-
-		locLocation = &vectLocation[6];
-		for (set_location::const_iterator it = teleports->begin(); it != teleports->end(); ++it) {
-			locLocation->x = (*it).x;
-			locLocation->y = (*it).y;
-			++locLocation;
-		}		
-	}
+	if (parNbTeleport > 0)
+		std::copy(teleports->begin(), teleports->end(), &vectLocation[6]);
 }
 
-void a_star_sort_and_merge(vector_a_star_node& openList, const size_t locNbAdded)
+static void a_star_explore_neighbours(gamemap::location const &dst, const double stop_at,
+                                      cost_calculator const *costCalculator,
+                                      const size_t parWidth, const size_t parHeight,
+                                      std::set<gamemap::location> const *teleports,
+                                      vector_location &vectLocation, vector_a_star_node &openList,
+                                      a_star_world &aStarGameWorld,
+                                      a_star_node *parCurNode, const size_t parNbTeleport)
 {
-	std::stable_sort(openList.end() - locNbAdded, openList.end(), compare_strict_sup_a_star_node);
-
-#ifdef _PARANO_ASTAR_
-	for (vector_a_star_node::iterator iter = openList.begin(); iter != openList.end(); ++iter)
-		wassert((*iter)->isInCloseList == false);		
-	if (openList.size() > locNbAdded) {
-		for (vector_a_star_node::iterator iter = openList.begin() + 1; iter != openList.end() - locNbAdded; ++iter)
-		{
-			vector_a_star_node::iterator iterPrev = iter - 1;			
-			wassert(compare_sup_equal_a_star_node(*iterPrev, *iter));
-		}
-	}
-	if (locNbAdded > 0) {
-		for (vector_a_star_node::iterator iter = openList.end() - locNbAdded + 1; iter != openList.end(); ++iter)
-		{
-			vector_a_star_node::iterator iterPrev = iter - 1;			
-			wassert(compare_sup_equal_a_star_node(*iterPrev, *iter));
-		}
-	}
-#endif
-
-	std::inplace_merge(openList.begin(), openList.end() - locNbAdded, openList.end(), compare_strict_sup_a_star_node);
-
-#ifdef _PARANO_ASTAR_
-	if (openList.size() > 0) {
-		vector_a_star_node::iterator iter;
-		for (iter = openList.begin() + 1; iter != openList.end(); ++iter)
-		{
-			vector_a_star_node::iterator iterPrev = iter - 1;			
-			wassert(compare_sup_equal_a_star_node(*iterPrev, *iter));				
-		}
-		for (iter = openList.begin(); iter != openList.end(); ++iter)
-			wassert((*iter)->isInCloseList == false);
-	}
-#endif
-}
-
-void a_star_explore_neighbours(gamemap::location const &dst, const double stop_at,
-															 cost_calculator const *costCalculator,
-															 const size_t parWidth, const size_t parHeight,
-															 std::set<gamemap::location> const *teleports,
-															 vector_location& vectLocation, vector_a_star_node& openList,
-															 a_star_world& aStarGameWorld, size_t& locNbAdded,
-															 a_star_node*	parCurNode, const size_t parNbTeleport)
-{
-	//----------------- PRE_CONDITIONS ------------------
-	wassert(parCurNode != NULL);	
-	//---------------------------------------------------
-	
 	typedef std::pair<vector_a_star_node::iterator, vector_a_star_node::iterator> pair_node_iter;
 
-	a_star_node*	 locNextNode;
-	double				 locCost;
+	a_star_node *locNextNode;
+	double locCost;
 	pair_node_iter locPlace;
-	size_t				 locSize;
-	bool					 locIsCreated = false;
-	const double	 locCostFather = parCurNode->g;
+	size_t locSize;
+	bool locIsCreated;
+	const double locCostFather = parCurNode->g;
 
-	get_adjacent_tiles(parCurNode->loc, &vectLocation[0]);		
+	get_adjacent_tiles(parCurNode->loc, &vectLocation[0]);
 
-	if ((parNbTeleport > 0) && (teleports->count(parCurNode->loc) > 0))
-	{
-		wassert(teleports != NULL);
-		wassert(teleports->size() == parNbTeleport);
-		wassert(vectLocation.size() == parNbTeleport + 6);
+	if (parNbTeleport > 0 && teleports->count(parCurNode->loc) > 0)
 		locSize = parNbTeleport + 6;
-	}
 	else
-	{
-		wassert(vectLocation.size() >= 6);
 		locSize = 6;
-	}	
+
+	bool broken_heap = false;
+	int locNbAdded = 0;
 
 	for (size_t i = 0; i != locSize; ++i)
 	{
@@ -177,98 +110,82 @@ void a_star_explore_neighbours(gamemap::location const &dst, const double stop_a
 			continue;
 		locNextNode = aStarGameWorld.getNodeFromLocation(locLocation, locIsCreated);
 		locCost = locCostFather + costCalculator->cost(locLocation, locCostFather, locLocation == dst);
-		if (locIsCreated)
-		{
+		if (locIsCreated) {
 			locNextNode->initNode(locLocation, dst, locCost, parCurNode, teleports);
-			if (locNextNode->g + locNextNode->h < stop_at) {					
-				openList.push_back(locNextNode);
-				++locNbAdded;					
-			}
-			else
-			{
-				wassert(locNextNode->isInCloseList == false);
-				locNextNode->isInCloseList = true;
-			}
-		}
-		else
-		{
-			// If this path is better that the previous for this node
-			if (locCost < locNextNode->g)
-			{										
-				if (locNextNode->isInCloseList) {
-					locNextNode->isInCloseList = false;
-				}
-				else
-				{					
-					locPlace = std::equal_range(openList.begin(), openList.end() - locNbAdded, locNextNode, compare_strict_sup_a_star_node);
-					assertParanoAstar(*(std::find(locPlace.first, locPlace.second, locNextNode)) == locNextNode);
-					openList.erase(std::find(locPlace.first, locPlace.second, locNextNode));
-				}
+			if (locNextNode->g + locNextNode->h < stop_at) {
 				openList.push_back(locNextNode);
 				++locNbAdded;
-				locNextNode->g = locCost;
-				locNextNode->nodeParent = parCurNode;					
-			}
-		}												
+			} else
+				locNextNode->isInCloseList = true;
+
+		} else if (locCost < locNextNode->g) {
+
+			if (locNextNode->isInCloseList) {
+				locNextNode->isInCloseList = false;
+				openList.push_back(locNextNode);
+				++locNbAdded;
+			} else
+				broken_heap = true;
+
+			locNextNode->g = locCost;
+			locNextNode->nodeParent = parCurNode;
+		}
 	}
+
+	vector_a_star_node::iterator openList_begin = openList.begin(),
+	                             openList_end = openList.end();
+	if (broken_heap)
+		std::make_heap(openList_begin, openList_end, compare_lt_a_star_node);
+	else
+		for(; locNbAdded > 0; --locNbAdded)
+			std::push_heap(openList_begin, openList_end - (locNbAdded - 1), compare_lt_a_star_node);
 }
 
-paths::route a_star_search( gamemap::location const &src, gamemap::location const &dst,
-													  double stop_at, cost_calculator const *costCalculator, const size_t parWidth, 
-														const size_t parHeight, std::set<gamemap::location> const *teleports)
+paths::route a_star_search(gamemap::location const &src, gamemap::location const &dst,
+                           double stop_at, cost_calculator const *costCalculator, const size_t parWidth,
+                           const size_t parHeight, std::set<gamemap::location> const *teleports)
 {
 	//----------------- PRE_CONDITIONS ------------------
-	wassert(parWidth > 0);
-	wassert(parHeight > 0);
-	wassert(src.valid());
-	wassert(dst.valid());
+	wassert(src.valid(parWidth, parHeight));
+	wassert(dst.valid(parWidth, parHeight));
 	wassert(costCalculator != NULL);
 	wassert(stop_at <= costCalculator->getNoPathValue());
-	//---------------------------------------------------			
-	static a_star_world				aStarGameWorld;	
-	static poss_a_star_node	  POSS_AStarNode;
-	
-	vector_a_star_node			openList;	
-	vector_location					vectLocation;
-	paths::route						locRoute;
-	size_t									locNbTeleport;	
-	size_t									locNbAdded;	
-	a_star_node*						locDestNode = NULL;
-	a_star_node*						locCurNode = NULL;
-	
-	wassert(openList.empty());	
-	wassert(aStarGameWorld.empty());
-	assertParanoAstar(aStarGameWorld.reallyEmpty());	
-	
-	LOG_PF << "A* search: " << src.x << ", " << src.y << " -> " << dst.x << ", " << dst.y << "\n";
+	//---------------------------------------------------
+	static a_star_world aStarGameWorld;
 
-	if ( (src.valid(int(parWidth), int(parHeight)) == false) ||
-			 (dst.valid(int(parWidth), int(parHeight)) == false) ||
-			 (costCalculator->cost(dst, 0, true) >= stop_at))
-	{
+	vector_a_star_node openList;
+	vector_location vectLocation;
+	paths::route locRoute;
+	size_t locNbTeleport;
+	a_star_node *locDestNode = NULL;
+	a_star_node *locCurNode = NULL;
+
+	LOG_PF << "A* search: " << src << " -> " << dst << '\n';
+
+	if (costCalculator->cost(dst, 0, true) >= stop_at) {
 		LOG_PF << "aborted A* search because Start or Dest is invalid\n";
 		locRoute.move_left = int(costCalculator->getNoPathValue());
-		return (locRoute);
-	}	
+		return locRoute;
+	}
 
 	a_star_init(src, dst, openList, aStarGameWorld, parWidth, parHeight, vectLocation, teleports, locNbTeleport);
 
-	while (!openList.empty()) 
+	while (!openList.empty())
 	{
-		locCurNode = openList.back();
-		wassert(locCurNode != NULL);		
-		
+		locCurNode = openList.front();
+		wassert(locCurNode != NULL);
+
 		//if we have found a solution
 		if (locCurNode->loc == dst)
 		{
-			locDestNode = locCurNode;			
-			
+			locDestNode = locCurNode;
+
 			LOG_PF << "found solution; calculating it...\n";
 			while (locCurNode != NULL)
 			{
 				locRoute.steps.push_back(locCurNode->loc);
 				locCurNode = locCurNode->nodeParent;
-			}	
+			}
 			std::reverse(locRoute.steps.begin(), locRoute.steps.end());
 			locRoute.move_left = int(locDestNode->g);
 
@@ -279,32 +196,24 @@ paths::route a_star_search( gamemap::location const &src, gamemap::location cons
 			goto label_AStarSearch_end;
 		}
 
+		std::pop_heap(openList.begin(), openList.end(), compare_lt_a_star_node);
 		openList.pop_back();
 
 		wassert(locCurNode->isInCloseList == false);
-		locCurNode->isInCloseList = true ;		
-		
-		locNbAdded = 0;
+		locCurNode->isInCloseList = true;
+
 		a_star_explore_neighbours(dst, stop_at, costCalculator, parWidth, parHeight, teleports, vectLocation,
-															openList, aStarGameWorld, locNbAdded, locCurNode, locNbTeleport);
+		                          openList, aStarGameWorld, locCurNode, locNbTeleport);
 
-		a_star_sort_and_merge(openList, locNbAdded);
 	}
-	
+
 	LOG_PF << "aborted a* search\n";
 	locRoute.move_left = int(costCalculator->getNoPathValue());
-	
+
 label_AStarSearch_end:
 	openList.clear();
-	POSS_AStarNode.reduce();
 	aStarGameWorld.clear();
-
-	//----------------- POST_CONDITIONS -----------------
-	wassert(openList.empty());
-	wassert(aStarGameWorld.empty());
-	assertParanoAstar(aStarGameWorld.reallyEmpty());	
-	//---------------------------------------------------
-	return (locRoute);
+	return locRoute;
 }
 
 namespace {
@@ -387,8 +296,8 @@ void get_tiles_radius(const gamemap::location& a, size_t radius, std::set<gamema
 	get_tiles_radius_internal(a,radius,res,visited);
 }
 
-void get_tiles_radius(const gamemap& map, const std::vector<gamemap::location>& locs, size_t radius,
-											std::set<gamemap::location>& res)
+void get_tiles_radius(gamemap const &map, std::vector<gamemap::location> const &locs,
+                      size_t radius, std::set<gamemap::location> &res)
 {
 	typedef std::set<gamemap::location> location_set;
 	location_set not_visited(locs.begin(), locs.end()), must_visit;
@@ -417,10 +326,10 @@ void get_tiles_radius(const gamemap& map, const std::vector<gamemap::location>&
 	}
 }
 
-bool enemy_zoc(const gamemap& map, const gamestatus& status, 
-							 const std::map<gamemap::location,unit>& units,
-							 const std::vector<team>& teams,
-							 const gamemap::location& loc, const team& current_team, int side)
+bool enemy_zoc(gamemap const &map, gamestatus const &status,
+               std::map<gamemap::location, unit> const &units,
+               std::vector<team> const &teams,
+               gamemap::location const &loc, team const &current_team, int side)
 {
 	gamemap::location locs[6];
 	get_adjacent_tiles(loc,locs);
@@ -448,14 +357,14 @@ namespace {
 		const gamemap::location& loc,
 		int move_left,
 		std::map<gamemap::location,paths::route>& routes,
-		std::vector<team>& teams,
+		std::vector<team> const &teams,
 		bool ignore_zocs, bool allow_teleport, int turns_left, bool starting_pos)
 	{
 		if(size_t(u.side()-1) >= teams.size()) {
 			return;
 		}
 
-		team& current_team = teams[u.side()-1];
+		team const &current_team = teams[u.side()-1];
 
 		//find adjacent tiles
 		std::vector<gamemap::location> locs(6);
@@ -540,12 +449,12 @@ namespace {
 
 } //end anon namespace
 
-paths::paths(const gamemap& map, const gamestatus& status,
-						 const game_data& gamedata,
-						 const std::map<gamemap::location,unit>& units,
-						 const gamemap::location& loc,
-						 std::vector<team>& teams,
-						 bool ignore_zocs, bool allow_teleport, int additional_turns)
+paths::paths(gamemap const &map, gamestatus const &status,
+             game_data const &gamedata,
+             std::map<gamemap::location, unit> const &units,
+             gamemap::location const &loc,
+             std::vector<team> const &teams,
+             bool ignore_zocs, bool allow_teleport, int additional_turns)
 {
 	const std::map<gamemap::location,unit>::const_iterator i = units.find(loc);
 	if(i == units.end()) {
@@ -559,8 +468,7 @@ paths::paths(const gamemap& map, const gamestatus& status,
 		ignore_zocs,allow_teleport,additional_turns,true);
 }
 
-int route_turns_to_complete(const unit& u, const gamemap& map,
-														const paths::route& rt)
+int route_turns_to_complete(unit const &u, gamemap const &map, paths::route const &rt)
 {
 	if(rt.steps.empty())
 		return 0;
@@ -584,13 +492,14 @@ int route_turns_to_complete(const unit& u, const gamemap& map,
 }
 
 
-shortest_path_calculator::shortest_path_calculator(const unit& u, const team& t,
-																									 const unit_map& units,
-																									 const std::vector<team>& teams,
-																									 const gamemap& map,
-																									 const gamestatus& status)
-																									 : unit_(u), team_(t), units_(units), teams_(teams), 
-																									 map_(map), status_(status)
+shortest_path_calculator::shortest_path_calculator(unit const &u, team const &t, unit_map const &units,
+                                                   std::vector<team> const &teams, gamemap const &map,
+                                                   gamestatus const &status)
+	: unit_(u), team_(t), units_(units), teams_(teams), map_(map),
+	  lawful_bonus_(status.get_time_of_day().lawful_bonus),
+	  unit_is_skirmisher_(unit_.type().is_skirmisher()),
+	  movement_left_(unit_.movement_left()),
+	  total_movement_(unit_.total_movement())
 {
 }
 
@@ -598,46 +507,51 @@ double shortest_path_calculator::cost(const gamemap::location& loc, const double
 {
 	wassert(map_.on_board(loc));
 
+	//the location is not valid
+	//1. if the loc is shrouded, or
+	//2. if moving in it costs more than the total movement of the unit, or
+	//3. if there is a visible enemy on the hex, or
+	//4. if the unit is not a skirmisher and there is a visible enemy with
+	//   a ZoC on an adjacent hex in the middle of the route
+
 	if (team_.shrouded(loc.x, loc.y))
-		return (getNoPathValue());
+		return getNoPathValue();
 
-	const unit_map::const_iterator enemy_unit = find_visible_unit(units_, loc, map_, status_.get_time_of_day().lawful_bonus, teams_, team_);
+	int const base_cost = unit_.movement_cost(map_, map_[loc.x][loc.y]);
+	wassert(base_cost >= 1); // pathfinding heuristic: the cost must be at least 1
+	if (total_movement_ < base_cost)
+		return getNoPathValue();
 
-	if (enemy_unit != units_.end() && team_.is_enemy(enemy_unit->second.side()))
-		return (getNoPathValue());
+	unit_map::const_iterator
+		enemy_unit = find_visible_unit(units_, loc, map_, lawful_bonus_, teams_, team_),
+		units_end = units_.end();
 
-	if ((isDst == false) && (unit_.type().is_skirmisher() == false)) 
-	{
+	if (enemy_unit != units_end && team_.is_enemy(enemy_unit->second.side()))
+		return getNoPathValue();
+
+	if (!isDst && !unit_is_skirmisher_) {
 		gamemap::location adj[6];
-		get_adjacent_tiles(loc,adj);
+		get_adjacent_tiles(loc, adj);
 
 		for (size_t i = 0; i != 6; ++i) {
-			const unit_map::const_iterator u = find_visible_unit(units_, adj[i],map_, status_.get_time_of_day().lawful_bonus, teams_, team_);
+			enemy_unit = find_visible_unit(units_, adj[i], map_, lawful_bonus_, teams_, team_);
 
-			if (u != units_.end() && team_.is_enemy(u->second.side()) &&
-			    !team_.fogged(adj[i].x, adj[i].y) && u->second.emits_zoc()) {
-				return (getNoPathValue());
-			}
+			if (enemy_unit != units_end && team_.is_enemy(enemy_unit->second.side()) &&
+			    !team_.fogged(adj[i].x, adj[i].y) && enemy_unit->second.emits_zoc())
+				return getNoPathValue();
 		}
 	}
 
-	int const base_cost = unit_.movement_cost(map_, map_[loc.x][loc.y]);
+	//compute how many movement points are left in the game turn needed to
+	//reach the previous hex
+	//total_movement_ is not zero, thanks to the pathfinding heuristic
+	int remaining_movement = movement_left_ - static_cast<int>(so_far);
+	if (remaining_movement < 0)
+		remaining_movement = total_movement_ - (-remaining_movement) % total_movement_;
 
 	//supposing we had 2 movement left, and wanted to move onto a hex which
 	//takes 3 movement, it's going to cost us 5 movement in total, since we
 	//sacrifice this turn's movement. Take that into account here.
-	const int current_cost(static_cast<int>(so_far));
-
-	const int starting_movement = unit_.movement_left();
-	int remaining_movement = starting_movement - current_cost;
-	if (remaining_movement < 0) {
-		const int total = unit_.total_movement();
-		if (total != 0) {
-			remaining_movement = total - (-remaining_movement) % total;
-		}
-	}
-
 	int additional_cost = base_cost > remaining_movement ? remaining_movement : 0;
-
 	return base_cost + additional_cost;
 }
