@@ -1,7 +1,7 @@
-/* $Id: playlevel.cpp,v 1.199 2005/03/28 19:08:58 gruikya Exp $ */
+/* $Id: playlevel.cpp,v 1.209 2005/06/18 22:13:09 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -104,7 +104,7 @@ namespace {
 				placed.insert(i->side);
 				positions_taken.insert(i->pos);
 				map.set_starting_position(i->side,i->pos);
-				LOG_NG << "placing side " << i->side << " at " << i->pos.x << "," << i->pos.y << "\n";
+				LOG_NG << "placing side " << i->side << " at " << i->pos << '\n';
 			}
 		}
 	}
@@ -122,7 +122,7 @@ namespace {
 }
 
 LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
-		config* level, CVideo& video,
+		config const* level, CVideo& video,
 		game_state& state_of_game,
 		const std::vector<config*>& story)
 {
@@ -146,14 +146,14 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 	if((*level)["scenario_generation"] != "") {
 		LOG_NG << "randomly generating scenario...\n";
 		const cursor::setter cursor_setter(cursor::WAIT);
-	
+
 		static config scenario;
 		scenario = random_generate_scenario((*level)["scenario_generation"],level->child("generator"));
 		level = &scenario;
 
 		state_of_game.starting_pos = scenario;
 	}
-	
+
 	std::string map_data = (*level)["map_data"];
 	if(map_data == "" && (*level)["map"] != "") {
 		map_data = read_map((*level)["map"]);
@@ -262,6 +262,15 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 
 		teams.push_back(team(**ui,ngold));
 
+		//update/fix the recall list for this side, by setting the
+		//"side" of each unit in it to be the "side" of the player.
+		int side = lexical_cast_default<int>((**ui)["side"], 1);
+		if(player != NULL) {
+			for(std::vector<unit>::iterator it = player->available_units.begin(); it != player->available_units.end(); ++it) {
+				it->set_side(side);
+			}
+		}
+
 		//if this team has no objectives, set its objectives to the
 		//level-global "objectives"
 		if(teams.back().objectives().empty())
@@ -289,9 +298,9 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 			gamemap::location start_pos(**ui);
 
 			if(has_loc.empty()) {
-				start_pos = map.starting_position(new_unit.side());
+				start_pos = map.starting_position(side);
 				LOG_NG << "initializing side '" << (**ui)["side"] << "' at "
-				       << start_pos.x << "," << start_pos.y << "\n";
+				       << start_pos << '\n';
 			}
 
 			if(map.empty()) {
@@ -314,17 +323,17 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 			std::copy(player->can_recruit.begin(),player->can_recruit.end(),
 					std::inserter(teams.back().recruits(),teams.back().recruits().end()));
 		}
-		
+
 		if(player != NULL) {
 			player->can_recruit = teams.back().recruits();
 		}
-		
+
 		//if there are additional starting units on this side
 		const config::child_list& starting_units = (*ui)->get_children("unit");
 		for(config::child_list::const_iterator su = starting_units.begin(); su != starting_units.end(); ++su) {
 			unit new_unit(gameinfo,**su);
 
-			new_unit.set_side(lexical_cast_default<int>((**ui)["side"],1));
+			new_unit.set_side(side);
 
 			const std::string& x = (**su)["x"];
 			const std::string& y = (**su)["y"];
@@ -355,10 +364,10 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 			LOG_NG << *it << std::endl;
 		}
 		LOG_NG << "Added all recruitable units\n";
-		std::copy(recruitable.begin(), recruitable.end(), 
+		std::copy(recruitable.begin(), recruitable.end(),
 				  std::inserter(enc_units, enc_units.begin()));
 	}
-		
+
 	// Add all units that exist at the start to the encountered units so
 	// that information about them are displayed to the user in the help
 	// system.
@@ -414,7 +423,7 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 	LOG_NG << "b... " << (SDL_GetTicks() - ticks) << "\n";
 
 	//this *needs* to be created before the show_intro and show_map_scene
-	//as that functions use the manager state_of_game 
+	//as that functions use the manager state_of_game
 	game_events::manager events_manager(*level,gui,map,units,teams,
 	                                    state_of_game,status,gameinfo);
 
@@ -463,50 +472,36 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 		//if a team is specified whose turn it is, it means we're loading a game
 		//instead of starting a fresh one
 		const bool loading_game = lvl["playing_team"].empty() == false;
-		
+
 		gui.begin_game();
 		gui.adjust_colours(0,0,0);
-		
 
 		LOG_NG << "scrolling... " << (SDL_GetTicks() - ticks) << "\n";
 		if(first_human_team != -1) {
-			clear_shroud(gui,status,map,gameinfo,units,teams,first_human_team);
 			LOG_NG << "b " << (SDL_GetTicks() - ticks) << "\n";
 			gui.scroll_to_tile(map.starting_position(first_human_team + 1).x,
 			                   map.starting_position(first_human_team + 1).y, display::WARP);
 			LOG_NG << "c " << (SDL_GetTicks() - ticks) << "\n";
 		}
-	
 		gui.scroll_to_tile(map.starting_position(1).x,map.starting_position(1).y,display::WARP);
 		LOG_NG << "done scrolling... " << (SDL_GetTicks() - ticks) << "\n";
 
 		bool replaying = (recorder.at_end() == false);
-	
+
 		int first_player = atoi(lvl["playing_team"].c_str());
 		if(first_player < 0 || first_player >= int(teams.size())) {
 			first_player = 0;
 		}
-		
-		events::raise_draw_event();
+
+		if(!loading_game) {
+			game_events::fire("prestart");
+		}
 		for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
 			clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
 		}
-		gui.recalculate_minimap();
-		gui.draw();
-		
-		if(!loading_game) {
-			game_events::fire("prestart");
-			events::raise_draw_event();
-			gui.draw();
-			for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
-				clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
-			}
-			gui.recalculate_minimap();
-		}
-		
 
 		std::deque<config> data_backlog;
-		
+
 		LOG_NG << "starting main loop\n" << (SDL_GetTicks() - ticks) << "\n";
 		for(bool first_time = true; true; first_time = false, first_player = 0) {
 			if(first_time) {
@@ -515,15 +510,16 @@ LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 				LOG_NG << "first_time..." << (recorder.skipping() ? "skipping" : "no skip") << "\n";
 				update_locker lock_display(gui.video(),recorder.skipping());
 				events::raise_draw_event();
+				gui.draw();
+				for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
+					clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
+				}
+
 				if(!loading_game) {
 					game_events::fire("start");
 					state_of_game.set_variable("turn_number", "1");
 				}
-				gui.draw();
 
-				for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
-					clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
-				}
 				gui.recalculate_minimap();
 			}
 			player_number = 0;
@@ -649,6 +645,8 @@ redo_turn:
 					ai_interface::info ai_info(gui,map,gameinfo,units,teams,player_number,status,turn_data);
 					util::scoped_ptr<ai_interface> ai_obj(create_ai(team_it->ai_algorithm(),ai_info));
 					ai_obj->play_turn();
+					recorder.end_turn();
+					ai_obj->sync_network();
 
 					gui.recalculate_minimap();
 					clear_shroud(gui,status,map,gameinfo,units,teams,player_number-1);
@@ -709,7 +707,7 @@ redo_turn:
 					gui.recalculate_minimap();
 					gui.invalidate_all();
 				}
-				
+
 				game_events::pump();
 
 				check_victory(units,teams);
@@ -717,7 +715,7 @@ redo_turn:
 
 			//time has run out
 			if(!status.next_turn()) {
-			
+
 				if(non_interactive()) {
 					std::cout << "time over (draw)\n";
 				}
@@ -753,9 +751,11 @@ redo_turn:
 				info["type"] = "termination";
 				info["condition"] = "game over";
 				network::send_data(cfg);
-			} else
+			} else {
 				gui::show_dialog(gui, NULL, _("Game Over"),
 				                 _("The game is over."), gui::OK_ONLY);
+				return QUIT;
+			}
 		}
 
 		if(end_level.result == QUIT) {
@@ -853,7 +853,7 @@ redo_turn:
 					}
 
 					// xgettext:no-c-format
-					report << "\n" << _("80% of gold is retained for the next scenario") << "\n"
+					report << '\n' << _("80% of gold is retained for the next scenario") << '\n'
 					       << _("Retained Gold: ") << player->gold;
 				}
 			}
@@ -862,7 +862,8 @@ redo_turn:
 				gui::show_dialog(gui, NULL, _("Victory"),
 				                 _("You have emerged victorious!"), gui::OK_ONLY);
 
-			if (state_of_game.players.size() > 0 && has_next_scenario)
+			if (state_of_game.players.size() > 0 && has_next_scenario ||
+					state_of_game.campaign_type == "test")
 				gui::show_dialog(gui, NULL, _("Scenario Report"), report.str(), gui::OK_ONLY);
 
 			return VICTORY;
