@@ -1,7 +1,7 @@
-/* $Id: playlevel.hpp,v 1.22 2005/03/22 23:42:36 ydirson Exp $ */
+/* $Id: playlevel.hpp,v 1.24 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -38,7 +38,7 @@ struct end_turn_exception {
 };
 
 LEVEL_RESULT play_level(const game_data& gameinfo, const config& terrain_config,
-		config* level, CVideo& video,
+		config const* level, CVideo& video,
 		game_state& state_of_game,
 		const std::vector<config*>& story);
 
