@@ -1,7 +1,7 @@
-/* $Id: playturn.cpp,v 1.357 2005/04/10 11:53:15 isaaccp Exp $ */
+/* $Id: playturn.cpp,v 1.383 2005/06/21 22:19:19 ydirson Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -110,7 +110,7 @@ void play_turn(const game_data& gameinfo, game_state& state_of_game,
 		unit_map::const_iterator ui = units.find(*g);
 		turn_data.move_unit_to_loc(ui,ui->second.get_goto(),false);
 	}
-	
+
 	turn_data.start_interactive_turn();
 
 	while(!turn_data.turn_over()) {
@@ -141,7 +141,7 @@ void play_turn(const game_data& gameinfo, game_state& state_of_game,
 }
 
 turn_info::turn_info(const game_data& gameinfo, game_state& state_of_game,
-                     const gamestatus& status, const config& terrain_config, 
+                     const gamestatus& status, const config& terrain_config,
 		     const config& level, CKey& key, display& gui, gamemap& map,
 		     std::vector<team>& teams, int team_num, unit_map& units,
 		     TURN_MODE mode, floating_textbox& textbox,
@@ -188,7 +188,7 @@ void turn_info::turn_slice()
 	const bool use_left_right = (textbox_.active() == false);
 	if(use_left_right && key_[SDLK_LEFT] || mousex < scroll_threshold)
 		gui_.scroll(-preferences::scroll_speed(),0);
-	
+
 	if(use_left_right && key_[SDLK_RIGHT] || mousex > gui_.x()-scroll_threshold)
 		gui_.scroll(preferences::scroll_speed(),0);
 
@@ -347,7 +347,7 @@ void turn_info::mouse_motion(int x, int y)
 			current_paths_ = paths();
 			gui_.set_paths(NULL);
 		}
-		
+
 		const gamemap::location& dest = attack_from.valid() ? attack_from : new_hex;
 		const unit_map::const_iterator dest_un = find_unit(dest);
 		if(dest == selected_hex_ || dest_un != units_.end()) {
@@ -355,9 +355,9 @@ void turn_info::mouse_motion(int x, int y)
 			gui_.set_route(NULL);
 		} else if(!current_paths_.routes.empty() && map_.on_board(selected_hex_) &&
 		   map_.on_board(new_hex)) {
-			
+
 			unit_map::const_iterator un = find_unit(selected_hex_);
-			
+
 			if((new_hex != last_hex_ || attack_from.valid()) && un != units_.end() && !un->second.stone()) {
 				const shortest_path_calculator calc(un->second,current_team,
 				                                    visible_units(),teams_,map_,status_);
@@ -408,7 +408,7 @@ namespace {
 bool command_active()
 {
 #ifdef __APPLE__
-	return (SDL_GetModState()&KMOD_LMETA) != 0;
+	return (SDL_GetModState()&KMOD_META) != 0;
 #else
 	return false;
 #endif
@@ -436,7 +436,7 @@ class simple_attack_rating
 public:
 	simple_attack_rating() : attacker_weapon_rating_(0), defender_weapon_rating_(0) {}
 
-	simple_attack_rating(const battle_stats& stats) : 
+	simple_attack_rating(const battle_stats& stats) :
 		attacker_weapon_rating_(stats.chance_to_hit_defender *
 				stats.damage_defender_takes * stats.nattacks),
 		defender_weapon_rating_(stats.chance_to_hit_attacker *
@@ -448,7 +448,7 @@ public:
 		if(attacker_weapon_rating_ < a.attacker_weapon_rating_)
 			return true;
 
-		//if both weapons are the same but 
+		//if both weapons are the same but
 		//ours makes the enemy retaliate for more damage, it's worse
 		else if(attacker_weapon_rating_ == a.attacker_weapon_rating_ &&
 		   defender_weapon_rating_ > a.defender_weapon_rating_)
@@ -510,8 +510,8 @@ void turn_info::mouse_press(const SDL_MouseButtonEvent& event)
 
 		gui_.scroll(xdisp,ydisp);
 		}
-	} else if(event.button == SDL_BUTTON_WHEELUP ||
-	          event.button == SDL_BUTTON_WHEELDOWN) {
+	} else if((event.button == SDL_BUTTON_WHEELUP ||
+	          event.button == SDL_BUTTON_WHEELDOWN) && !commands_disabled) {
 		const int speed = preferences::scroll_speed() *
 			(event.button == SDL_BUTTON_WHEELUP ? -1:1);
 
@@ -737,7 +737,7 @@ bool turn_info::move_unit_along_current_route(bool check_shroud)
 
 	selected_hex_ = gamemap::location();
 	gui_.select_hex(gamemap::location());
-	
+
 	gui_.set_route(NULL);
 	gui_.set_paths(NULL);
 	current_paths_ = paths();
@@ -758,10 +758,10 @@ bool turn_info::move_unit_along_current_route(bool check_shroud)
 			selected_hex_ = dst;
 			gui_.select_hex(dst);
 		}
-		
+
 		current_route_.steps.clear();
 		show_attack_options(u);
-		
+
 		if(current_paths_.routes.empty() == false) {
 			current_paths_.routes[dst] = paths::route();
 			selected_hex_ = dst;
@@ -1036,10 +1036,12 @@ bool turn_info::can_execute_command(hotkey::HOTKEY_COMMAND command) const
 	//commands we can always do
 	case hotkey::HOTKEY_LEADER:
 	case hotkey::HOTKEY_CYCLE_UNITS:
+	case hotkey::HOTKEY_CYCLE_BACK_UNITS:
 	case hotkey::HOTKEY_ZOOM_IN:
 	case hotkey::HOTKEY_ZOOM_OUT:
 	case hotkey::HOTKEY_ZOOM_DEFAULT:
 	case hotkey::HOTKEY_FULLSCREEN:
+	case hotkey::HOTKEY_SCREENSHOT:
 	case hotkey::HOTKEY_ACCELERATED:
 	case hotkey::HOTKEY_TOGGLE_GRID:
 	case hotkey::HOTKEY_STATUS_TABLE:
@@ -1052,10 +1054,8 @@ bool turn_info::can_execute_command(hotkey::HOTKEY_COMMAND command) const
 	case hotkey::HOTKEY_SEARCH:
 	case hotkey::HOTKEY_HELP:
 	case hotkey::HOTKEY_USER_CMD:
-		return true;
-
 	case hotkey::HOTKEY_SAVE_GAME:
-		return !commands_disabled;
+		return true;
 
 	case hotkey::HOTKEY_LOAD_GAME:
 		return network::nconnections() == 0; //can only load games if not in a network game
@@ -1085,13 +1085,14 @@ bool turn_info::can_execute_command(hotkey::HOTKEY_COMMAND command) const
 		if (i == units_.end()) return false;
 		return i->second.move_interrupted();
 	}
-	
+
 	case hotkey::HOTKEY_DELAY_SHROUD:
 		return !browse_ && (current_team().uses_fog() || current_team().uses_shroud());
 	case hotkey::HOTKEY_UPDATE_SHROUD:
 		return !browse_ && !commands_disabled && current_team().auto_shroud_updates() == false;
 
 	//commands we can only do if we are actually playing, not just viewing
+	case hotkey::HOTKEY_UNIT_HOLD_POSITION:
 	case hotkey::HOTKEY_END_UNIT_TURN:
 	case hotkey::HOTKEY_RECRUIT:
 	case hotkey::HOTKEY_REPEAT_RECRUIT:
@@ -1132,7 +1133,7 @@ namespace {
 	private:
 		const turn_info& info_;
 	};
-	
+
 	struct not_in_context_menu {
 		not_in_context_menu(const turn_info& info) : info_(info) {}
 		bool operator()(const std::string& str) const {
@@ -1170,7 +1171,7 @@ void turn_info::show_menu(const std::vector<std::string>& items_arg, int xloc, i
 			has_image = true;
 			str << IMAGE_PREFIX << img << COLUMN_SEPARATOR;
 		}
-		
+
 		str << hk.get_description() << COLUMN_SEPARATOR << hk.get_name();
 
 		menu.push_back(str.str());
@@ -1199,14 +1200,12 @@ bool turn_info::unit_in_cycle(unit_map::const_iterator it) const
 	}
 
 	return false;
-	
+
 }
 
 void turn_info::cycle_units()
 {
-
 	unit_map::const_iterator it = units_.find(next_unit_);
-	unit_map::const_iterator yellow_it = units_.end();
 	if(it != units_.end()) {
 		for(++it; it != units_.end(); ++it) {
 			if(unit_in_cycle(it)) {
@@ -1245,6 +1244,47 @@ void turn_info::cycle_units()
 	}
 }
 
+void turn_info::cycle_back_units()
+{
+	unit_map::const_iterator it = units_.find(next_unit_);
+	if(it != units_.begin()) {
+		for(--it; it != units_.begin(); --it) {
+			if(unit_in_cycle(it)) {
+				break;
+			}
+		}
+	}
+
+	if(it == units_.begin()) {
+		for(it = units_.end(); it != units_.begin(); --it) {
+			if(unit_in_cycle(it)) {
+				break;
+			}
+		}
+	}
+
+	if(it != units_.begin() && !gui_.fogged(it->first.x,it->first.y)) {
+		const bool ignore_zocs = it->second.type().is_skirmisher();
+		const bool teleport = it->second.type().teleports();
+		current_paths_ = paths(map_,status_,gameinfo_,units_,it->first,teams_,ignore_zocs,teleport,path_turns_);
+		gui_.set_paths(&current_paths_);
+
+		gui_.scroll_to_tile(it->first.x,it->first.y,display::WARP);
+	}
+
+	if(it != units_.begin()) {
+		next_unit_ = it->first;
+		selected_hex_ = next_unit_;
+		gui_.select_hex(selected_hex_);
+		gui_.highlight_hex(selected_hex_);
+		current_route_.steps.clear();
+		gui_.set_route(NULL);
+		show_attack_options(it);
+	} else {
+		next_unit_ = gamemap::location();
+	}
+}
+
 void turn_info::end_turn()
 {
 	if(browse_)
@@ -1272,7 +1312,7 @@ void turn_info::end_turn()
 			}
 		}
 	}
-	
+
 	// Ask for confirmation if units still have movement left
 	if(preferences::yellow_confirm() && partmoved_units) {
 		const int res = gui::show_dialog(gui_,NULL,"",_("Some units have movement left. Do you really want to end your turn?"),gui::YES_NO);
@@ -1313,6 +1353,28 @@ void turn_info::goto_leader()
 	}
 }
 
+void turn_info::unit_hold_position()
+{
+	if(browse_)
+		return;
+
+	const unit_map::iterator un = units_.find(selected_hex_);
+	if(un != units_.end() && un->second.side() == team_num_ && un->second.movement_left() >= 0) {
+		un->second.set_hold_position(!un->second.hold_position());
+		gui_.draw_tile(selected_hex_.x,selected_hex_.y);
+
+		gui_.set_route(NULL);
+		gui_.set_paths(NULL);
+		current_paths_ = paths();
+		gui_.draw();
+
+		if(un->second.hold_position()) {
+			un->second.set_user_end_turn(true);
+			cycle_units();
+		}
+	}
+}
+
 void turn_info::end_unit_turn()
 {
 	if(browse_)
@@ -1321,6 +1383,9 @@ void turn_info::end_unit_turn()
 	const unit_map::iterator un = units_.find(selected_hex_);
 	if(un != units_.end() && un->second.side() == team_num_ && un->second.movement_left() >= 0) {
 		un->second.set_user_end_turn(!un->second.user_end_turn());
+		if(un->second.hold_position() && !un->second.user_end_turn()){
+		  un->second.set_hold_position(false);
+		}
 		gui_.draw_tile(selected_hex_.x,selected_hex_.y);
 
 		gui_.set_route(NULL);
@@ -1375,18 +1440,21 @@ void turn_info::undo()
 			LOG_STREAM(err, engine) << "Illegal 'undo' found. Possible abuse of [allow_undo]?\n";
 			return;
 		}
-	
+
 		if(map_.is_village(route.front())) {
 			get_village(route.front(),teams_,action.original_village_owner,units_);
 		}
-	
+
 		action.starting_moves = u->second.movement_left();
-	
+
 		unit un = u->second;
 		un.set_goto(gamemap::location());
-		units_.erase(u);
-		unit_display::move_unit(gui_,map_,route,un,status_.get_time_of_day(),units_,teams_);
 
+		gui_.hide_unit(u->first,true);
+		unit_display::move_unit(gui_,map_,route,un,status_.get_time_of_day(),units_,teams_);
+		gui_.hide_unit(gamemap::location());
+
+		units_.erase(u);
 		un.set_movement(starting_moves);
 		units_.insert(std::pair<gamemap::location,unit>(route.back(),un));
 		gui_.draw_tile(route.back().x,route.back().y);
@@ -1440,14 +1508,20 @@ void turn_info::redo()
 			// Redo recall
 			std::vector<unit>& recall_list = player->available_units;
 			unit& un = recall_list[action.recall_pos];
-			recruit_unit(map_,team_num_,units_,un,action.recall_loc,&gui_);
-			statistics::recall_unit(un);
-			team& current_team = teams_[team_num_-1];
-			current_team.spend_gold(game_config::recall_cost);
-			recall_list.erase(recall_list.begin()+action.recall_pos);
-			recorder.add_recall(action.recall_pos,action.recall_loc);
 
-			gui_.draw_tile(action.recall_loc.x,action.recall_loc.y);
+			recorder.add_recall(action.recall_pos,action.recall_loc);
+			const std::string& msg = recruit_unit(map_,team_num_,units_,un,action.recall_loc,&gui_);
+			if(msg.empty()) {
+				statistics::recall_unit(un);
+				team& current_team = teams_[team_num_-1];
+				current_team.spend_gold(game_config::recall_cost);
+				recall_list.erase(recall_list.begin()+action.recall_pos);
+
+				gui_.draw_tile(action.recall_loc.x,action.recall_loc.y);
+			} else {
+				recorder.undo();
+				gui::show_dialog(gui_,NULL,"",msg,gui::OK_ONLY);
+			}
 		}
 	} else {
 		// Redo movement action
@@ -1458,22 +1532,26 @@ void turn_info::redo()
 			wassert(false);
 			return;
 		}
-	
+
 		action.starting_moves = u->second.movement_left();
-	
+
 		unit un = u->second;
 		un.set_goto(gamemap::location());
-		units_.erase(u);
+
+		gui_.hide_unit(u->first,true);
 		unit_display::move_unit(gui_,map_,route,un,status_.get_time_of_day(),units_,teams_);
+		gui_.hide_unit(gamemap::location());
+
+		units_.erase(u);
 		un.set_movement(starting_moves);
 		units_.insert(std::pair<gamemap::location,unit>(route.back(),un));
-	
+
 		if(map_.is_village(route.back())) {
 			get_village(route.back(),teams_,un.side()-1,units_);
 		}
-	
+
 		gui_.draw_tile(route.back().x,route.back().y);
-	
+
 		recorder.add_movement(route.front(),route.back());
 	}
 	gui_.invalidate_unit();
@@ -1496,10 +1574,13 @@ void turn_info::rename_unit()
 	const unit_map::iterator un = current_unit();
 	if(un == units_.end() || gui_.viewing_team()+1 != un->second.side())
 		return;
+	if(un->second.unrenamable())
+		return;
 
 	std::string name = un->second.description();
 	const int res = gui::show_dialog(gui_,NULL,_("Rename Unit"),"", gui::OK_CANCEL,NULL,NULL,"",&name);
 	if(res == 0) {
+		recorder.add_rename(name, un->first);
 		un->second.rename(name);
 		gui_.invalidate_unit();
 	}
@@ -1532,7 +1613,7 @@ void turn_info::save_game(const std::string& message, gui::DIALOG_TYPE dialog_ty
 {
 	std::stringstream stream;
 
-	const std::string ellipsed_name = font::make_text_ellipsis(state_of_game_.label, 
+	const std::string ellipsed_name = font::make_text_ellipsis(state_of_game_.label,
 			font::SIZE_NORMAL, 200);
 	stream << ellipsed_name << " " << _("Turn")
 	       << " " << status_.turn();
@@ -1656,6 +1737,14 @@ void turn_info::write_game_snapshot(config& start) const
 	status_.write(start);
 	game_events::write_events(start);
 
+	// Write terrain_graphics data in snapshot, too
+	const config::child_list& terrains = level_.get_children("terrain_graphics");
+	for(config::child_list::const_iterator tg = terrains.begin();
+			tg != terrains.end(); ++tg) {
+
+		start.add_child("terrain_graphics", **tg);
+	}
+
 	write_game(state_of_game_,start,WRITE_SNAPSHOT_ONLY);
 
 	// Clobber gold values to make sure the snapshot uses the values
@@ -1680,18 +1769,22 @@ void turn_info::toggle_grid()
 
 void turn_info::status_table()
 {
-	std::vector<std::string> items;
 	std::stringstream heading;
-	heading << _("Leader") << COLUMN_SEPARATOR << ' ' << COLUMN_SEPARATOR
+	heading << HEADING_PREFIX << _("Leader") << COLUMN_SEPARATOR << ' ' << COLUMN_SEPARATOR
 	        << _("Gold") << COLUMN_SEPARATOR
 	        << _("Villages") << COLUMN_SEPARATOR
 	        << _("Units") << COLUMN_SEPARATOR
 	        << _("Upkeep") << COLUMN_SEPARATOR
 	        << _("Income");
 
+	gui::menu::basic_sorter sorter;
+	sorter.set_redirect_sort(0,1).set_alpha_sort(1).set_numeric_sort(2).set_numeric_sort(3)
+	      .set_numeric_sort(4).set_numeric_sort(5).set_numeric_sort(6).set_numeric_sort(7);
+
 	if(game_config::debug)
 		heading << COLUMN_SEPARATOR << _("Gold");
 
+	std::vector<std::string> items;
 	items.push_back(heading.str());
 
 	const team& viewing_team = teams_[gui_.viewing_team()];
@@ -1717,11 +1810,13 @@ void turn_info::status_table()
 		std::stringstream str;
 
 		const unit_map::const_iterator leader = team_leader(n+1,units_);
+		//output the number of the side first, and this will
+		//cause it to be displayed in the correct colour
 		if(leader != units_.end()) {
 			str << IMAGE_PREFIX << leader->second.type().image() << COLUMN_SEPARATOR
-			    << char(n+1) << leader->second.description() << COLUMN_SEPARATOR;
+			    << "\033[3" << lexical_cast<char, size_t>(n+1) << 'm' << leader->second.description() << COLUMN_SEPARATOR;
 		} else {
-			str << char(n+1) << '-' << COLUMN_SEPARATOR << char(n+1) << '-' << COLUMN_SEPARATOR;
+			str << ' ' << COLUMN_SEPARATOR << "\033[3" << lexical_cast<char, size_t>(n+1) << "m-" << COLUMN_SEPARATOR;
 		}
 
 		if(enemy) {
@@ -1729,11 +1824,9 @@ void turn_info::status_table()
 		} else {
 			str << data.gold << COLUMN_SEPARATOR;
 		}
-		//output the number of the side first, and this will
-		//cause it to be displayed in the correct colour
 		str << data.villages << COLUMN_SEPARATOR
 		    << data.units << COLUMN_SEPARATOR << data.upkeep << COLUMN_SEPARATOR
-			<< (data.net_income < 0 ? font::BAD_TEXT:font::NULL_MARKUP) << data.net_income;
+		    << (data.net_income < 0 ? font::BAD_TEXT : font::NULL_MARKUP) << data.net_income;
 
 		if(game_config::debug)
 			str << COLUMN_SEPARATOR << teams_[n].gold();
@@ -1745,7 +1838,8 @@ void turn_info::status_table()
 		items.push_back(IMAGE_PREFIX + std::string("random-enemy.png") + COLUMN_SEPARATOR +
 		                IMAGE_PREFIX + "random-enemy.png");
 
-	gui::show_dialog(gui_,NULL,"","",gui::CLOSE_ONLY,&items);
+	gui::show_dialog(gui_,NULL,"","",gui::CLOSE_ONLY,&items,
+	                 NULL,"",NULL,0,NULL,NULL,-1,-1,NULL,NULL,"",&sorter);
 }
 
 void turn_info::recruit()
@@ -1934,7 +2028,7 @@ void turn_info::recall()
 
 	//sort the available units into order by value
 	//so that the most valuable units are shown first
-	std::sort(recall_list.begin(),recall_list.end(),compare_unit_values());
+	sort_units(recall_list);
 
 	gui_.draw(); //clear the old menu
 
@@ -1951,14 +2045,26 @@ void turn_info::recall()
 		gui::show_dialog(gui_,NULL,"",msg.str());
 	} else {
 		std::vector<std::string> options;
+
+		std::ostringstream heading;
+		heading << HEADING_PREFIX << COLUMN_SEPARATOR << _("Type")
+			    << COLUMN_SEPARATOR << _("Name")
+				<< COLUMN_SEPARATOR << _("Level")
+				<< COLUMN_SEPARATOR << _("XP");
+
+		gui::menu::basic_sorter sorter;
+		sorter.set_alpha_sort(1).set_alpha_sort(2).set_id_sort(3).set_numeric_sort(4);
+
+		options.push_back(heading.str());
+
 		for(std::vector<unit>::const_iterator u = recall_list.begin(); u != recall_list.end(); ++u) {
 			std::stringstream option;
 			const std::string& description = u->description().empty() ? "-" : u->description();
 			option << IMAGE_PREFIX << u->type().image() << COLUMN_SEPARATOR
 			       << u->type().language_name() << COLUMN_SEPARATOR
 			       << description << COLUMN_SEPARATOR
-			       << _("level: ") << u->type().level() << COLUMN_SEPARATOR
-			       << _("XP: ") << u->experience() << "/";
+			       << u->type().level() << COLUMN_SEPARATOR
+			       << u->experience() << "/";
 
 			if(u->can_advance() == false) {
 				option << "-";
@@ -1986,23 +2092,24 @@ void turn_info::recall()
 					_("Select unit:") + std::string("\n"),
 					gui::OK_CANCEL,&options,
 					&preview_panes,"",NULL,-1,
-					NULL,NULL,-1,-1,NULL,&buttons);
+					NULL,NULL,-1,-1,NULL,&buttons,"",&sorter);
 		}
 
 		if(res >= 0) {
 			unit& un = recall_list[res];
 			gamemap::location loc = last_hex_;
+			recorder.add_recall(res,loc);
 			const std::string err = recruit_unit(map_,team_num_,units_,un,loc,&gui_);
 			if(!err.empty()) {
+				recorder.undo();
 				gui::show_dialog(gui_,NULL,"",err,gui::OK_ONLY);
 			} else {
 				statistics::recall_unit(un);
 				current_team.spend_gold(game_config::recall_cost);
-				recorder.add_recall(res,loc);
 
 				undo_stack_.push_back(undo_action(un,loc,res));
 				redo_stack_.clear();
-				
+
 				recall_list.erase(recall_list.begin()+res);
 				gui_.invalidate_game_status();
 			}
@@ -2104,7 +2211,7 @@ void turn_info::change_unit_side()
 
 void turn_info::preferences()
 {
-	preferences::show_preferences_dialog(gui_);
+	preferences::show_preferences_dialog(gui_,terrain_config_);
 	gui_.redraw_everything();
 }
 
@@ -2116,7 +2223,8 @@ void turn_info::objectives()
 
 void turn_info::unit_list()
 {
-	const std::string heading = _("Type") + std::string(1, COLUMN_SEPARATOR) +
+	const std::string heading = std::string(1,HEADING_PREFIX) +
+	                            _("Type") + COLUMN_SEPARATOR +
 	                            _("Name") + COLUMN_SEPARATOR +
 	                            _("HP") + COLUMN_SEPARATOR +
 	                            _("XP") + COLUMN_SEPARATOR +
@@ -2124,13 +2232,17 @@ void turn_info::unit_list()
 	                            _("Moves") + COLUMN_SEPARATOR +
 	                            _("Location");
 
+	gui::menu::basic_sorter sorter;
+	sorter.set_alpha_sort(0).set_alpha_sort(1).set_numeric_sort(2).set_numeric_sort(3)
+		  .set_alpha_sort(4).set_numeric_sort(5).set_numeric_sort(6);
+
 	std::vector<std::string> items;
 	items.push_back(heading);
 
 	std::vector<gamemap::location> locations_list;
 	std::vector<unit> units_list;
 	for(unit_map::const_iterator i = units_.begin(); i != units_.end(); ++i) {
- 		if(i->second.side() != (gui_.viewing_team()+1))
+		if(i->second.side() != (gui_.viewing_team()+1))
 			continue;
 
 		std::stringstream row;
@@ -2148,16 +2260,10 @@ void turn_info::unit_list()
 		    << i->second.traits_description() << COLUMN_SEPARATOR
 		    << i->second.movement_left() << "/"
 		    << i->second.total_movement() << COLUMN_SEPARATOR
-		    << (i->first.x + 1) << "-" << (i->first.y + 1);
+		    << i->first;
 
 		items.push_back(row.str());
 
-		//extra unit for the first row to make up the heading
-		if(units_list.empty()) {
-			locations_list.push_back(i->first);
-			units_list.push_back(i->second);
-		}
-
 		locations_list.push_back(i->first);
 		units_list.push_back(i->second);
 	}
@@ -2171,7 +2277,8 @@ void turn_info::unit_list()
 		preview_panes.push_back(&unit_preview);
 
 		selected = gui::show_dialog(gui_,NULL,_("Unit List"),"",
-		                            gui::OK_ONLY,&items,&preview_panes);
+		                            gui::OK_ONLY,&items,&preview_panes,
+									"",NULL,0,NULL,NULL,-1,-1,NULL,NULL,"",&sorter);
 	}
 
 	if(selected > 0 && selected < int(locations_list.size())) {
@@ -2325,9 +2432,9 @@ void turn_info::do_search(const std::string& new_search)
 {
 	if(new_search.empty() == false && new_search != last_search_)
 		last_search_ = new_search;
-	
+
 	if(last_search_.empty()) return;
-	
+
 	bool found = false;
 	gamemap::location loc = last_search_hit_;
 	//If this is a location search, just center on that location.
@@ -2350,7 +2457,7 @@ void turn_info::do_search(const std::string& new_search)
 		loc.x = (loc.x + 1) % map_.x();
 		if(loc.x == 0)
 			loc.y = (loc.y + 1) % map_.y();
-		
+
 		//Search label
 		const std::string label = gui_.labels().get_label(loc);
 		if(label.empty() == false) {
@@ -2410,7 +2517,7 @@ void turn_info::do_command(const std::string& str)
 			}
 		} else if (teams_[index].is_ai()) {
 			teams_[index].make_human();
-		}       
+		}
 	} else if (cmd == "theme") {
 	        int action = 0;
 		std::vector<std::string> options=theme::get_known_themes();
@@ -2418,7 +2525,7 @@ void turn_info::do_command(const std::string& str)
 		action = gui::show_dialog(gui_,NULL,"",current_theme,gui::OK_CANCEL,&options);
 		if(action >-1){
 		  preferences::set_theme(options[action]);
-		//it would be preferable for the new theme to take effect 
+		//it would be preferable for the new theme to take effect
 		//immediately, however, this will have to do for now.
 		  gui::show_dialog(gui_,NULL,"",_("New theme will take effect on next new or loaded game."),gui::MESSAGE);
 		}
@@ -2434,8 +2541,24 @@ void turn_info::do_command(const std::string& str)
 		if(j != data.end()) {
 			const std::string side(data.begin(),j);
 			const std::string player(j+1,data.end());
-
-			change_side_controller(side,player);
+			int side_num;
+			try {
+				side_num = lexical_cast<int, std::string>(side);
+			} catch(bad_lexical_cast& bad_lexical) {
+				return;
+			}
+			if(side_num > 0 && side_num <= teams_.size()) {
+				if(teams_[static_cast<size_t>(side_num - 1)].is_human()) {
+					change_side_controller(side,player,true);
+					if(team_num_ == side_num) {
+						teams_[static_cast<size_t>(side_num - 1)].make_network();
+						close_textbox();
+						throw end_turn_exception(side_num);
+					}
+				} else {
+					change_side_controller(side,player);
+				}
+			}
 		}
 	} else if(cmd == "clear") {
 		gui_.clear_chat_messages();
@@ -2520,12 +2643,12 @@ bool turn_info::enemies_visible() const
 	// If we aren't using fog/shroud, this is easy :)
 	if(current_team().uses_fog() == false && current_team().uses_shroud() == false)
 		return true;
-	
+
 	//See if any enemies are visible
 	for(unit_map::const_iterator u = units_.begin(); u != units_.end(); ++u)
 		if(current_team().is_enemy(u->second.side()) && !gui_.fogged(u->first.x,u->first.y))
 			return true;
-		
+
 	return false;
 }
 
@@ -2533,7 +2656,7 @@ bool turn_info::enemies_visible() const
 void turn_info::show_enemy_moves(bool ignore_units)
 {
 	all_paths_ = paths();
-	
+
 	// Compute enemy movement positions
 	for(unit_map::iterator u = units_.begin(); u != units_.end(); ++u) {
 		if(current_team().is_enemy(u->second.side()) && !gui_.fogged(u->first.x,u->first.y) && !u->second.stone()) {
@@ -2544,7 +2667,7 @@ void turn_info::show_enemy_moves(bool ignore_units)
 			units.insert(*u);
 			const paths& path = paths(map_,status_,gameinfo_,ignore_units?units:units_,
 									  u->first,teams_,is_skirmisher,teleports);
-			
+
 			for (paths::routes_map::const_iterator route = path.routes.begin(); route != path.routes.end(); ++route) {
 				// map<...>::operator[](const key_type& key) inserts key into
 				// the map with a default instance of value_type
@@ -2552,7 +2675,7 @@ void turn_info::show_enemy_moves(bool ignore_units)
 			}
 		}
 	}
-	
+
 	gui_.set_paths(&all_paths_);
 }
 
@@ -2565,7 +2688,7 @@ void turn_info::toggle_shroud_updates() {
 
 bool turn_info::clear_shroud()
 {
-	bool cleared = current_team().auto_shroud_updates() && 
+	bool cleared = current_team().auto_shroud_updates() &&
 		::clear_shroud(gui_,status_,map_,gameinfo_,units_,teams_,team_num_-1);
 	enemies_visible_ = enemies_visible();
 	return cleared;
@@ -2813,6 +2936,15 @@ void turn_info::enter_textbox()
 	close_textbox();
 }
 
+namespace {
+
+bool tab_complete(const std::string& partword, const std::string& name)
+{
+	return name.size() >= partword.size() && std::equal(partword.begin(),partword.end(),name.begin(),chars_equal_insensitive);
+}
+
+}
+
 void turn_info::tab_textbox()
 {
 	if(textbox_.active() == false) {
@@ -2835,7 +2967,7 @@ void turn_info::tab_textbox()
 
 		if(last_space == std::string::npos) {
 			beginning = true;
-			semiword = text;	
+			semiword = text;
 		}else{
 			beginning = false;
 			semiword.assign(text,last_space+1,text.size());
@@ -2849,24 +2981,24 @@ void turn_info::tab_textbox()
 			}
 			const unit_map::const_iterator leader = team_leader(n+1,units_);
 			if(leader != units_.end()) {
-				const std::string name = leader->second.description();
-				if(name.find(semiword) == 0) {
-					if(guess.size() == 0) {
-						guess = name;
-					}else{
-						size_t i;
-						for(i=0; (i < guess.size()) || (i < name.size()); i++) {
-							if(guess[i] != name[i]) {
-								break;
-							}
-						}
-						guess.assign(guess,0,i);
-					}
+				const std::string& name = leader->second.description();
+				if(tab_complete(semiword,name)) {
+					guess = name;
 				}
 			}
 		}
 
-		if(guess.size() != 0) {
+		if(guess.empty()) {
+			const std::set<std::string>& observers = gui_.observers();
+			for(std::set<std::string>::const_iterator i = observers.begin(); i != observers.end(); ++i) {
+				if(tab_complete(semiword,*i)) {
+					guess = *i;
+					break;
+				}
+			}
+		}
+
+		if(guess.empty() == false) {
 			std::string add = beginning ? ": " : " ";
 			text.replace(last_space+1, semiword.size(), guess + add);
 			textbox_.box->set_text(text);
