@@ -1,7 +1,7 @@
-/* $Id: preferences.cpp,v 1.151 2005/04/05 13:37:15 ydirson Exp $ */
+/* $Id: preferences.cpp,v 1.160 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -87,7 +87,7 @@ manager::manager()
 
 manager::~manager()
 {
-	
+
 	std::vector<std::string> v;
 	std::copy(encountered_units_set.begin(), encountered_units_set.end(), std::back_inserter(v));
 	prefs["encountered_units"] = utils::join(v);
@@ -185,7 +185,7 @@ void set_resolution(const std::pair<int,int>& resolution)
 {
 	std::pair<int,int> res = resolution;
 
-	// - Ayin: disabled the following code. Why would one want to enforce that? 
+	// - Ayin: disabled the following code. Why would one want to enforce that?
 	// Some 16:9, or laptop screens, may have resolutions which do not
 	// comply to this rule (see bug 10630). I'm commenting this until it
 	// proves absolutely necessary.
@@ -480,37 +480,37 @@ bool show_combat()
 }
 
 bool allow_observers()
-{	
+{
 	return prefs["allow_observers"] != "no";
 }
 
 void set_allow_observers(bool value)
-{	
+{
 	prefs["allow_observers"] = value ? "yes" : "no";
 }
 
 bool fog()
-{	
-	return prefs["mp_fog"] != "no";
+{
+	return prefs["mp_fog"] == "yes";
 }
 
 void set_fog(bool value)
-{	
+{
 	prefs["mp_fog"] = value ? "yes" : "no";
 }
 
 bool shroud()
-{	
-	return prefs["mp_shroud"] != "no";
+{
+	return prefs["mp_shroud"] == "yes";
 }
 
 void set_shroud(bool value)
-{	
+{
 	prefs["mp_shroud"] = value ? "yes" : "no";
 }
 
 int turns()
-{	
+{
 	static const int default_value = 50;
 	int value = 0;
 	const string_map::const_iterator i = prefs.values.find("mp_turns");
@@ -526,15 +526,15 @@ int turns()
 }
 
 void set_turns(int value)
-{	
+{
 	std::stringstream stream;
 	stream << value;
 	prefs["mp_turns"] = stream.str();
 }
 
 int village_gold()
-{	
-	static const int default_value = 1;
+{
+	static const int default_value = 2;
 	int value = 0;
 	const string_map::const_iterator i = prefs.values.find("mp_village_gold");
 	if(i != prefs.values.end() && i->second.empty() == false) {
@@ -549,15 +549,15 @@ int village_gold()
 }
 
 void set_village_gold(int value)
-{	
+{
 	std::stringstream stream;
 	stream << value;
 	prefs["mp_village_gold"] = stream.str();
 }
 
 int xp_modifier()
-{	
-	static const int default_value = 100;
+{
+	static const int default_value = 70;
 	int value = 0;
 	const string_map::const_iterator i = prefs.values.find("mp_xp_modifier");
 	if(i != prefs.values.end() && i->second.empty() == false) {
@@ -572,14 +572,14 @@ int xp_modifier()
 }
 
 void set_xp_modifier(int value)
-{	
+{
 	std::stringstream stream;
 	stream << value;
 	prefs["mp_xp_modifier"] = stream.str();
 }
 
 int era()
-{	
+{
 	int value = 0;
 	const string_map::const_iterator i = prefs.values.find("mp_era");
 	if(i != prefs.values.end() && i->second.empty() == false) {
@@ -590,14 +590,14 @@ int era()
 }
 
 void set_era(int value)
-{	
+{
 	std::stringstream stream;
 	stream << value;
 	prefs["mp_era"] = stream.str();
 }
 
 int map()
-{	
+{
 	int value = 0;
 	const string_map::const_iterator i = prefs.values.find("mp_map");
 	if(i != prefs.values.end() && i->second.empty() == false) {
@@ -607,14 +607,14 @@ int map()
 }
 
 void set_map(int value)
-{	
+{
 	std::stringstream stream;
 	stream << value;
 	prefs["mp_map"] = stream.str();
 }
 
 bool show_ai_moves()
-{	
+{
 	return prefs["show_ai_moves"] != "no";
 }
 
@@ -651,6 +651,15 @@ std::string client_type()
 		return "human";
 }
 
+std::string clock_format()
+{
+	if(prefs["clock_format"].size())
+		return prefs["clock_format"];
+	else
+	  prefs["clock_format"]="%H:%M:%S";
+		return "%H:%M:%S";
+}
+
 const std::string& theme()
 {
 	if(non_interactive()) {
@@ -791,7 +800,7 @@ namespace {
 class preferences_dialog : public gui::preview_pane
 {
 public:
-	preferences_dialog(display& disp);
+	preferences_dialog(display& disp, const config& game_cfg);
 
 	struct video_mode_change_exception
 	{
@@ -809,26 +818,32 @@ private:
 	bool left_side() const { return false; }
 	void set_selection(int index);
 	void update_location(SDL_Rect const &rect);
+	const config* get_advanced_pref() const;
+	void set_advanced_menu();
 
 	gui::slider music_slider_, sound_slider_, scroll_slider_, gamma_slider_;
 	gui::button fullscreen_button_, turbo_button_, show_ai_moves_button_,
 	            show_grid_button_, show_floating_labels_button_, turn_dialog_button_,
 	            turn_bell_button_, show_team_colours_button_, show_colour_cursors_button_,
 	            show_haloing_button_, video_mode_button_, hotkeys_button_, gamma_button_,
-				flip_time_button_;
+				flip_time_button_, advanced_button_;
 	gui::label music_label_, sound_label_, scroll_label_, gamma_label_;
 	unsigned slider_label_width_;
 
-	enum TAB { GENERAL_TAB, DISPLAY_TAB, SOUND_TAB };
+	gui::menu advanced_;
+	int advanced_selection_;
+
+	enum TAB { GENERAL_TAB, DISPLAY_TAB, SOUND_TAB, ADVANCED_TAB };
 	TAB tab_;
 	display &disp_;
+	const config& game_cfg_;
 };
 
-preferences_dialog::preferences_dialog(display& disp)
+preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	: gui::preview_pane(disp.video()),
 	  music_slider_(disp.video()), sound_slider_(disp.video()),
 	  scroll_slider_(disp.video()), gamma_slider_(disp.video()),
-	  fullscreen_button_(disp.video(), _("Full Screen"), gui::button::TYPE_CHECK),
+	  fullscreen_button_(disp.video(), _("Toggle Full Screen"), gui::button::TYPE_CHECK),
 	  turbo_button_(disp.video(), _("Accelerated Speed"), gui::button::TYPE_CHECK),
 	  show_ai_moves_button_(disp.video(), _("Skip AI Moves"), gui::button::TYPE_CHECK),
 	  show_grid_button_(disp.video(), _("Show Grid"), gui::button::TYPE_CHECK),
@@ -842,9 +857,11 @@ preferences_dialog::preferences_dialog(display& disp)
 	  hotkeys_button_(disp.video(), _("Hotkeys")),
 	  gamma_button_(disp.video(), _("Adjust Gamma"), gui::button::TYPE_CHECK),
 	  flip_time_button_(disp.video(), _("Reverse Time Graphics"), gui::button::TYPE_CHECK),
+	  advanced_button_(disp.video(), "", gui::button::TYPE_CHECK),
 	  music_label_(disp.video(), _("Music Volume:")), sound_label_(disp.video(), _("SFX Volume:")),
 	  scroll_label_(disp.video(), _("Scroll Speed:")), gamma_label_(disp.video(), _("Gamma:")),
-	  slider_label_width_(0), tab_(GENERAL_TAB), disp_(disp)
+	  slider_label_width_(0), advanced_(disp.video(),std::vector<std::string>()), advanced_selection_(-1),
+	  tab_(GENERAL_TAB), disp_(disp), game_cfg_(game_cfg)
 {
 	// FIXME: this box should be vertically centered on the screen, but is not
 #if USE_TINY_GUI
@@ -917,6 +934,8 @@ preferences_dialog::preferences_dialog(display& disp)
 	show_haloing_button_.set_help_string(_("Use graphical special effects (may be slower)"));
 
 	hotkeys_button_.set_help_string(_("View and configure keyboard shortcuts"));
+
+	set_advanced_menu();
 }
 
 void preferences_dialog::update_location(SDL_Rect const &rect)
@@ -971,6 +990,15 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 				rect.w - slider_label_width_ - border, 0 };
 	sound_slider_.set_location(sound_rect);
 
+	//Advanced tab
+	ypos = rect.y;
+	advanced_.set_location(rect.x,ypos);
+	advanced_.set_max_height(height()-100);
+
+	ypos += advanced_.height() + border;
+
+	advanced_button_.set_location(rect.x,ypos);
+
 	set_selection(tab_);
 }
 
@@ -1014,6 +1042,65 @@ void preferences_dialog::process_event()
 	set_music_volume(music_slider_.value());
 	set_scroll_speed(scroll_slider_.value());
 	set_gamma(gamma_slider_.value());
+
+	if(advanced_.selection() != advanced_selection_) {
+		advanced_selection_ = advanced_.selection();
+		const config* const adv = get_advanced_pref();
+		if(adv != NULL) {
+			const config& pref = *adv;
+			advanced_button_.set_width(0);
+			advanced_button_.set_label(pref["name"]);
+			std::string value = prefs[pref["field"]];
+			if(value.empty()) {
+				value = pref["default"];
+			}
+
+			advanced_button_.set_check(value == "yes");
+		}
+	}
+
+	if(advanced_button_.pressed()) {
+		const config* const adv = get_advanced_pref();
+		if(adv != NULL) {
+			const config& pref = *adv;
+			prefs[pref["field"]] = advanced_button_.checked() ? "yes" : "no";
+			set_advanced_menu();
+		}
+	}
+}
+
+const config* preferences_dialog::get_advanced_pref() const
+{
+	const config::child_list& adv = game_cfg_.get_children("advanced_preference");
+	if(advanced_selection_ >= 0 && advanced_selection_ < int(adv.size())) {
+		return adv[advanced_selection_];
+	} else {
+		return NULL;
+	}
+}
+
+void preferences_dialog::set_advanced_menu()
+{
+	std::vector<std::string> advanced_items;
+	const config::child_list& adv = game_cfg_.get_children("advanced_preference");
+	for(config::child_list::const_iterator i = adv.begin(); i != adv.end(); ++i) {
+		std::ostringstream str;
+		std::string field = prefs[(**i)["field"]];
+		if(field.empty()) {
+			field = (**i)["default"];
+		}
+
+		if(field == "yes") {
+			field = _("yes");
+		} else if(field == "no") {
+			field = _("no");
+		}
+
+		str << (**i)["name"] << COLUMN_SEPARATOR << field;
+		advanced_items.push_back(str.str());
+	}
+
+	advanced_.set_items(advanced_items,true,true);
 }
 
 void preferences_dialog::set_selection(int index)
@@ -1022,7 +1109,7 @@ void preferences_dialog::set_selection(int index)
 	set_dirty();
 	bg_restore();
 
-	bool hide_general = tab_ != GENERAL_TAB;
+	const bool hide_general = tab_ != GENERAL_TAB;
 	scroll_label_.hide(hide_general);
 	scroll_slider_.hide(hide_general);
 	turbo_button_.hide(hide_general);
@@ -1033,7 +1120,7 @@ void preferences_dialog::set_selection(int index)
 	show_team_colours_button_.hide(hide_general);
 	show_grid_button_.hide(hide_general);
 
-	bool hide_display = tab_ != DISPLAY_TAB, hide_gamma = hide_display || !adjust_gamma();
+	const bool hide_display = tab_ != DISPLAY_TAB, hide_gamma = hide_display || !adjust_gamma();
 	gamma_label_.hide(hide_gamma);
 	gamma_slider_.hide(hide_gamma);
 	gamma_button_.hide(hide_display);
@@ -1044,16 +1131,20 @@ void preferences_dialog::set_selection(int index)
 	video_mode_button_.hide(hide_display);
 	flip_time_button_.hide(hide_display);
 
-	bool hide_sound = tab_ != SOUND_TAB;
+	const bool hide_sound = tab_ != SOUND_TAB;
 	music_label_.hide(hide_sound);
 	music_slider_.hide(hide_sound);
 	sound_label_.hide(hide_sound);
 	sound_slider_.hide(hide_sound);
+
+	const bool hide_advanced = tab_ != ADVANCED_TAB;
+	advanced_.hide(hide_advanced);
+	advanced_button_.hide(hide_advanced);
 }
 
 }
 
-void show_preferences_dialog(display& disp)
+void show_preferences_dialog(display& disp, const config& game_cfg)
 {
 	std::vector<std::string> items;
 
@@ -1062,12 +1153,13 @@ void show_preferences_dialog(display& disp)
 	items.push_back(pre + "general.png" + sep + dsgettext(GETTEXT_DOMAIN,"Prefs section^General"));
 	items.push_back(pre + "display.png" + sep + dsgettext(GETTEXT_DOMAIN,"Prefs section^Display"));
 	items.push_back(pre + "music.png" + sep + dsgettext(GETTEXT_DOMAIN,"Prefs section^Sound"));
-	
+	items.push_back(pre + "advanced.png" + sep + dsgettext(GETTEXT_DOMAIN,"Advanced section^Advanced"));
+
 	for(;;) {
 		try {
 			const events::event_context dialog_events_context;
 
-			preferences_dialog dialog(disp);
+			preferences_dialog dialog(disp,game_cfg);
 			std::vector<gui::preview_pane*> panes;
 			panes.push_back(&dialog);
 
@@ -1178,9 +1270,9 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 	std::vector<gui::button*> buttons;
 	buttons.push_back(&close_button);
 
-	surface_restorer restorer;	
+	surface_restorer restorer;
 	gui::draw_dialog(xpos,ypos,width,height,disp.video(),_("Hotkey Settings"),NULL,&buttons,&restorer);
-	
+
 	SDL_Rect clip_rect = { 0, 0, disp.x (), disp.y () };
 	SDL_Rect text_size = font::draw_text(NULL, clip_rect, font::SIZE_PLUS,
 					     font::NORMAL_COLOUR,_("Press desired HotKey"),
@@ -1195,15 +1287,24 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 		std::stringstream str,name;
 		name << i->get_description();
 		str << name.str();
-		str << COLUMN_SEPARATOR << "  :  " << COLUMN_SEPARATOR;
+		str << COLUMN_SEPARATOR;
 		str << i->get_name();
-		menu_items.push_back (str.str ());
+		menu_items.push_back(str.str());
 	}
 
-	gui::menu menu_(disp.video(), menu_items, false, height);
-	menu_.set_width(font::relative_size(400));	
+	std::ostringstream heading;
+	heading << HEADING_PREFIX << _("Action") << COLUMN_SEPARATOR << _("Binding");
+	menu_items.push_back(heading.str());
+
+	gui::menu::basic_sorter sorter;
+	sorter.set_alpha_sort(0).set_alpha_sort(1);
+
+	gui::menu menu_(disp.video(), menu_items, false, height, -1, &sorter);
+	menu_.sort_by(0);
+	menu_.reset_selection();
+	menu_.set_width(font::relative_size(400));
 	menu_.set_location(xpos + font::relative_size(20), ypos);
-	
+
 	gui::button change_button (disp.video(), _("Change Hotkey"));
 	change_button.set_location(xpos + width - change_button.width () - font::relative_size(30),ypos + font::relative_size(80));
 
@@ -1221,8 +1322,8 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 								centery-text_size.h/2 - 16,
 									text_size.w+60,
 									text_size.h+32};
-			surface_restorer restorer(&disp.video(),dlgr);										
-		 	gui::draw_dialog_frame (centerx-text_size.w/2 - 20, 
+			surface_restorer restorer(&disp.video(),dlgr);
+			gui::draw_dialog_frame (centerx-text_size.w/2 - 20,
 									centery-text_size.h/2 - 6,
 									text_size.w+40,
 									text_size.h+12,disp.video());
@@ -1238,9 +1339,9 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 			do {
 				if (event.type==SDL_KEYDOWN)
 				{
-				 	key=event.key.keysym.sym;
-				 	mod=event.key.keysym.mod;
-				};			
+					key=event.key.keysym.sym;
+					mod=event.key.keysym.mod;
+				};
 				SDL_PollEvent(&event);
 				disp.flip();
 				SDL_Delay(10);
@@ -1248,17 +1349,17 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 			restorer.restore();
 			disp.update_display();
 
-			const hotkey::hotkey_item& oldhk = hotkey::get_hotkey(key, (mod & KMOD_SHIFT) != 0, 
+			const hotkey::hotkey_item& oldhk = hotkey::get_hotkey(key, (mod & KMOD_SHIFT) != 0,
 					(mod & KMOD_CTRL) != 0, (mod & KMOD_ALT) != 0, (mod & KMOD_LMETA) != 0);
 			hotkey::hotkey_item& newhk = hotkey::get_visible_hotkey(menu_.selection());
 
 			if(oldhk.get_id() != newhk.get_id() && !oldhk.null()) {
 				gui::show_dialog(disp,NULL,"",_("This HotKey is already in use."),gui::MESSAGE);
 			} else {
-				newhk.set_key(key, (mod & KMOD_SHIFT) != 0, 
+				newhk.set_key(key, (mod & KMOD_SHIFT) != 0,
 						(mod & KMOD_CTRL) != 0, (mod & KMOD_ALT) != 0, (mod & KMOD_LMETA) != 0);
 
-				menu_.change_item(menu_.selection(), 2, newhk.get_name());
+				menu_.change_item(menu_.selection(), 1, newhk.get_name());
 			};
 		}
 		if (save_button.pressed()) {
@@ -1274,7 +1375,7 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 		events::pump();
 		events::raise_process_event();
 		events::raise_draw_event();
-		
+
 		disp.update_display();
 
 		SDL_Delay(10);
