@@ -1,8 +1,8 @@
-/* $Id: random.hpp,v 1.1 2005/03/22 22:09:28 ydirson Exp $ */
+/* $Id: random.hpp,v 1.4 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
    Copyright (C) 2005 by Yann Dirson <ydirson@altern.org>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -19,17 +19,30 @@ class config;
 int get_random();
 const config* get_random_results();
 void set_random_results(const config& cfg);
+/** Ensures that the next "random results" operations will not happen on the
+ * current random context, eventually adding a dummy random context if
+ * necessary.
+ */
+void add_random_separator();
 
 class rng
 {
 public:
 	rng();
 	int get_random();
-	const config* get_random_results() const;
+
+	const config* get_random_results();
 	void set_random_results(const config& cfg);
 
+	void add_random_separator();
 protected:
+	config* random();
+	config* set_random(config*);
+
+private:
 	config* random_;
+	bool separator_, started_;
+	std::string remaining_values_;
 };
 
 struct set_random_generator {
