@@ -1,7 +1,7 @@
-/* $Id: replay.cpp,v 1.104 2005/03/30 19:48:17 gruikya Exp $ */
+/* $Id: replay.cpp,v 1.113 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -44,11 +44,6 @@
 
 //functions to verify that the unit structure on both machines is identical
 namespace {
-	std::ostream &operator<<(std::ostream &s, gamemap::location const &l) {
-		s << (l.x + 1) << ',' << (l.y + 1);
-		return s;
-	}
-
 	void verify(const unit_map& units, const config& cfg)
 	{
 		LOG_NW << "verifying unit structure...\n";
@@ -87,8 +82,7 @@ namespace {
 			if(u == units.end()) {
 				ERR_NW << "SYNC VERIFICATION FAILED: data source says there is a '"
 				       << (**i)["type"] << "' (side " << (**i)["side"] << ") at "
-				       << (**i)["x"] << ',' << (**i)["y"]
-				       << " but there is no local record of it\n";
+				       << loc << " but there is no local record of it\n";
 				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
@@ -100,8 +94,7 @@ namespace {
 			for(const std::string* str = fields; str->empty() == false; ++str) {
 				if(cfg[*str] != (**i)[*str]) {
 					ERR_NW << "ERROR IN FIELD '" << *str << "' for unit at "
-					       << (**i)["x"] << ',' << (**i)["y"]
-					       << " data source: '" << (**i)[*str]
+					       << loc << " data source: '" << (**i)[*str]
 					       << "' local: '" << cfg[*str] << "'\n";
 					is_ok = false;
 				}
@@ -244,11 +237,7 @@ void replay::add_recruit(int value, const gamemap::location& loc)
 	sprintf(buf,"%d",value);
 	val["value"] = buf;
 
-	sprintf(buf,"%d",loc.x+1);
-	val["x"] = buf;
-
-	sprintf(buf,"%d",loc.y+1);
-	val["y"] = buf;
+	loc.write(val);
 
 	cmd->add_child("recruit",val);
 }
@@ -263,11 +252,7 @@ void replay::add_recall(int value, const gamemap::location& loc)
 	sprintf(buf,"%d",value);
 	val["value"] = buf;
 
-	sprintf(buf,"%d",loc.x+1);
-	val["x"] = buf;
-
-	sprintf(buf,"%d",loc.y+1);
-	val["y"] = buf;
+	loc.write(val);
 
 	cmd->add_child("recall",val);
 }
@@ -304,16 +289,8 @@ void replay::add_pos(const std::string& type,
 	config* const cmd = add_command();
 
 	config move, src, dst;
-
-	char buf[100];
-	sprintf(buf,"%d",a.x+1);
-	src["x"] = buf;
-	sprintf(buf,"%d",a.y+1);
-	src["y"] = buf;
-	sprintf(buf,"%d",b.x+1);
-	dst["x"] = buf;
-	sprintf(buf,"%d",b.y+1);
-	dst["y"] = buf;
+	a.write(src);
+	b.write(dst);
 
 	move.add_child("source",src);
 	move.add_child("destination",dst);
@@ -340,7 +317,7 @@ void replay::choose_option(int index)
 
 void replay::add_label(const std::string& text, const gamemap::location& loc)
 {
-	config* const cmd = add_command();
+	config* const cmd = add_command(false);
 
 	(*cmd)["undo"] = "no";
 
@@ -352,16 +329,25 @@ void replay::add_label(const std::string& text, const gamemap::location& loc)
 	cmd->add_child("label",val);
 }
 
+void replay::add_rename(const std::string& name, const gamemap::location& loc)
+{
+	config* const cmd = add_command(false);
+	(*cmd)["undo"] = "no";
+	config val;
+	loc.write(val);
+	val["name"] = name;
+	cmd->add_child("rename", val);
+}
+
 void replay::end_turn()
 {
 	config* const cmd = add_command();
 	cmd->add_child("end_turn");
-	random_ = current_;
 }
 
 void replay::speak(const config& cfg)
 {
-	config* const cmd = add_command();
+	config* const cmd = add_command(false);
 	if(cmd != NULL) {
 		cmd->add_child("speak",cfg);
 		(*cmd)["undo"] = "no";
@@ -421,7 +407,8 @@ void replay::undo()
 
 	if(cmd.first != cmd.second) {
 		cfg_.remove_child("command",cmd.second - cmd.first - 1);
-		current_ = random_ = NULL;
+		current_ = NULL;
+		set_random(NULL);
 	}
 }
 
@@ -442,11 +429,12 @@ void replay::mark_current()
 	}
 }
 
-config* replay::add_command()
+config* replay::add_command(bool update_random_context)
 {
 	pos_ = ncommands()+1;
 	current_ = &cfg_.add_child("command");
-	random_ = current_;
+	if(update_random_context)
+		set_random(current_);
 
 	return current_;
 }
@@ -463,18 +451,15 @@ config* replay::get_next_action()
 
 	LOG_NW << "up to replay action " << pos_ << "/" << commands().size() << "\n";
 
-	random_ = current_ = commands()[pos_];
+	current_ = commands()[pos_];
+	set_random(current_);
 	++pos_;
 	return current_;
 }
 
 void replay::pre_replay()
 {
-
-	if(pos_ >= commands().size())
-		return;
-
-	while(commands()[pos_]->child("start") != NULL) {
+	while(pos_ < commands().size() && commands()[pos_]->child("start") != NULL) {
 		if(get_next_action() == NULL)
 			return;
 	}
@@ -489,7 +474,7 @@ void replay::set_to_end()
 {
 	pos_ = commands().size();
 	current_ = NULL;
-	random_ = NULL;
+	set_random(NULL);
 }
 
 void replay::clear()
@@ -497,7 +482,7 @@ void replay::clear()
 	cfg_ = config();
 	pos_ = 0;
 	current_ = NULL;
-	random_ = NULL;
+	set_random(NULL);
 	skip_ = 0;
 }
 
@@ -572,7 +557,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 	for(;;) {
 		config* const cfg = replayer.get_next_action();
 		config* child;
-		
+
 		//do we need to recalculate shroud after this action is processed?
 		bool fix_shroud = false;
 
@@ -580,7 +565,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		if(advancing_units.empty() == false) {
 			if(cfg == NULL) {
 				ERR_NW << "promotion expected, but none found\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			//if there is a promotion, we process it and go onto the next command
@@ -607,22 +592,6 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		//if there is nothing more in the records
 		if(cfg == NULL) {
 			replayer.set_skip(0);
-			const config current_stats = statistics::write_stats();
-			if(current_stats != statistics::replay_verify_stats) {
-				ERR_NW << "replay statistics verification failed\n";
-				//Disabling because it breaks multiplayer.
-				//Apparently, statistics::replay_verify_stats
-				//is only written to in game::load_game() (!)
-				//Re-enable when fixed / if I am wrong.
-				//
-				//By the way, making a "config" object global
-				//is highly objectable IMO.
-				//
-				// --Ayin 30 mar. 2005
-				//
-				//if (!game_config::ignore_replay_errors) 
-				//	throw replay::error();
-			}
 			return false;
 		}
 
@@ -646,7 +615,21 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const std::string& text = (*child)["text"];
 
 			disp.labels().set_label(loc,text);
-		} 
+		}
+
+		else if((child = cfg->child("rename")) != NULL) {
+			const gamemap::location loc(*child);
+			const std::string& name = (*child)["name"];
+
+			std::map<gamemap::location,unit>::iterator u = units.find(loc);
+
+			if(u->second.unrenamable()) {
+				ERR_NW << "renaming unrenamable unit " << u->second.name() << "\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
+			}
+
+			u->second.rename(name);
+		}
 
 		//if there is an end turn directive
 		else if(cfg->child("end_turn") != NULL) {
@@ -672,7 +655,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				ERR_NW << "recruitment index is illegal: " << val
 				       << " while this side only has " << recruits.size()
 				       << " units available for recruitment\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			std::set<std::string>::const_iterator itor = recruits.begin();
@@ -680,7 +663,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const std::map<std::string,unit_type>::const_iterator u_type = gameinfo.unit_types.find(*itor);
 			if(u_type == gameinfo.unit_types.end()) {
 				ERR_NW << "recruiting illegal unit: '" << *itor << "'\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			unit new_unit(&(u_type->second),team_num,true);
@@ -695,10 +678,14 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				       << u_type->second.cost() << "/" << current_team.gold() << "\n";
 				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
+			LOG_NW << "recruit: team=" << team_num << " '" << u_type->second.id() << "' at (" << loc
+			       << ") cost=" << u_type->second.cost() << " from gold=" << current_team.gold() << ' ';
+
 
 			statistics::recruit_unit(new_unit);
 
 			current_team.spend_gold(u_type->second.cost());
+			LOG_NW << "-> " << (current_team.gold()) << "\n";
 			fix_shroud = true;
 		}
 
@@ -706,10 +693,10 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			player_info* player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
 				ERR_NW << "illegal recall\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
-			std::sort(player->available_units.begin(),player->available_units.end(),compare_unit_values());
+			sort_units(player->available_units);
 
 			const std::string& recall_num = (*child)["value"];
 			const int val = atoi(recall_num.c_str());
@@ -732,10 +719,10 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			player_info* const player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
 				ERR_NW << "illegal disband\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
-			std::sort(player->available_units.begin(),player->available_units.end(),compare_unit_values());
+			sort_units(player->available_units);
 			const std::string& unit_num = (*child)["value"];
 			const int val = atoi(unit_num.c_str());
 
@@ -754,7 +741,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			if(destination == NULL || source == NULL) {
 				ERR_NW << "no destination/source found in movement\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			const gamemap::location src(*source);
@@ -769,8 +756,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			u = units.find(src);
 			if(u == units.end()) {
 				ERR_NW << "unfound location for source of movement: "
-				       << src << '-' << dst << '\n';
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				       << src << " -> " << dst << '\n';
+				throw replay::error();
 			}
 
 			const bool ignore_zocs = u->second.type().is_skirmisher();
@@ -785,11 +772,11 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			if(rt == paths_list.routes.end()) {
 
 				for(rt = paths_list.routes.begin(); rt != paths_list.routes.end(); ++rt) {
-					ERR_NW << "can get to: " << rt->first.y << '\n';
+					ERR_NW << "can get to: " << rt->first << '\n';
 				}
 
 				ERR_NW << "src cannot get to dst: " << current_unit.movement_left() << ' '
-				       << paths_list.routes.size() << ' ' << src << '-' << dst << '\n';
+				       << paths_list.routes.size() << ' ' << src << " -> " << dst << '\n';
 				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
@@ -823,6 +810,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			}
 
 			game_events::fire("moveto",dst);
+			//FIXME: what's special about team 1?
 			if(team_num != 1 && (teams.front().uses_shroud() || teams.front().uses_fog()) && !teams.front().fogged(dst.x,dst.y)) {
 				game_events::fire("sighted",dst);
 			}
@@ -837,7 +825,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			if(destination == NULL || source == NULL) {
 				ERR_NW << "no destination/source found in attack\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			const gamemap::location src(*source);
@@ -849,7 +837,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			std::map<gamemap::location,unit>::iterator u = units.find(src);
 			if(u == units.end()) {
 				ERR_NW << "unfound location for source of attack\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				throw replay::error();
 			}
 
 			if(size_t(weapon_num) >= u->second.attacks().size()) {
