@@ -1,7 +1,7 @@
-/* $Id: replay.hpp,v 1.33 2005/03/28 19:08:58 gruikya Exp $ */
+/* $Id: replay.hpp,v 1.36 2005/06/04 19:16:05 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -54,6 +54,7 @@ public:
 	                int weapon);
 	void choose_option(int index);
 	void add_label(const std::string& text, const gamemap::location& loc);
+	void add_rename(const std::string& name, const gamemap::location& loc);
 	void end_turn();
 
 	void speak(const config& cfg);
@@ -99,7 +100,18 @@ private:
 	void add_value(const std::string& type, int value);
 
 	const config::child_list& commands() const;
-	config* add_command();
+	/** Adds a new empty command to the command list.
+	 *
+	 * @param update_random_context  If set to false, do not update the
+	 *           random context variables: all random generation will take
+	 *           place in the previous random context. Used for commands
+	 *           for which "random context" is pointless, and which can be
+	 *           issued while some other commands are still taking place,
+	 *           like, for example, messages during combats.
+	 *
+	 * @return a pointer to the added command
+	 */
+	config* add_command(bool update_random_context=true);
 	config cfg_;
 	unsigned int pos_;
 
@@ -128,7 +140,7 @@ struct replay_undo
 	replay_undo(replay& obj) : obj_(&obj) {}
 	~replay_undo() { if(obj_) obj_->undo(); }
 	void confirm_transaction() { obj_ = NULL; }
-	
+
 private:
 	replay* obj_;
 };
