@@ -8,7 +8,9 @@
 #include "language.hpp"
 #include "reports.hpp"
 #include "wassert.hpp"
+#include "preferences.hpp"
 
+#include <ctime>
 #include <map>
 #include <set>
 #include <sstream>
@@ -18,8 +20,9 @@ namespace {
 		"unit_traits","unit_status","unit_alignment","unit_abilities","unit_hp","unit_xp",
 		"unit_moves","unit_weapons","unit_image","unit_profile","time_of_day",
 		"turn","gold","villages","num_units","upkeep", "expenses",
-		 "income", "terrain", "position", "side_playing", "observers", "selected_terrain",
-		 "edit_left_button_function"};
+		 "income", "terrain", "position", "side_playing", "observers",
+		 "report_clock",
+		 "selected_terrain","edit_left_button_function"};
 	std::map<reports::TYPE, std::string> report_contents;
 }
 
@@ -62,7 +65,7 @@ report generate_report(TYPE type, const gamemap& map, const unit_map& units,
                        const gamestatus& status, const std::set<std::string>& observers)
 {
 	unit_map::const_iterator u = units.end();
-	
+
 	if(int(type) >= int(UNIT_REPORTS_BEGIN) && int(type) < int(UNIT_REPORTS_END) || type == POSITION) {
 
 		if(!current_team.fogged(mouseover.x,mouseover.y)) {
@@ -177,7 +180,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 	case UNIT_WEAPONS: {
 		report res;
 		std::stringstream tooltip;
-		
+
 		const size_t team_index = u->second.side()-1;
 		if(team_index >= teams.size()) {
 			std::cerr << "illegal team index in reporting: " << team_index << "\n";
@@ -189,18 +192,18 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 		    at_it != attacks.end(); ++at_it) {
 			const std::string& lang_weapon = gettext(at_it->name().c_str());
 			const std::string& lang_type = gettext(at_it->type().c_str());
-			
+
 			str << lang_weapon << " (" << lang_type << ")\n";
 
 			tooltip << lang_weapon << " (" << lang_type << ")\n";
 
 			//find all the unit types on the map, and show this weapon's bonus against all the different units
-			std::set<const unit_type*> seen_units;
+			std::set<std::string> seen_units;
 			std::map<int,std::vector<std::string> > resistances;
 			for(unit_map::const_iterator u_it = units.begin(); u_it != units.end(); ++u_it) {
 				if(teams[team_index].is_enemy(u_it->second.side()) && !current_team.fogged(u_it->first.x,u_it->first.y) &&
-				   seen_units.count(&u_it->second.type()) == 0) {
-					seen_units.insert(&u_it->second.type());
+				   seen_units.count(u_it->second.type().id()) == 0) {
+					seen_units.insert(u_it->second.type().id());
 					const int resistance = u_it->second.type().movement_type().resistance_against(*at_it) - 100;
 					resistances[resistance].push_back(u_it->second.type().language_name());
 				}
@@ -221,20 +224,20 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			}
 
 			res.add_text(str,tooltip);
-			
+
 			if (!at_it->special().empty()) {
 				str << gettext(at_it->special().c_str()) << "\n";
 				tooltip << string_table["weapon_special_" + at_it->special() + "_description"];
 				res.add_text(str,tooltip);
 			}
-			
+
 			str << at_it->damage() << "-" << at_it->num_attacks() << " -- "
 		        << (at_it->range() == attack_type::SHORT_RANGE ?
 		            _("melee") :
 					_("ranged"));
 			tooltip << at_it->damage() << " " << _("damage") << ", "
 					<< at_it->num_attacks() << " " << _("attacks");
-			
+
 			str << "\n";
 			res.add_text(str,tooltip);
 		}
@@ -248,14 +251,14 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 	case TIME_OF_DAY: {
 		const time_of_day& tod = timeofday_at(status,units,mouseover);
 		std::stringstream tooltip;
-		
+
 		tooltip << tod.name << "\n"
 		        << _("Lawful units: ")
 				<< (tod.lawful_bonus > 0 ? "+" : "") << tod.lawful_bonus << "%\n"
 				<< _("Neutral units: ") << "0%\n"
 				<< _("Chaotic units: ")
 				<< (tod.lawful_bonus < 0 ? "+" : "") << (tod.lawful_bonus*-1) << "%";
-		
+
 		return report("",tod.image,tooltip.str());
 	}
 	case TURN:
@@ -321,7 +324,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 
 		if(underlying.size() != 1 || underlying[0] != terrain) {
 			str << " (";
-			
+
 			for(std::string::const_iterator i = underlying.begin(); i != underlying.end(); ++i) {
 				str << map.get_terrain_info(*i).name();
 				if(i+1 != underlying.end()) {
@@ -339,7 +342,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			break;
 		}
 
-		str << (mouseover.x+1) << ", " << (mouseover.y+1);
+		str << mouseover;
 
 		if(u == units.end() || current_team.shrouded(mouseover.x,mouseover.y))
 			break;
@@ -355,7 +358,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			str << " (" << defense << "%," << move_cost << ")";
 		}
 
-		break;	
+		break;
 	}
 
 	case SIDE_PLAYING: {
@@ -399,17 +402,29 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			return report();
 		}
 	}
+	case REPORT_CLOCK: {
+	  time_t t=time(NULL);
+	  struct tm *lt=localtime(&t);
+	  char temp[10];
+	  size_t s=strftime(temp,10,preferences::clock_format().c_str(),lt);
+	  if(s>0){
+	    return report(temp);
+	  }else{
+	    return report();
+	  }
+	}
+
 	default:
 		wassert(false);
 		break;
 	}
 	return report(str.str());
 }
-	
-	
+
+
 void set_report_content(const TYPE which_report, const std::string &content) {
 	report_contents[which_report] = content;
 }
-	
+
 
 }
