@@ -1,4 +1,4 @@
-/* $Id: binary_or_text.cpp,v 1.8 2005/04/05 19:42:07 gruikya Exp $ */
+/* $Id: binary_or_text.cpp,v 1.13 2005/05/16 22:44:20 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -12,6 +12,8 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "serialization/binary_wml.hpp"
@@ -26,7 +28,7 @@ bool detect_format_and_read(config &cfg, std::istream &in, std::string* error_lo
 		read_compressed(cfg, in);
 		return true;
 	} else {
-		read(cfg, in, NULL, error_log);
+		read(cfg, in, error_log);
 		return false;
 	}
 }
