@@ -1,6 +1,6 @@
-/* $Id: parser.cpp,v 1.13 2005/03/29 21:01:30 gruikya Exp $ */
+/* $Id: parser.cpp,v 1.19 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -38,70 +38,48 @@
 
 static const size_t max_recursion_levels = 100;
 
-line_source get_line_source(std::vector< line_source > const &line_src, int line)
-{
-	line_source res(line, "", 0);
-	std::vector< line_source >::const_iterator it =
-		std::upper_bound(line_src.begin(), line_src.end(), res);
-	if (it != line_src.begin()) {
-		--it;
-		res.file = it->file;
-		res.fileline = it->fileline + (line - it->linenum);
-	}
-
-	return res;
-}
-
 namespace {
 
 class parser
 {
 public:
-	parser(config& cfg, std::istream& in, std::vector<line_source> const* line_sources);
+	parser(config& cfg, std::istream& in);
 	void operator() (std::string* error_log=NULL);
 
 private:
 	void parse_element();
 	void parse_variable();
 	void parse_directive();
-	std::string lineno_string(utils::string_map& map, size_t lineno,
-			const std::string& string1, const std::string& string2);
+	std::string lineno_string(utils::string_map &map, std::string const &lineno,
+			          std::string const &error_string);
 	void error(const std::string& message);
 
 	config& cfg_;
 	tokenizer tok_;
-	std::vector<line_source> const* line_sources;
 
 	struct element {
-		element(config* cfg, const std::string& name, size_t start_line, const std::string& textdomain) :
-			cfg(cfg), name(name), textdomain(textdomain), start_line(start_line){};
+		element(config *cfg, std::string const &name, std::string const &start_line)
+			: cfg(cfg), name(name), start_line(start_line) {}
 
 		config* cfg;
 		std::string name;
 
 		std::map<std::string, config*> last_element_map;
-		std::string textdomain;
-		size_t start_line;
+		std::string start_line;
 	};
 
 	std::stack<element> elements;
-
-	std::string current_textdomain_location;
 };
 
-parser::parser(config &cfg, std::istream &in, std::vector<line_source> const *line_sources) :
-	cfg_(cfg),
-	tok_(in),
-	line_sources(line_sources),
-	current_textdomain_location("")
+parser::parser(config &cfg, std::istream &in)
+	: cfg_(cfg), tok_(in)
 {
 }
 
 void parser::operator()(std::string* error_log)
 {
 	cfg_.clear();
-	elements.push(element(&cfg_, "", 0, PACKAGE));
-	tok_.textdomain() = PACKAGE;
+	elements.push(element(&cfg_, "", ""));
 
 	do {
 		try {
@@ -125,7 +103,7 @@ void parser::operator()(std::string* error_log)
 		} catch(config::error& e) {
 			if(error_log == NULL)
 				throw;
-			
+
 			// On error, dump tokens to the next LF
 			while(tok_.current_token().type != token::LF &&
 					tok_.current_token().type != token::END) {
@@ -143,12 +121,11 @@ void parser::operator()(std::string* error_log)
 		utils::string_map i18n_symbols;
 		i18n_symbols["tag"] = elements.top().name;
 		error(lineno_string(i18n_symbols, elements.top().start_line,
-				N_("Missing closing tag for tag $tag (file $file, line $line)"),
-				N_("Missing closing tag for tag $tag (line $line)")));
+				N_("Missing closing tag for tag $tag at $pos")));
 	}
 }
 
-void parser::parse_element() 
+void parser::parse_element()
 {
 	tok_.next_token();
 	std::string elname;
@@ -164,8 +141,7 @@ void parser::parse_element()
 		// Add the element
 		current_element = &(elements.top().cfg->add_child(elname));
 		elements.top().last_element_map[elname] = current_element;
-		elements.top().textdomain = tok_.textdomain();
-		elements.push(element(current_element, elname, tok_.get_line(), elements.top().textdomain));
+		elements.push(element(current_element, elname, tok_.get_line()));
 		break;
 
 	case '+': // [+element]
@@ -184,8 +160,7 @@ void parser::parse_element()
 			current_element = last_element_itor->second;
 		}
 		elements.top().last_element_map[elname] = current_element;
-		elements.top().textdomain = tok_.textdomain();
-		elements.push(element(current_element, elname, tok_.get_line(), elements.top().textdomain));
+		elements.push(element(current_element, elname, tok_.get_line()));
 		break;
 
 	case '/': // [/element]
@@ -199,13 +174,12 @@ void parser::parse_element()
 		if(elname != elements.top().name) {
 			utils::string_map i18n_symbols;
 			i18n_symbols["tag"] = elements.top().name;
+			i18n_symbols["tag2"] = elname;
 			error(lineno_string(i18n_symbols, elements.top().start_line,
-					N_("Found invalid closing tag for tag $tag (file $file, line $line)"),
-					N_("Found invalid closing tag for tag $tag (line $line)")));
+					N_("Found invalid closing tag $tag2 for tag $tag (opened at $pos)")));
 		}
 
 		elements.pop();
-		tok_.textdomain() = elements.top().textdomain;
 		break;
 	default:
 		error(_("Invalid tag name"));
@@ -238,10 +212,10 @@ void parser::parse_variable()
 		}
 		tok_.next_token();
 	}
-	if(variables.back().empty()) 
+	if(variables.back().empty())
 		error(_("Empty variable name"));
 
-	std::vector<std::string>::const_iterator curvar = variables.begin(); 
+	std::vector<std::string>::const_iterator curvar = variables.begin();
 
 	bool ignore_next_newlines = false;
 	while(1) {
@@ -303,25 +277,23 @@ void parser::parse_variable()
 	}
 }
 
-std::string parser::lineno_string(utils::string_map& i18n_symbols, size_t lineno,
-			const std::string& string1, const std::string& string2)
+std::string parser::lineno_string(utils::string_map &i18n_symbols, std::string const &lineno,
+			          std::string const &error_string)
 {
+	std::vector< std::string > pos = utils::split(lineno, ' ');
+	std::vector< std::string >::const_iterator i = pos.begin(), end = pos.end();
+	std::string included_from = _(" included from ");
 	std::string res;
-
-	if(line_sources != NULL) {
-		const line_source src = get_line_source(*line_sources, lineno);
-		i18n_symbols["file"] = lexical_cast<std::string>(src.file);
-		i18n_symbols["line"] = lexical_cast<std::string>(src.fileline);
-		i18n_symbols["column"] = lexical_cast<std::string>(tok_.get_column());
-
-		res = vgettext(string1.c_str(), i18n_symbols);
-	} else {
-		i18n_symbols["line"] = lexical_cast<std::string>(lineno);
-		i18n_symbols["column"] = lexical_cast<std::string>(tok_.get_column());
-
-		res = vgettext(string2.c_str(), i18n_symbols);
+	while (i != end) {
+		std::string const &line = *(i++);
+		std::string const &file = i != end ? *(i++) : "<unknown>";
+		if (!res.empty())
+			res += included_from;
+		res += file + ':' + line;
 	}
-	return res;
+	if (res.empty()) res = "???";
+	i18n_symbols["pos"] = res;
+	return vgettext(error_string.c_str(), i18n_symbols);
 }
 
 void parser::error(const std::string& error_type)
@@ -329,17 +301,16 @@ void parser::error(const std::string& error_type)
 	utils::string_map i18n_symbols;
 	i18n_symbols["error"] = error_type;
 
-	throw config::error(lineno_string(i18n_symbols, tok_.get_line(), 
-				N_("$error in file $file (line $line, column $column)"),
-				N_("$error (line $line, column $column)")));
+	throw config::error(
+		lineno_string(i18n_symbols, tok_.get_line(),
+		              N_("$error at $pos")));
 }
 
 } // end anon namespace
 
-void read(config &cfg, std::istream &data_in, std::vector< line_source > const *line_sources,
-		std::string* error_log)
+void read(config &cfg, std::istream &data_in, std::string* error_log)
 {
-	parser(cfg, data_in, line_sources)(error_log);
+	parser(cfg, data_in)(error_log);
 }
 
 static char const *AttributeEquals = "=";
@@ -383,30 +354,30 @@ static void write_internal(config const &cfg, std::ostream &out, std::string tex
 
 				if(w.translatable()) {
 					if(w.textdomain() != textdomain) {
-						out << TextdomainPrefix 
-							<< w.textdomain() 
+						out << TextdomainPrefix
+							<< w.textdomain()
 							<< TextdomainPostfix;
 						textdomain = w.textdomain();
 					}
 
 					if(first) {
-						out << std::string(tab, '\t') 
-							<< i->first 
+						out << std::string(tab, '\t')
+							<< i->first
 							<< AttributeEquals;
 					}
 
-					out << TranslatableAttributePrefix 
+					out << TranslatableAttributePrefix
 						<< escaped_string(part)
 						<< AttributePostfix;
 
 				} else {
 					if(first) {
-						out << std::string(tab, '\t') 
-							<< i->first 
+						out << std::string(tab, '\t')
+							<< i->first
 							<< AttributeEquals;
 					}
 
-					out << AttributePrefix 
+					out << AttributePrefix
 						<< escaped_string(part)
 						<< AttributePostfix;
 				}
@@ -417,7 +388,7 @@ static void write_internal(config const &cfg, std::ostream &out, std::string tex
 					out << AttributeContPostfix;
 					out << std::string(tab+1, '\t');
 				}
-				
+
 				first = false;
 			}
 		}
