@@ -1,4 +1,4 @@
-/* $Id: parser.hpp,v 1.6 2005/03/29 21:01:30 gruikya Exp $ */
+/* $Id: parser.hpp,v 1.9 2005/04/27 22:17:37 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -18,13 +18,9 @@
 #include <vector>
 
 class config;
-struct line_source;
-
-line_source get_line_source(std::vector< line_source > const &line_src, int line);
 
 //read data in, clobbering existing data.
-void read(config &cfg, std::istream &in, std::vector< line_source > const *lines = 0, 
-		std::string* error_log=NULL); //throws config::error
+void read(config &cfg, std::istream &in, std::string* error_log = NULL); //throws config::error
 
 void write(std::ostream &out, config const &cfg);
 
