@@ -1,6 +1,6 @@
-/* $Id: string_utils.hpp,v 1.10 2005/04/05 18:32:31 gruikya Exp $ */
+/* $Id: string_utils.hpp,v 1.14 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -18,9 +18,10 @@
 #include <map>
 #include <string>
 #include <vector>
-#include "SDL_types.h"
 
-class t_string;
+#include "SDL_types.h"
+#include "tstring.hpp"
+
 
 class variable_set
 {
@@ -70,7 +71,7 @@ std::string interpolate_variables_into_string(std::string const &str, variable_s
 class invalid_utf8_exception : public std::exception {
 };
 
-class utf8_iterator 
+class utf8_iterator
 {
 public:
 	typedef std::input_iterator_tag iterator_category;
@@ -104,6 +105,16 @@ std::string wchar_to_string(const wchar_t);
 ucs2_string utf8_string_to_ucs2_string(const utf8_string& src);
 utf8_string ucs2_string_to_utf8_string(const ucs2_string& src);
 
+/** Returns a version of the string with the first letter capitalized */
+utf8_string capitalize(const utf8_string&);
+/** Returns an uppercased version of the string */
+utf8_string uppercase(const utf8_string&);
+/** Returns a lowercased version of the string */
+utf8_string lowercase(const utf8_string&);
+
 }
 
+// handy wrapper around interpolate_variables_into_string and gettext
+std::string vgettext(const char*, const utils::string_map&);
+
 #endif
