@@ -1,4 +1,4 @@
-/* $Id: tokenizer.hpp,v 1.2 2005/03/29 20:19:33 gruikya Exp $ */
+/* $Id: tokenizer.hpp,v 1.5 2005/06/04 19:16:06 ott Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -19,14 +19,14 @@
 
 class config;
 
-struct token 
+struct token
 {
 	enum token_type {
 		STRING,
 		QSTRING,
 		UNTERMINATED_QSTRING,
 		MISC,
-		
+
 		LF = '\n',
 		EQUALS = '=',
 		COMMA = ',',
@@ -42,15 +42,14 @@ struct token
 	std::string value;
 };
 
-class tokenizer 
+class tokenizer
 {
 public:
 	tokenizer(std::istream& in);
 
 	const token& next_token();
 	const token& current_token();
-	const size_t get_line();
-	const size_t get_column();
+	std::string get_line();
 	std::string& textdomain();
 private:
 	void next_char();
@@ -58,16 +57,15 @@ private:
 	int peek_char();
 	bool is_space(int c);
 	bool is_alnum(int c);
-//	void textdomain_init(const std::string& domain, const std::string& path);
+	void skip_comment();
 
 	std::istream& in_;
 	int current_;
 
 	std::string textdomain_;
+	std::string file_;
 	size_t tokenstart_lineno_;
-	size_t tokenstart_colno_;
 	size_t lineno_;
-	size_t colno_;
 	token token_;
 };
 
