@@ -38,7 +38,7 @@ public:
 	void update_side_data();
 
 	const std::string& transfer_side_control(const config& cfg);
-	
+
 	size_t available_slots() const;
 
 	//function to set the description to the number of slots
@@ -58,6 +58,7 @@ public:
 	void send_data_observers(const config& data);
 
 	void record_data(const config& data);
+	void reset_history();
 
 	//the full scenario data
 	bool level_init() const;
@@ -109,8 +110,9 @@ private:
 	static int id_num;
 	int id_;
 	std::vector<network::connection> players_;
-	std::map<network::connection,std::string> sides_;
-	std::set<std::string> sides_taken_;
+	std::multimap<network::connection,size_t> sides_;
+	std::vector<bool> sides_taken_;
+	std::vector<std::string> side_controllers_;
 	bool started_;
 
 	config level_;
