@@ -50,7 +50,7 @@ config construct_server_message(const std::string& message, const game& g)
 	} else {
 		config& msg = turn.add_child("message");
 		msg["sender"] = "server";
-		msg["message"] = message; 
+		msg["message"] = message;
 	}
 
 	return turn;
@@ -161,7 +161,7 @@ void server::dump_stats()
 {
 	time_t old_stats = last_stats_;
 	last_stats_ = time(NULL);
-	
+
 	std::cout <<
 		"Statistics\n"
 		"\tnum_players = " << players_.size() << "\n"
@@ -232,7 +232,7 @@ void server::process_command(const std::string& cmd)
 			bans_.erase(itor,bans_.end());
 			std::cout << "ban removed on '" << mask << "'" << std::endl;
 		}
-		
+
 	} else if(command == "kick") {
 		if(i == cmd.end()) {
 			std::cout << "you must enter a nick to kick\n";
@@ -249,6 +249,7 @@ void server::process_command(const std::string& cmd)
 		}
 	} else {
 		std::cout << "command '" << command << "' is not recognized" << std::endl;
+		std::cout << "available commands are: msg <message>, status, metrics, ban [<nick>], unban <nick>, kick <nick>" << std::endl;
 	}
 }
 
@@ -272,7 +273,7 @@ void server::run()
 			if(input_.read_line(admin_cmd)) {
 				process_command(admin_cmd);
 			}
-			
+
 			//make sure we log stats every 5 minutes
 			if((loop%100) == 0 && last_stats_+5*60 < time(NULL)) {
 				dump_stats();
@@ -297,7 +298,7 @@ void server::run()
 			}
 
 			metrics_.no_requests();
-			
+
 		} catch(network::error& e) {
 			if(!e.socket) {
 				std::cerr << "fatal network error: " << e.message << "\n";
@@ -314,7 +315,7 @@ void server::run()
 
 					players_.erase(pl_it);
 				}
-				
+
 				not_logged_in_.remove_player(e.socket);
 				lobby_players_.remove_player(e.socket);
 				for(std::vector<game>::iterator i = games_.begin(); i != games_.end(); ++i) {
@@ -408,7 +409,7 @@ void server::process_login(const network::connection sock, const config& data, c
 		              "This username is already taken"),sock);
 		return;
 	}
-	
+
 	network::send_data(join_lobby_response_, sock);
 
 	config* const player_cfg = &initial_response_.add_child("user");
@@ -441,11 +442,11 @@ void server::process_query(const network::connection sock, const config& query,
 	std::ostringstream response;
 	if(query["type"] == "metrics") {
 		//a query for server data from a player
-		response << metrics_;	
+		response << metrics_;
 	} else {
 		response << "Error: unrecognized query";
 	}
-		
+
 	network::send_data(construct_server_message(response.str(),lobby_players_),sock);
 }
 
@@ -585,7 +586,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			name = u["username"];
 			ban = false;
 		}
-	
+
 		player_map::iterator pl;
 		for(pl = players_.begin(); pl != players_.end(); ++pl) {
 			if(pl->second.name() == name) {
@@ -599,7 +600,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 				const config& msg = construct_server_message("You have been banned",*g);
 				network::send_data(msg, pl->first);
 			} else {
-				g->remove_player(pl->first, false);
+				g->remove_player(pl->first);
 			}
 
 			config leave_game;
@@ -657,7 +658,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 				g->level().values["map_data"] = data["map_data"];
 				g->level().values["map"] = data["map"];
 			}
-			
+
 			//update our config object which describes the
 			//open games, and notifies the game of where its description
 			//is located at
@@ -684,6 +685,22 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 		return;
 	}
 
+	//if this is data telling us that the scenario did change.
+	else if(g->is_owner(sock) && data.child("next_scenario") != NULL) {
+		config* scenario = data.child("next_scenario");
+
+		if(g->level_init()) {
+			g->level() = *scenario;
+			g->reset_history();
+			g->update_side_data();
+		} else {
+			// next_scenario sent while the scenario was not
+			// initialized. Something's broken here.
+			std::cerr << "Error: next_scenario sent while the scenario is not yet initialized";
+			return;
+		}
+	}
+
 	const string_map::const_iterator side = data.values.find("side");
 	if(side != data.values.end()) {
 		const bool res = g->take_side(sock,data);
@@ -797,7 +814,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			lobby_players_.send_data(sync_initial_response());
 		}
 
-		//any private 'speak' commands must be repackaged seperate
+		//any private 'speak' commands must be repackaged separate
 		//to other commands, and re-sent, since they should only go
 		//to some clients.
 		const config::child_itors speaks = turn->child_range("command");
@@ -861,7 +878,7 @@ void server::delete_game(std::vector<game>::iterator i)
 		const size_t index = g - vg.first;
 		gamelist->remove_child("game",index);
 	}
-	
+
 	i->disconnect();
 	games_.erase(i);
 }
@@ -892,7 +909,7 @@ int main(int argc, char** argv)
 			std::cout << "usage: " << argv[0]
 				<< " [options]\n"
 				<< "  -d  --daemon               Runs wesnothd as a daemon\n"
-				<< "  -m, --max_packet_size n    Sets the maximal packet size to n\n" 
+				<< "  -m, --max_packet_size n    Sets the maximal packet size to n\n"
 				<< "  -p, --port                 Binds the server to the specified port\n"
 				<< "  -v, --version              Returns the server version\n";
 			return 0;
@@ -925,7 +942,7 @@ int main(int argc, char** argv)
 	}
 
 	input_stream input(fifo_path);
-	
+
 	try {
 		server(port,input).run();
 	} catch(network::error& e) {
