@@ -1,7 +1,7 @@
-/* $Id: show_dialog.cpp,v 1.124 2005/04/04 20:05:58 ydirson Exp $ */
+/* $Id: show_dialog.cpp,v 1.134 2005/06/19 13:32:13 Sirp Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -28,6 +28,7 @@
 #include "log.hpp"
 #include "playlevel.hpp"
 #include "show_dialog.hpp"
+#include "thread.hpp"
 #include "language.hpp"
 #include "sdl_utils.hpp"
 #include "tooltips.hpp"
@@ -35,6 +36,7 @@
 #include "video.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
+#include "widgets/progressbar.hpp"
 #include "widgets/textbox.hpp"
 #include "wassert.hpp"
 
@@ -263,7 +265,7 @@ void draw_dialog(int x, int y, int w, int h, CVideo &video, const std::string& t
 #ifdef OK_BUTTON_ON_RIGHT
 		std::reverse(buttons->begin(),buttons->end());
 #endif
-		
+
 		for(std::vector<button*>::const_iterator b = buttons->begin(); b != buttons->end(); ++b) {
 			(**b).set_location(buttons_area.x,buttons_area.y);
 			buttons_area.x += (**b).width() + ButtonHPadding;
@@ -294,7 +296,7 @@ private:
 
 	bool can_execute_command(hotkey::HOTKEY_COMMAND cmd) const
 	{
-		return topic_.empty() == false && cmd == hotkey::HOTKEY_HELP;
+		return (topic_.empty() == false && cmd == hotkey::HOTKEY_HELP) || cmd == hotkey::HOTKEY_SCREENSHOT;
 	}
 
 	display& disp_;
@@ -322,7 +324,7 @@ int show_dialog(display& disp, surface image,
 				int text_widget_max_chars,
 				dialog_action* action, std::vector<check_item>* options, int xloc, int yloc,
 				const std::string* dialog_style, std::vector<dialog_button>* action_buttons,
-				const std::string& help_topic)
+				const std::string& help_topic, const menu::sorter* sorter)
 {
 	if(disp.update_locked())
 		return -1;
@@ -361,7 +363,7 @@ int show_dialog(display& disp, surface image,
 	int text_widget_width = 0;
 	int text_widget_height = 0;
 	if(use_textbox) {
-		
+
 		text_widget.set_wrap(!editable_textbox);
 
 		const SDL_Rect& area = font::text_area(*text_widget_text,message_font_size);
@@ -372,7 +374,12 @@ int show_dialog(display& disp, surface image,
 		text_widget_height = text_widget.location().h + message_font_size;
 	}
 
-	menu menu_(screen,menu_items,type == MESSAGE);
+#ifdef USE_TINY_GUI
+	const int max_menu_width = 150;
+#else
+	const int max_menu_width = -1;
+#endif
+	menu menu_(screen,menu_items,type == MESSAGE,-1,max_menu_width,sorter);
 
 	menu_.set_numeric_keypress_selection(use_textbox == false);
 
@@ -406,7 +413,7 @@ int show_dialog(display& disp, surface image,
 			break;
 
 		case OK_ONLY: {
-			static const char* thebuttons[] = { N_("Ok"), "" };
+			static const char* thebuttons[] = { N_("OK"), "" };
 			button_list = thebuttons;
 			break;
 		}
@@ -419,7 +426,7 @@ int show_dialog(display& disp, surface image,
 		}
 
 		case OK_CANCEL: {
-			static const char* thebuttons[] = { N_("Ok"),
+			static const char* thebuttons[] = { N_("OK"),
 							    N_("Cancel"),""};
 			button_list = thebuttons;
 			break;
@@ -555,7 +562,7 @@ int show_dialog(display& disp, surface image,
 		if(yloc < yframe) {
 			yframe = yloc;
 		}
-	}	
+	}
 
 	std::vector<button*> buttons_ptr;
 	for(std::vector<button>::iterator bt = buttons.begin(); bt != buttons.end(); ++bt) {
@@ -574,7 +581,7 @@ int show_dialog(display& disp, surface image,
 
 	//calculate the positions of the preview panes to the sides of the dialog
 	if(preview_panes != NULL) {
-		
+
 		int left_preview_pane = xframe;
 		int right_preview_pane = xframe + total_width + left_preview_pane_width;
 		int above_left_preview_pane = xframe + frame_width/2;
@@ -744,7 +751,7 @@ int show_dialog(display& disp, surface image,
 			const int res = menu_.process();
 			if(res != -1)
 			{
-				return res;	
+				return res;
 			}
 		}
 
@@ -765,7 +772,7 @@ int show_dialog(display& disp, surface image,
 
 		left_button = new_left_button;
 		right_button = new_right_button;
-		key_down = new_key_down;	
+		key_down = new_key_down;
 
 		for(std::vector<button>::iterator button_it = buttons.begin();
 		    button_it != buttons.end(); ++button_it) {
@@ -811,6 +818,14 @@ int show_dialog(display& disp, surface image,
 				} else if(res == dialog_button_action::CLOSE_DIALOG) {
 					return -1;
 				}
+
+				//reset button-tracking flags so that if the action displays a dialog, a button-press
+				//at the end of the dialog won't be mistaken for a button-press in this dialog.
+				//(We should eventually use a proper event-handling system instead of tracking
+				//flags to avoid problems like this altogether).
+				left_button = true;
+				right_button = true;
+				key_down = true;
 			}
 		}
 
@@ -830,65 +845,106 @@ int show_dialog(display& disp, surface image,
 
 }
 
-namespace {
-class dialog_action_receive_network : public gui::dialog_action
-{
-public:
-	dialog_action_receive_network(network::connection connection, config& cfg, const std::pair<int,int>& connection_stats);
-	int do_action();
-	network::connection result() const;
-
-	enum { CONNECTION_COMPLETE = 1, CONNECTION_CONTINUING = 2 };
-private:
-	config& cfg_;
-	network::connection connection_, res_;
-	std::pair<int,int> stats_;
-};
-
-dialog_action_receive_network::dialog_action_receive_network(network::connection connection, config& cfg,
-															 const std::pair<int,int>& stats)
-: cfg_(cfg), connection_(connection), res_(0), stats_(stats)
-{
-}
-
-int dialog_action_receive_network::do_action()
-{
-	res_ = network::receive_data(cfg_,connection_,100);
-	if(res_ != 0)
-		return CONNECTION_COMPLETE;
-	else if(network::current_transfer_stats().first != stats_.first) {
-		LOG_STREAM(info, network) << "continuing connection...\n";
-		return CONNECTION_CONTINUING;
-	} else
-		return CONTINUE_DIALOG;
-}
-
-network::connection dialog_action_receive_network::result() const
-{
-	return res_;
-}
-
-}
-
 namespace gui {
 
 network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num)
 {
+	const std::string title = _("Receiving data...");
+
+	const size_t width = 300;
+	const size_t height = 80;
+	const size_t border = 20;
+	const int left = disp.x()/2 - width/2;
+	const int top = disp.y()/2 - height/2;
+
+	gui::button cancel_button(disp.video(),_("Cancel"));
+	std::vector<gui::button*> buttons_ptr(1,&cancel_button);
+
+	surface_restorer restorer;
+	gui::draw_dialog(left,top,width,height,disp.video(),title,NULL,&buttons_ptr,&restorer);
+
+	const SDL_Rect progress_rect = {left+border,top+border,width-border*2,height-border*2};
+	gui::progress_bar progress(disp.video());
+	progress.set_location(progress_rect);
+
+	events::raise_draw_event();
+	disp.flip();
+
+	std::pair<int,int> old_stats = network::current_transfer_stats();
+
 	cfg.clear();
 	for(;;) {
+		const network::connection res = network::receive_data(cfg,connection_num,100);
+
 		const std::pair<int,int> stats = network::current_transfer_stats();
-		std::stringstream str;
-		str << msg;
-		if(stats.first != -1) {
-			str << ": " << (stats.first/1024) << "/" << (stats.second/1024) << _("KB");
+		if(stats.first != -1 && stats.second != 0 && stats != old_stats) {
+			old_stats = stats;
+			progress.set_progress_percent((stats.first*100)/stats.second);
+			std::ostringstream stream;
+			stream << stats.first/1024 << "/" << stats.second/1024 << _("KB");
+			progress.set_text(stream.str());
 		}
 
-		dialog_action_receive_network receiver(connection_num,cfg,stats);
-		const int res = show_dialog(disp,NULL,"",str.str(),CANCEL_ONLY,NULL,NULL,"",NULL,-1,&receiver);
-		if(res != int(dialog_action_receive_network::CONNECTION_CONTINUING)) {
-			return receiver.result();
+		events::raise_draw_event();
+		disp.flip();
+
+		if(res != 0) {
+			return res;
+		}
+
+		events::pump();
+		if(cancel_button.pressed()) {
+			return res;
 		}
 	}
 }
 
+namespace {
+
+class connect_waiter : public threading::waiter
+{
+public:
+	connect_waiter(display& disp, gui::button& button) : disp_(disp), button_(button)
+	{}
+	ACTION process();
+
+private:
+	display& disp_;
+	gui::button& button_;
+};
+
+connect_waiter::ACTION connect_waiter::process()
+{
+	events::raise_draw_event();
+	disp_.flip();
+	events::pump();
+	if(button_.pressed()) {
+		return ABORT;
+	} else {
+		return WAIT;
+	}
+}
+
+}
+
+network::connection network_connect_dialog(display& disp, const std::string& msg, const std::string& hostname, int port)
+{
+	const size_t width = 250;
+	const size_t height = 20;
+	const int left = disp.x()/2 - width/2;
+	const int top = disp.y()/2 - height/2;
+
+	gui::button cancel_button(disp.video(),_("Cancel"));
+	std::vector<gui::button*> buttons_ptr(1,&cancel_button);
+
+	surface_restorer restorer;
+	gui::draw_dialog(left,top,width,height,disp.video(),msg,NULL,&buttons_ptr,&restorer);
+
+	events::raise_draw_event();
+	disp.flip();
+
+	connect_waiter waiter(disp,cancel_button);
+	return network::connect(hostname,port,waiter);
+}
+
 } //end namespace gui
