@@ -1,7 +1,7 @@
-/* $Id: show_dialog.hpp,v 1.47 2005/03/23 21:30:46 ydirson Exp $ */
+/* $Id: show_dialog.hpp,v 1.51 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -24,6 +24,7 @@ class CVideo;
 #include "tooltips.hpp"
 
 #include "widgets/button.hpp"
+#include "widgets/menu.hpp"
 
 #include "SDL.h"
 
@@ -133,12 +134,14 @@ int show_dialog(display &screen, surface image,
 				std::vector<check_item>* options=NULL, int xloc=-1, int yloc=-1,
 				const std::string* dialog_style=NULL,
 				std::vector<dialog_button>* buttons=NULL,
-				const std::string& help_topic=""
+				const std::string& help_topic="",
+				const menu::sorter* sorter=NULL
 			 );
 
 void show_error_message(display &disp, std::string const &message);
 
 network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num=0);
+network::connection network_connect_dialog(display& disp, const std::string& msg, const std::string& hostname, int port);
 
 void check_quit(CVideo &video);
 
