@@ -1,7 +1,7 @@
-/* $Id: sound.cpp,v 1.28 2005/02/08 21:38:41 silene Exp $ */
+/* $Id: sound.cpp,v 1.30 2005/06/06 20:12:47 gruikya Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -19,6 +19,7 @@
 #include "sound.hpp"
 #include "wesconfig.h"
 
+#include "SDL.h"
 #include "SDL_mixer.h"
 
 #include <iostream>
@@ -45,6 +46,8 @@ manager::manager(bool sound_on)
 		return;
 	}
 
+	SDL_Init(SDL_INIT_AUDIO);
+
 //sounds don't sound good on Windows unless the buffer size is 4k,
 //but this seems to cause crashes on other systems...
 #ifdef WIN32
@@ -100,7 +103,7 @@ void play_music(const std::string& file)
 	std::map<std::string,Mix_Music*>::const_iterator itor = music_cache.find(file);
 	if(itor == music_cache.end()) {
 		const std::string& filename = get_binary_file_location("music",file);
-		
+
 		if(filename.empty()) {
 			return;
 		}
