@@ -434,7 +434,8 @@ bool theme::set_resolution(const SDL_Rect& screen)
 
 	const config::child_list& resolutions = cfg_.get_children("resolution");
 	int current_rating = 1000000;
-	config::child_list::const_iterator i, current = resolutions.end();
+	config::child_list::const_iterator i;
+	config::child_list::const_iterator current = resolutions.end();
 	for(i = resolutions.begin(); i != resolutions.end(); ++i) {
 		const int width = atoi((**i)["width"].c_str());
 		const int height = atoi((**i)["height"].c_str());
@@ -479,7 +480,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	} else {
 		mini_map_ = object();
 	}
-	
+
 	const config* const status_cfg = cfg.child("status");
 	if(status_cfg != NULL) {
 		for(config::child_map::const_iterator i = status_cfg->all_children().begin(); i != status_cfg->all_children().end(); ++i) {
@@ -560,7 +561,7 @@ void theme::set_known_themes(const config* cfg){
 	       return;
 	const config& v = *cfg;
 	const config::child_list& known_themes_cfg = v.get_children("theme");
-	
+
 	for(config::child_list::const_iterator thm = known_themes_cfg.begin(); thm != known_themes_cfg.end(); ++thm) {
 	       std::string thm_name=(**thm)["name"];
 	       if(thm_name!="null" && thm_name!="editor"){
@@ -571,7 +572,7 @@ void theme::set_known_themes(const config* cfg){
 
 std::vector<std::string> theme::get_known_themes(){
         std::vector<std::string> names;
- 
+
 
         for(std::map<std::string, config>::iterator p_thm=known_themes.begin();p_thm!=known_themes.end();p_thm++){
 	  names.push_back(p_thm->first);
