@@ -1,6 +1,7 @@
-/* $Id: tstring.cpp,v 1.5 2005/04/09 17:36:07 silene Exp $ */
+/* $Id: tstring.cpp,v 1.14 2005/06/04 19:16:06 ott Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -11,6 +12,9 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
+#include <sstream>
 #include <vector>
 #include <map>
 
@@ -19,6 +23,7 @@
 #include "filesystem.hpp"
 #include "log.hpp"
 
+#define LOG_CF lg::info(lg::config)
 #define ERR_CF lg::err(lg::config)
 
 namespace {
@@ -100,7 +105,7 @@ void t_string::walker::update()
 
 	switch(string_[begin_]) {
 	case TRANSLATABLE_PART: {
-		std::string::size_type textdomain_end = 
+		std::string::size_type textdomain_end =
 			string_.find(TEXTDOMAIN_SEPARATOR, begin_ + 1);
 
 		if(textdomain_end == std::string::npos || textdomain_end >= string_.size() - 1) {
@@ -173,7 +178,9 @@ t_string::t_string() :
 
 t_string::t_string(const t_string& string) :
 	translatable_(string.translatable_),
-	value_(string.value_)
+	last_untranslatable_(string.last_untranslatable_),
+	value_(string.value_),
+	translated_value_(string.translated_value_)
 {
 }
 
@@ -185,19 +192,16 @@ t_string::t_string(const std::string& string) :
 
 t_string::t_string(const std::string& string, const std::string& textdomain) :
 	translatable_(true),
+	last_untranslatable_(false),
 	value_(1, ID_TRANSLATABLE_PART)
 {
 	std::map<std::string, unsigned int>::const_iterator idi = textdomain_to_id.find(textdomain);
 	unsigned int id;
 
 	if(idi == textdomain_to_id.end()) {
-		textdomain_to_id[textdomain] = id_to_textdomain.size();
 		id = id_to_textdomain.size();
+		textdomain_to_id[textdomain] = id;
 		id_to_textdomain.push_back(textdomain);
-
-		// Register and bind this textdomain
-		bindtextdomain(textdomain.c_str(), get_intl_dir().c_str());
-		bind_textdomain_codeset(textdomain.c_str(), "UTF-8");
 	} else {
 		id = idi->second;
 	}
@@ -213,20 +217,16 @@ t_string::t_string(const char* string) :
 {
 }
 
-t_string::~t_string()
-{
-}
-
 t_string t_string::from_serialized(const std::string& string)
 {
 	t_string orig(string);
-	
+
 	if(!string.empty() && (string[0] == TRANSLATABLE_PART || string[0] == UNTRANSLATABLE_PART)) {
 		orig.translatable_ = true;
 	} else {
 		orig.translatable_ = false;
 	}
-	
+
 	t_string res;
 
 	for(walker w(orig); !w.eos(); w.next()) {
@@ -252,7 +252,8 @@ std::string t_string::to_serialized() const
 		std::string substr(w.begin(), w.end());
 		if(w.translatable()) {
 			chunk.translatable_ = true;
-			chunk.value_ = TRANSLATABLE_PART + w.textdomain() + 
+			chunk.last_untranslatable_ = false;
+			chunk.value_ = TRANSLATABLE_PART + w.textdomain() +
 				TEXTDOMAIN_SEPARATOR + substr;
 		} else {
 			chunk.translatable_ = false;
@@ -269,6 +270,7 @@ t_string& t_string::operator=(const t_string& string)
 {
 	value_ = string.value_;
 	translatable_ = string.translatable_;
+	last_untranslatable_ = string.last_untranslatable_;
 	translated_value_ = string.translated_value_;
 
 	return *this;
@@ -309,91 +311,55 @@ bool t_string::operator==(const char* string) const
 
 t_string t_string::operator+(const t_string& string) const
 {
-	t_string res;
-
-	if(translatable_ || string.translatable_) {
-		res.translatable_ = true;
-		if(translatable_) {
-			res.value_ += value_;
-		} else {
-			if (!value_.empty()) {
-				res.value_ += UNTRANSLATABLE_PART;
-				res.value_ += value_;
-			}
-		}
-		if(string.translatable_) {
-			res.value_ += string.value_;
-		} else {
-			if (!string.empty()) {
-				res.value_ += UNTRANSLATABLE_PART;
-				res.value_ += string.value_;
-			}
-		}
-	} else {
-		res.translatable_ = false;
-		res.value_ = value_ + string.value_;
-	}
-
+	t_string res(*this);
+	res += string;
 	return res;
 }
 
 t_string t_string::operator+(const std::string& string) const
 {
-	t_string res;
-
-	if(translatable_) {
-		res.translatable_ = true;
-		res.value_ = value_;
-		if (!string.empty()) {
-			res.value_ += UNTRANSLATABLE_PART;
-			res.value_ += string;
-		}
-	} else {
-		res.translatable_ = false;
-		res.value_ = value_ + string;
-	}
-
+	t_string res(*this);
+	res += string;
 	return res;
 }
 
 t_string t_string::operator+(const char* string) const
 {
-	t_string res;
-
-	if(translatable_) {
-		res.translatable_ = true;
-		res.value_ = value_;
-		if (string[0] != 0) {
-			res.value_ += UNTRANSLATABLE_PART;
-			res.value_ += string;
-		}
-	} else {
-		res.translatable_ = false;
-		res.value_ = value_ + string;
-	}
-
+	t_string res(*this);
+	res += string;
 	return res;
 }
 
 t_string& t_string::operator+=(const t_string& string)
 {
+	if (string.value_.empty())
+		return *this;
+	if (value_.empty()) {
+		*this = string;
+		return *this;
+	}
+
 	if(translatable_ || string.translatable_) {
 		if(!translatable_) {
-			if (!value_.empty()) {
-				value_ = UNTRANSLATABLE_PART + value_;
-			}
-		}
+			value_ = UNTRANSLATABLE_PART + value_;
+			translatable_ = true;
+			last_untranslatable_ = true;
+		} else
+			translated_value_ = "";
 		if(string.translatable_) {
-			value_ += string.value_;
-		} else {
-			if (!string.value_.empty()) {
-				value_ += UNTRANSLATABLE_PART;
+			if (last_untranslatable_ && string.value_[0] == UNTRANSLATABLE_PART)
+				value_.append(string.value_.begin() + 1, string.value_.end());
+			else
 				value_ += string.value_;
+			last_untranslatable_ = string.last_untranslatable_;
+		} else {
+			if (!last_untranslatable_) {
+				value_ += UNTRANSLATABLE_PART;
+				last_untranslatable_ = true;
 			}
+			value_ += string.value_;
 		}
-		translatable_ = true;
 	} else {
-		translatable_ = false;
 		value_ += string.value_;
 	}
 
@@ -402,11 +368,20 @@ t_string& t_string::operator+=(const t_string& string)
 
 t_string& t_string::operator+=(const std::string& string)
 {
+	if (string.empty())
+		return *this;
+	if (value_.empty()) {
+		*this = string;
+		return *this;
+	}
+
 	if(translatable_) {
-		if (!string.empty()) {
+		if (!last_untranslatable_) {
 			value_ += UNTRANSLATABLE_PART;
-			value_ += string;
+			last_untranslatable_ = true;
 		}
+		value_ += string;
+		translated_value_ = "";
 	} else {
 		value_ += string;
 	}
@@ -414,13 +389,22 @@ t_string& t_string::operator+=(const std::string& string)
 	return *this;
 }
 
-t_string& t_string::operator+=(const char* string) 
+t_string& t_string::operator+=(const char* string)
 {
+	if (string[0] == 0)
+		return *this;
+	if (value_.empty()) {
+		*this = string;
+		return *this;
+	}
+
 	if(translatable_) {
-		if (string[0] != 0) {
+		if (!last_untranslatable_) {
 			value_ += UNTRANSLATABLE_PART;
-			value_ += string;
+			last_untranslatable_ = true;
 		}
+		value_ += string;
+		translated_value_ = "";
 	} else {
 		value_ += string;
 	}
@@ -432,12 +416,12 @@ bool t_string::operator!=(const t_string& string) const { return !(*this == stri
 bool t_string::operator!=(const std::string& string) const { return !(*this == string); }
 bool t_string::operator!=(const char* string) const { return !(*this == string); }
 
-bool t_string::operator<(const t_string& string) const 
+bool t_string::operator<(const t_string& string) const
 {
 	return value_ < string.value_;
 }
 
-bool t_string::empty() const 
+bool t_string::empty() const
 {
 	return value_.empty();
 }
@@ -469,7 +453,7 @@ const std::string& t_string::str() const
 			translated_value_ += part;
 		}
 	}
-	
+
 	return translated_value_;
 }
 
@@ -483,30 +467,44 @@ const std::string& t_string::value() const
 	return value_;
 }
 
+void t_string::reset_translation() const
+{
+	translated_value_ = "";
+}
+
+void t_string::add_textdomain(const std::string& name, const std::string& path)
+{
+	LOG_CF << "Binding textdomain " << name << " to path " << path << "\n";
+
+	// Register and (re-)bind this textdomain
+	bindtextdomain(name.c_str(), path.c_str());
+	bind_textdomain_codeset(name.c_str(), "UTF-8");
+}
+
 std::ostream& operator<<(std::ostream& stream, const t_string& string)
 {
 	stream << string.str();
 	return stream;
 }
 
-bool operator==(const std::string& a, const t_string& b) 
-{ 
-	return b == a; 
+bool operator==(const std::string& a, const t_string& b)
+{
+	return b == a;
 }
 
-bool operator==(const char* a, const t_string& b) 
-{ 
-	return b == a; 
+bool operator==(const char* a, const t_string& b)
+{
+	return b == a;
 }
 
-bool operator!=(const std::string& a, const t_string& b) 
-{ 
-	return b != a; 
+bool operator!=(const std::string& a, const t_string& b)
+{
+	return b != a;
 }
 
-bool operator!=(const char* a, const t_string& b) 
-{ 
-	return b != a; 
+bool operator!=(const char* a, const t_string& b)
+{
+	return b != a;
 }
 
 t_string operator+(const std::string& a, const t_string& b)
