@@ -1,7 +1,7 @@
-/* $Id: unit.cpp,v 1.138 2005/04/07 21:16:50 gruikya Exp $ */
+/* $Id: unit.cpp,v 1.154 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -26,6 +26,10 @@
 #include "wassert.hpp"
 #include "serialization/string_utils.hpp"
 
+//DEBUG
+#include <iostream>
+#include <serialization/parser.hpp>
+
 #include <algorithm>
 #include <cstdlib>
 #include <iostream>
@@ -39,7 +43,7 @@ namespace {
 	                                    sizeof(*ModificationTypes);
 }
 
-bool compare_unit_values::operator()(const unit& a, const unit& b) const
+static bool compare_unit_values(unit const &a, unit const &b)
 {
 	const int lvla = a.type().level();
 	const int lvlb = b.type().level();
@@ -54,12 +58,16 @@ bool compare_unit_values::operator()(const unit& a, const unit& b) const
 	                      lvla == lvlb && namea == nameb && xpa < xpb;
 }
 
+void sort_units(std::vector< unit > &units)
+{
+	std::sort(units.begin(), units.end(), compare_unit_values);
+}
+
 //constructor for reading a unit
-unit::unit(const game_data& data, const config& cfg) : 
+unit::unit(const game_data& data, const config& cfg) :
 	state_(STATE_NORMAL),
 	moves_(0), user_end_turn_(false), facingLeft_(true),
-	resting_(false),
-	recruit_(false),
+	resting_(false), hold_position_(false), recruit_(false),
 	guardian_(false), upkeep_(UPKEEP_FREE)
 {
 	read(data,cfg);
@@ -88,7 +96,8 @@ unit::unit(const unit_type* t, int side, bool use_traits, bool dummy_unit, unit_
 	       user_end_turn_(false), facingLeft_(side != 1),
 	       maxMovement_(type_->movement()),
 	       backupMaxMovement_(type_->movement()),
-	       recruit_(false), attacks_(type_->attacks()),
+	       resting_(false), hold_position_(false), recruit_(false),
+	       attacks_(type_->attacks()),
 	       backupAttacks_(type_->attacks()),
                guardian_(false), upkeep_(UPKEEP_FULL_PRICE),
                unrenamable_(false)
@@ -120,6 +129,7 @@ unit::unit(const unit_type* t, const unit& u) :
 	user_end_turn_(false), facingLeft_(u.facingLeft_),
 	maxMovement_(type_->movement()),
 	backupMaxMovement_(type_->movement()),
+	resting_(u.resting_), hold_position_(u.hold_position_),
 	underlying_description_(u.underlying_description_),
 	description_(u.description_), recruit_(u.recruit_),
 	role_(u.role_), statusFlags_(u.statusFlags_),
@@ -154,8 +164,6 @@ void unit::generate_traits()
 		candidate_traits.erase(candidate_traits.begin()+num);
 	}
 
-	std::vector<std::string> description;
-	
 	for(std::vector<config*>::const_iterator j = traits.begin(); j != traits.end(); ++j) {
 		modifications_.add_child("trait",**j);
 	}
@@ -254,10 +262,22 @@ bool unit::can_attack() const
 
 void unit::set_movement(int moves)
 {
+	hold_position_ = false;
 	user_end_turn_ = false;
 	moves_ = moves;
 }
 
+//This does not mark the unit's turn as being done even if value is true. To do that, either call void unit_hold_position(), or set_user_end_turn(value).
+void unit::set_hold_position(bool value)
+{
+	hold_position_ = value;
+}
+
+bool unit::hold_position() const
+{
+	return hold_position_;
+}
+
 void unit::set_user_end_turn(bool value)
 {
 	user_end_turn_ = value;
@@ -270,7 +290,14 @@ bool unit::user_end_turn() const
 
 void unit::set_attacked()
 {
-	moves_ = ATTACKED; 
+	moves_ = ATTACKED;
+	set_hold_position(false);
+}
+
+void unit::unit_hold_position()
+{
+	hold_position_ = true;
+	user_end_turn_ = true;
 }
 
 void unit::end_unit_turn()
@@ -292,6 +319,9 @@ void unit::new_turn()
 		set_flag("nightstalk");
 	if(stone())
 		set_attacked();
+	if (hold_position_) {
+		user_end_turn_ = true;
+	}
 }
 
 bool unit::move_interrupted() const {
@@ -338,7 +368,7 @@ void unit::new_level()
 
 void unit::set_resting(bool resting)
 {
-	resting_ = resting;	
+	resting_ = resting;
 }
 
 bool unit::is_resting() const
@@ -407,8 +437,8 @@ static bool is_terrain(std::string const &terrain, gamemap::TERRAIN type) {
 	return std::count(terrain.begin(), terrain.end(), static_cast<gamemap::TERRAIN>(type)) != 0;
 }
 
-bool unit::invisible(const std::string& terrain, int lawful_bonus, 
-		const gamemap::location& loc, 
+bool unit::invisible(const std::string& terrain, int lawful_bonus,
+		const gamemap::location& loc,
 		const unit_map& units,const std::vector<team>& teams) const
 {
 	bool is_inv = false;
@@ -428,7 +458,7 @@ bool unit::invisible(const std::string& terrain, int lawful_bonus,
 	if(is_inv){
 		for(unit_map::const_iterator u = units.begin(); u != units.end(); ++u) {
 			if(teams[side_-1].is_enemy(u->second.side())) {
-				if(tiles_adjacent(loc,u->first)) 
+				if(tiles_adjacent(loc,u->first))
 					return false;
 			}
 		}
@@ -599,10 +629,13 @@ const std::vector<std::string>& unit::overlays() const
 void unit::read(const game_data& data, const config& cfg)
 {
 	std::map<std::string,unit_type>::const_iterator i = data.unit_types.find(cfg["type"]);
-	if(i != data.unit_types.end())
+	if(i != data.unit_types.end()) {
 		type_ = &i->second;
-	else
+	} else {
+		//DEBUG
+		::write(std::cerr, cfg);
 		throw game::load_game_failed("Unit not found: '" + cfg["type"] + "'");
+	}
 
 	wassert(type_ != NULL);
 
@@ -788,7 +821,7 @@ void unit::write(config& cfg) const
 	sprintf(buf,"%d",moves_);
 	cfg["moves"] = buf;
 
-	cfg["resting"] = resting_ ? "yes" : "no"; 
+	cfg["resting"] = resting_ ? "yes" : "no";
 	cfg["unrenamable"] = unrenamable_ ? "yes" : "no";
 }
 
@@ -882,7 +915,7 @@ void unit::set_defending(bool hits, attack_type::RANGE range, int start_frame, i
 {
 	state_ = range == attack_type::LONG_RANGE ? STATE_DEFENDING_LONG : STATE_DEFENDING_SHORT;
 	getsHit_ = hits;
-					   
+
 	const unit_animation* const anim = get_animation();
 	if(anim != NULL) {
 		anim_ = *anim;
@@ -1026,10 +1059,10 @@ void unit::add_modification(const std::string& type,
 					if(first_attack) {
 						first_attack = false;
 					} else {
-						description += "; ";
+						description += t_string(N_("; "), "wesnoth");
 					}
-					
-					description += a->name() + " " + desc;
+
+					description += t_string(a->name(), "wesnoth") + " " + desc;
 				}
 			}
 		} else if(apply_to == "hitpoints") {
@@ -1043,12 +1076,12 @@ void unit::add_modification(const std::string& type,
 
 			if(increase_total.empty() == false) {
 				description += (increase_total[0] != '-' ? "+" : "") + increase_total +
-					t_string(N_("HP"), "wesnoth");
+					" " + t_string(N_("HP"), "wesnoth");
 
 				//a percentage on the end means increase by that many percent
 				if(increase_total[increase_total.size()-1] == '%') {
 					const std::string inc(increase_total.begin(),increase_total.end()-1);
-					maxHitpoints_ += (maxHitpoints_*atoi(inc.c_str()))/100;
+					maxHitpoints_ += div100(maxHitpoints_ * atoi(inc.c_str()));
 				} else {
 					maxHitpoints_ += atoi(increase_total.c_str());
 				}
@@ -1079,12 +1112,12 @@ void unit::add_modification(const std::string& type,
 			const std::string& set_to = (**i.first)["set"];
 
 			if(increase.empty() == false) {
-				description += (increase[0] != '-' ? "+" : "") + increase + 
-					t_string(N_("Moves"), "wesnoth");
+				description += (increase[0] != '-' ? "+" : "") + increase +
+					" " + t_string(N_("Moves"), "wesnoth");
 
 				if(increase[increase.size()-1] == '%') {
 					const std::string inc(increase.begin(),increase.end()-1);
-					maxMovement_ += (maxMovement_*atoi(inc.c_str()))/100;
+					maxMovement_ += div100(maxMovement_ * atoi(inc.c_str()));
 				} else {
 					maxMovement_ += atoi(increase.c_str());
 				}
@@ -1104,11 +1137,12 @@ void unit::add_modification(const std::string& type,
 
 			if(increase.empty() == false) {
 				description += (increase[0] != '-' ? "+" : "") +
-					increase + t_string(N_("XP"), "wesnoth");
+					increase + " " +
+					t_string(N_("XP to advance"), "wesnoth");
 
 				if(increase[increase.size()-1] == '%') {
 					const std::string inc(increase.begin(),increase.end()-1);
-					maxExperience_ += (maxExperience_*atoi(inc.c_str()))/100;
+					maxExperience_ += div100(maxExperience_ * atoi(inc.c_str()));
 				} else {
 					maxExperience_ += atoi(increase.c_str());
 				}
@@ -1127,7 +1161,7 @@ void unit::add_modification(const std::string& type,
 			if(add.empty() == false) {
 				set_flag(add);
 			}
-			
+
 			if(remove.empty() == false) {
 				remove_flag(remove);
 			}
@@ -1139,22 +1173,22 @@ void unit::add_modification(const std::string& type,
 
 	t_string& description = modificationDescriptions_[type];
 
-	// FIXME: the colon must be translatable.
-	description += mod["name"] + ": ";
-	if(!mod["description"].empty()) {
-		description += mod["description"];
-		description += " ";
-	}
+	// Punctuation should be translatable: not all languages use latin punctuation.
+	// (however, there maybe is a better way to do it)
+	description += mod["name"];
+	if (!mod["description"].empty())
+		description += t_string(N_(": "), "wesnoth") + mod["description"];
+	description += " ";
 
 	if(effects_description.empty() == false) {
-		description += "(";
-		for(std::vector<t_string>::const_iterator i = effects_description.begin(); 
+		description += t_string(N_("("), "wesnoth");
+		for(std::vector<t_string>::const_iterator i = effects_description.begin();
 				i != effects_description.end(); ++i) {
 			description += *i;
 			if(i+1 != effects_description.end())
-				description += "; ";
+				description += t_string(N_("; "), "wesnoth");
 		}
-		description += ")";
+		description += t_string(N_(")"), "wesnoth");
 	}
 
 	description += "\n";
@@ -1177,32 +1211,36 @@ void unit::apply_modifications()
 	log_scope("apply mods");
 	modificationDescriptions_.clear();
 
-	std::vector<t_string> descriptions;
-
 	for(size_t i = 0; i != NumModificationTypes; ++i) {
 		const std::string& mod = ModificationTypes[i];
 		const config::child_list& mods = modifications_.get_children(mod);
 		for(config::child_list::const_iterator j = mods.begin(); j != mods.end(); ++j) {
 			log_scope("add mod");
 			add_modification(ModificationTypes[i],**j,true);
-
-			const t_string& name = (**j)["name"];
-			if(name.empty() == false) {
-				descriptions.push_back(name);
-			}
 		}
 	}
 
 	traitsDescription_ = "";
 
-	//we want to make sure the description always has a consistent ordering
-	std::sort(descriptions.begin(),descriptions.end());
-	for(std::vector<t_string>::const_iterator j = descriptions.begin(); j != descriptions.end(); ++j) {
-		if(j != descriptions.begin()) {
+	std::vector< t_string > traits;
+	config::child_list const &mods = modifications_.get_children("trait");
+	for(config::child_list::const_iterator j = mods.begin(), j_end = mods.end(); j != j_end; ++j) {
+		t_string const &name = (**j)["name"];
+		if (!name.empty())
+			traits.push_back(name);
+	}
+
+	std::vector< t_string >::iterator k = traits.begin(), k_end = traits.end();
+	if (k != k_end) {
+		// we want to make sure the traits always have a consistent ordering
+	        // try out not sorting, since quick,resilient can give
+		// different HP to resilient,quick so rather preserve order
+		// std::sort(k, k_end);
+		for(;;) {
+			traitsDescription_ += *(k++);
+			if (k == k_end) break;
 			traitsDescription_ += ", ";
 		}
-
-		traitsDescription_ += *j;
 	}
 }
 
@@ -1291,8 +1329,8 @@ std::string team_name(int side, const unit_map& units)
 }
 
 unit_map::iterator find_visible_unit(unit_map& units,
-		const gamemap::location loc, 
-		const gamemap& map, int lawful_bonus, 
+		const gamemap::location loc,
+		const gamemap& map, int lawful_bonus,
 		const std::vector<team>& teams, const team& current_team)
 {
 	unit_map::iterator u = units.find(loc);
@@ -1310,8 +1348,8 @@ unit_map::iterator find_visible_unit(unit_map& units,
 }
 
 unit_map::const_iterator find_visible_unit(const unit_map& units,
-		const gamemap::location loc, 
-		const gamemap& map, int lawful_bonus, 
+		const gamemap::location loc,
+		const gamemap& map, int lawful_bonus,
 		const std::vector<team>& teams, const team& current_team)
 {
 	unit_map::const_iterator u = units.find(loc);
