@@ -1,7 +1,7 @@
-/* $Id: unit_types.cpp,v 1.96 2005/03/29 17:41:09 darthfool Exp $ */
+/* $Id: unit_types.cpp,v 1.104 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -27,7 +27,7 @@
 
 unit_animation::frame::frame(const config& cfg)
 {
-	xoffset = atoi(cfg["xoffset"].c_str()); 
+	xoffset = atoi(cfg["xoffset"].c_str());
 	image = cfg["image"];
 	image_diagonal = cfg["image_diagonal"];
 	halo = cfg["halo"];
@@ -38,10 +38,10 @@ unit_animation::frame::frame(const config& cfg)
 unit_animation::unit_animation()
 {}
 
-unit_animation::unit_animation(const config& cfg) 
+unit_animation::unit_animation(const config& cfg)
 {
 	config::const_child_itors range = cfg.child_range("frame");
-	
+
 	int last_end = INT_MIN;
 	for(; range.first != range.second; ++range.first) {
 		unit_frames_.add_frame(atoi((**range.first)["begin"].c_str()), frame(**range.first));
@@ -174,7 +174,7 @@ attack_type::attack_type(const config& cfg)
 	  defense_weight_ = 1.0;
 }
 
-const t_string& attack_type::name() const
+const std::string& attack_type::name() const
 {
 	return name_;
 }
@@ -294,11 +294,10 @@ bool attack_type::apply_modification(const config& cfg, std::string* description
 
 	if(increase_damage.empty() == false) {
 		int increase = 0;
-		
+
 		if(increase_damage[increase_damage.size()-1] == '%') {
 			const std::string inc(increase_damage.begin(),increase_damage.end()-1);
-			const int percent = atoi(inc.c_str());
-			increase = (damage_*percent)/100;
+			increase = div100(damage_ * atoi(inc.c_str()));
 		} else {
 			increase = atoi(increase_damage.c_str());
 		}
@@ -309,19 +308,27 @@ bool attack_type::apply_modification(const config& cfg, std::string* description
 		}
 
 		if(description != NULL) {
-			desc << (increase_damage[0] == '-' ? "" : "+") << increase_damage << _("Dmg");
+			desc << (increase_damage[0] == '-' ? "" : "+") << increase_damage << " " << _("damage");
 		}
 	}
 
 	if(increase_attacks.empty() == false) {
-		const int increase = atoi(increase_attacks.c_str());
+		int increase = 0;
+
+		if(increase_attacks[increase_attacks.size()-1] == '%') {
+			const std::string inc(increase_attacks.begin(),increase_attacks.end()-1);
+			increase = div100(num_attacks_ * atoi(inc.c_str()));
+		} else {
+			increase = atoi(increase_attacks.c_str());
+		}
+
 		num_attacks_ += increase;
 		if(num_attacks_ < 1) {
 			num_attacks_ = 1;
 		}
 
 		if(description != NULL) {
-			desc << (increase > 0 ? "+" : "") << increase << _("strikes");
+			desc << (increase_attacks[0] == '-' ? "" : "+") << increase_attacks << " " << _("strikes");
 		}
 	}
 
@@ -527,7 +534,8 @@ unit_type::unit_type(const unit_type& o)
       leadership_(o.leadership_), illuminates_(o.illuminates_),
       skirmish_(o.skirmish_), teleport_(o.teleport_),
       nightvision_(o.nightvision_), steadfast_(o.steadfast_),
-      can_advance_(o.can_advance_), alignment_(o.alignment_),
+      advances_to_(o.advances_to_), experience_needed_(o.experience_needed_),
+      alignment_(o.alignment_),
       movementType_(o.movementType_), possibleTraits_(o.possibleTraits_),
       genders_(o.genders_), defensive_animations_(o.defensive_animations_),
       teleport_animations_(o.teleport_animations_), death_animations_(o.death_animations_)
@@ -621,7 +629,7 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 	teleport_ = has_ability("teleport");
 	nightvision_ = has_ability("night vision");
 	steadfast_ = has_ability("steadfast");
-	
+
 	const std::string& align = cfg_["alignment"];
 	if(align == "lawful")
 		alignment_ = LAWFUL;
@@ -647,7 +655,11 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 		movementType_.set_parent(&(it->second));
 	}
 
-	can_advance_ = advances_to().empty() == false;
+	const std::string& advance_to_val = cfg_["advanceto"];
+	if(advance_to_val != "null" && advance_to_val != "")
+		advances_to_ = utils::split(advance_to_val);
+
+	experience_needed_=lexical_cast_default<int>(cfg_["experience"],500);
 
 	const config::child_list& defends = cfg_.get_children("defend");
 	for(config::child_list::const_iterator d = defends.begin(); d != defends.end(); ++d) {
@@ -694,7 +706,9 @@ const unit_type& unit_type::get_variation(const std::string& name) const
 	}
 }
 
-int unit_type::num_traits() const { return race_->num_traits(); }
+int unit_type::num_traits() const {
+  return (cfg_["num_traits"].size() ? atoi(cfg_["num_traits"].c_str()) : race_->num_traits());
+}
 
 std::string unit_type::generate_description() const
 {
@@ -883,16 +897,12 @@ unit_type::experience_accelerator::~experience_accelerator()
 
 int unit_type::experience_needed() const
 {
-	return (lexical_cast_default<int>(cfg_["experience"],500)*experience_modifier)/100;
+       return (experience_needed_ * experience_modifier) /100;
 }
 
 std::vector<std::string> unit_type::advances_to() const
 {
-	const std::string& val = cfg_["advanceto"];
-	if(val == "null" || val == "")
-		return std::vector<std::string>();
-	else
-		return utils::split(val);
+    return advances_to_;
 }
 
 const config::child_list& unit_type::modification_advancements() const
@@ -1001,7 +1011,7 @@ bool unit_type::not_living() const
 
 bool unit_type::can_advance() const
 {
-	return can_advance_;
+	return !advances_to_.empty();
 }
 
 bool unit_type::has_zoc() const
@@ -1137,6 +1147,38 @@ const unit_animation* unit_type::die_animation(const attack_type* attack) const
 	}
 }
 
+void unit_type::add_advancement(const unit_type &to_unit,int xp)
+{
+	const std::string &to_id =  to_unit.cfg_["id"];
+	const std::string &from_id =  cfg_["id"];
+
+	// add extra advancement path to this unit type
+	lg::info(lg::config) << "adding advancement from " << from_id << " to " << to_id << "\n";
+	advances_to_.push_back(to_id);
+	if(xp>0 && experience_needed_>xp) experience_needed_=xp;
+
+	// add advancements to gendered subtypes, if supported by to_unit
+	for(int gender=0; gender<=1; ++gender) {
+		if(gender_types_[gender] == NULL) continue;
+		if(to_unit.gender_types_[gender] == NULL) {
+			lg::warn(lg::config) << to_id << " does not support gender " << gender << "\n";
+			continue;
+		}
+		lg::info(lg::config) << "gendered advancement " << gender << ": ";
+		gender_types_[gender]->add_advancement(*(to_unit.gender_types_[gender]),xp);
+	}
+
+	// add advancements to variation subtypes
+	// since these are still a rare and special-purpose feature,
+	// we assume that the unit designer knows what they're doing,
+	// and don't block advancements that would remove a variation
+	for(variations_map::iterator v=variations_.begin();
+	    v!=variations_.end(); ++v) {
+		lg::info(lg::config) << "variation advancement: ";
+		v->second->add_advancement(to_unit,xp);
+	}
+}
+
 game_data::game_data()
 {}
 
@@ -1148,7 +1190,7 @@ game_data::game_data(const config& cfg)
 void game_data::set_config(const config& cfg)
 {
 	static const std::vector<config*> dummy_traits;
-	
+
 	const config::child_list& unit_traits = cfg.get_children("trait");
 
 	for(config::const_child_itors i = cfg.child_range("movetype");
@@ -1170,6 +1212,27 @@ void game_data::set_config(const config& cfg)
 		const unit_type u_type(**j.first,movement_types,races,unit_traits);
 		unit_types.insert(std::pair<std::string,unit_type>(u_type.id(),u_type));
 	}
+
+        // fix up advance_from references
+        for(config::const_child_itors k = cfg.child_range("unit");
+            k.first != k.second; ++k.first)
+          for(config::const_child_itors af = (*k.first)->child_range("advancefrom");
+            af.first != af.second; ++af.first) {
+                const std::string &to = (**k.first)["id"];
+                const std::string &from = (**af.first)["unit"];
+                const int xp = lexical_cast_default<int>((**af.first)["experience"],0);
+
+                unit_type_map::iterator from_unit = unit_types.find(from);
+                unit_type_map::iterator to_unit = unit_types.find(to);
+                if(from_unit==unit_types.end()) {
+                  lg::warn(lg::config) << "unknown unit " << from << " in advancefrom\n";
+                        continue;
+                }
+                wassert(to_unit!=unit_types.end());
+
+                from_unit->second.add_advancement(to_unit->second,xp);
+        }
+
 }
 
 void game_data::clear()
