@@ -1,7 +1,7 @@
-/* $Id: unit_types.hpp,v 1.67 2005/03/29 17:41:09 darthfool Exp $ */
+/* $Id: unit_types.hpp,v 1.70 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -79,7 +79,7 @@ public:
 	enum RANGE { SHORT_RANGE, LONG_RANGE };
 
 	attack_type(const config& cfg);
-	const t_string& name() const;
+	const std::string& name() const;
 	const std::string& type() const;
 	const std::string& special() const;
 	const std::string& icon() const;
@@ -102,7 +102,7 @@ public:
 private:
 	std::vector<unit_animation> animation_;
 	std::vector<unit_animation> direction_animation_[6];
-	t_string name_;
+	std::string name_;
 	std::string type_;
 	std::string special_;
 	std::string icon_;
@@ -163,6 +163,11 @@ public:
 
 	~unit_type();
 
+        // adds an additional advancement path to a unit type
+        // this is used to implement the [advancefrom] tag
+        void add_advancement(const unit_type &advance_to,int experience);
+
+
 	const unit_type& get_gender_unit_type(unit_race::GENDER gender) const;
 	const unit_type& get_variation(const std::string& name) const;
         //info on the type of unit that the unit reanimates as
@@ -278,7 +283,9 @@ private:
 	bool teleport_;
 	bool nightvision_;
 	bool steadfast_;
-	bool can_advance_;
+        std::vector<std::string> advances_to_;
+        int experience_needed_;
+
 	ALIGNMENT alignment_;
 
 	unit_movement_type movementType_;
