@@ -1,7 +1,7 @@
-/* $Id: util.hpp,v 1.21 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: util.hpp,v 1.25 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -40,6 +40,11 @@ inline bool is_odd(T num) {
 template<typename T>
 inline bool is_even(T num) { return !is_odd(num); }
 
+// guarantees portable results for division by 100
+inline int div100(int num) {
+  return num < 0 ? -((-num) / 100) : num / 100;
+}
+
 struct bad_lexical_cast {};
 
 template<typename To, typename From>
@@ -47,7 +52,7 @@ To lexical_cast(From a)
 {
 	To res;
 	std::stringstream str;
-	
+
 	if(!(str << a && str >> res)) {
 		throw bad_lexical_cast();
 	} else {
@@ -60,7 +65,7 @@ To lexical_cast_default(From a, To def=To())
 {
 	To res;
 	std::stringstream str;
-	
+
 	if(!(str << a && str >> res)) {
 		return def;
 	} else {
@@ -75,6 +80,7 @@ std::string str_cast(From a)
 }
 
 inline bool chars_equal_insensitive(char a, char b) { return tolower(a) == tolower(b); }
+inline bool chars_less_insensitive(char a, char b) { return tolower(a) < tolower(b); }
 
 //a definition of 'push_back' for strings, since some implementations
 //don't support string::push_back
