@@ -1,7 +1,7 @@
-/* $Id: video.cpp,v 1.63 2005/03/26 17:10:32 silene Exp $ */
+/* $Id: video.cpp,v 1.69 2005/06/06 20:12:47 gruikya Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -120,9 +120,9 @@ surface get_video_surface()
 	return frameBuffer;
 }
 
-const SDL_Rect& screen_area()
+SDL_Rect screen_area()
 {
-	static const SDL_Rect res = {0,0,frameBuffer->w,frameBuffer->h};
+	const SDL_Rect res = {0,0,frameBuffer->w,frameBuffer->h};
 	return res;
 }
 
@@ -201,7 +201,7 @@ void update_whole_screen()
 }
 CVideo::CVideo() : bpp(0), fake_screen(false), help_string_(0), updatesLocked_(0)
 {
-	const int res = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE);
+	const int res = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
 
 	if(res < 0) {
 		ERR_DP << "Could not initialize SDL: " << SDL_GetError() << "\n";
@@ -212,7 +212,7 @@ CVideo::CVideo() : bpp(0), fake_screen(false), help_string_(0), updatesLocked_(0
 CVideo::CVideo( int x, int y, int bits_per_pixel, int flags)
 		 : bpp(0), fake_screen(false), help_string_(0), updatesLocked_(0)
 {
-	const int res = SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE);
+	const int res = SDL_Init( SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
 	if(res < 0) {
 		ERR_DP << "Could not initialize SDL: " << SDL_GetError() << "\n";
 		throw CVideo::error();
@@ -236,7 +236,7 @@ void CVideo::blit_surface(int x, int y, surface surf, SDL_Rect* srcrect, SDL_Rec
 {
 	const surface target(getSurface());
 	SDL_Rect dst = {x,y,0,0};
-	
+
 	if(clip_rect != NULL) {
 		const clip_rect_setter clip_setter(target,*clip_rect);
 		SDL_BlitSurface(surf,srcrect,target,&dst);
@@ -334,7 +334,17 @@ void CVideo::flip()
 	if(update_all) {
 		::SDL_Flip(frameBuffer);
 	} else if(update_rects.empty() == false) {
-		SDL_UpdateRects(frameBuffer,update_rects.size(),&update_rects[0]);
+		size_t sum = 0;
+		for(size_t n = 0; n != update_rects.size(); ++n) {
+			sum += update_rects[n].w*update_rects[n].h;
+		}
+
+		const size_t redraw_whole_screen_threshold = 80;
+		if(sum > ((getx()*gety())*redraw_whole_screen_threshold)/100) {
+			::SDL_Flip(frameBuffer);
+		} else {
+			SDL_UpdateRects(frameBuffer,update_rects.size(),&update_rects[0]);
+		}
 	}
 
 	clear_updates();
