@@ -1,7 +1,7 @@
-/* $Id: video.hpp,v 1.34 2005/03/24 21:35:52 ydirson Exp $ */
+/* $Id: video.hpp,v 1.36 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -23,7 +23,7 @@
 
 surface display_format_alpha(surface surf);
 surface get_video_surface();
-const SDL_Rect& screen_area();
+SDL_Rect screen_area();
 
 bool non_interactive();
 
