@@ -1,7 +1,7 @@
-/* $Id: button.hpp,v 1.30 2005/03/18 21:21:48 ydirson Exp $ */
+/* $Id: button.hpp,v 1.33 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -57,6 +57,8 @@ protected:
 
 private:
 
+	void calculate_size();
+
 	std::string label_;
 	surface image_, pressedImage_, activeImage_, pressedActiveImage_;
 	SDL_Rect textRect_;
@@ -72,6 +74,10 @@ private:
 
 	bool pressed_;
 
+	SPACE_CONSUMPTION spacing_;
+
+	int base_height_, base_width_;
+
 	bool hit(int x, int y) const;
 }; //end class button
 
