@@ -14,7 +14,13 @@ progress_bar::progress_bar(CVideo& video) : widget(video), progress_(0)
 void progress_bar::set_progress_percent(int progress)
 {
 	progress_ = progress;
-	set_dirty(true);
+	set_dirty();
+}
+
+void progress_bar::set_text(const std::string& text)
+{
+	text_ = text;
+	set_dirty();
 }
 
 void progress_bar::draw_contents()
@@ -30,7 +36,8 @@ void progress_bar::draw_contents()
 		inner_area.w = (inner_area.w*progress_)/100;
 		SDL_FillRect(surf,&inner_area,SDL_MapRGB(surf->format,150,0,0));
 
-		const std::string text = str_cast(progress_) + "%";
+		const std::string text = text_.empty() ? str_cast(progress_) + "%" :
+		                         text_ + " (" + str_cast(progress_) + "%)";
 		SDL_Rect text_area = font::text_area(text,font::SIZE_NORMAL);
 
 		text_area.x = area.x + area.w/2 - text_area.w/2;
