@@ -11,11 +11,13 @@ public:
 	progress_bar(CVideo& video);
 
 	void set_progress_percent(int progress);
+	void set_text(const std::string& text);
 
 	void draw_contents();
 
 private:
 	int progress_;
+	std::string text_;
 };
 
 }
