@@ -1,7 +1,7 @@
-/* $Id: slider.hpp,v 1.23 2005/03/18 21:21:48 ydirson Exp $ */
+/* $Id: slider.hpp,v 1.26 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -38,7 +38,10 @@ public:
 	int min_value() const;
 
 	virtual void set_location(SDL_Rect const &rect);
-	using widget::set_location;
+
+	//VC++ doesn't like a 'using scrollarea::set_location' directive here, so we declare
+	//an inline forwarding function instead
+	void set_location(int x, int y) { widget::set_location(x,y); }
 
 protected:
 	virtual void handle_event(const SDL_Event& event);
