@@ -1,7 +1,7 @@
-/* $Id: textbox.cpp,v 1.75 2005/03/26 17:10:32 silene Exp $ */
+/* $Id: textbox.cpp,v 1.82 2005/06/04 19:16:06 ott Exp $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
@@ -19,8 +19,10 @@
 #include "../language.hpp"
 #include "../log.hpp"
 #include "../sdl_utils.hpp"
+#include "../serialization/string_utils.hpp"
 #include "../util.hpp"
 #include "../video.hpp"
+
 #include "SDL.h"
 
 #include <algorithm>
@@ -76,6 +78,11 @@ void textbox::append_text(const std::string& text)
 		return;
 	}
 
+	//disallow adding multi-line text to a single-line text box
+	if(wrap_ == false && std::find_if(text.begin(),text.end(),utils::isnewline) != text.end()) {
+		return;
+	}
+
 	const wide_string& wtext = utils::string_to_wstring(text);
 
 	const surface new_text = add_text_line(wtext);
@@ -125,9 +132,13 @@ void textbox::draw_contents()
 	surface surf = video().getSurface();
 	draw_solid_tinted_rectangle(loc.x,loc.y,loc.w,loc.h,0,0,0,
 				    focus() ? alpha_focus_ : alpha_, surf);
-	
+
 	SDL_Rect src;
 
+	if(text_image_ == NULL) {
+		update_text_cache(true);
+	}
+
 	if(text_image_ != NULL) {
 		src.y = yscroll_;
 		src.w = minimum<size_t>(loc.w,text_image_->w);
@@ -239,16 +250,16 @@ surface textbox::add_text_line(const wide_string& text)
 	}
 
 	char_x_.push_back(0);
-	
+
 	// Re-calculate the position of each glyph. We approximate this by asking the
 	// width of each substring, but this is a flawed assumption which won't work with
 	// some more complex scripts (that is, RTL languages). This part of the work should
 	// actually be done by the font-rendering system.
 	std::string visible_string;
 	wide_string wrapped_text;
-	 
+
 	wide_string::const_iterator backup_itor = text.end();
-	
+
 	wide_string::const_iterator itor = text.begin();
 	while(itor != text.end()) {
 		//If this is a space, save copies of the current state so we can roll back
@@ -288,7 +299,7 @@ surface textbox::add_text_line(const wide_string& text)
 	}
 
 	const std::string s = utils::wstring_to_string(wrapped_text);
-	const surface res(font::get_rendered_text(s, font_size, font::NORMAL_COLOUR));		
+	const surface res(font::get_rendered_text(s, font_size, font::NORMAL_COLOUR));
 
 	return res;
 }
@@ -318,7 +329,7 @@ void textbox::update_text_cache(bool changed)
 	}
 }
 
-bool textbox::is_selection() 
+bool textbox::is_selection()
 {
 	return (selstart_ != -1) && (selend_ != -1) && (selstart_ != selend_);
 }
@@ -327,7 +338,7 @@ void textbox::erase_selection()
 {
 	if(!is_selection())
 		return;
-	
+
 	wide_string::iterator itor = text_.begin() + minimum(selstart_, selend_);
 	text_.erase(itor, itor + abs(selend_ - selstart_));
 	cursor_ = minimum(selstart_, selend_);
@@ -336,8 +347,13 @@ void textbox::erase_selection()
 
 void textbox::handle_event(const SDL_Event& event)
 {
+	scrollarea::handle_event(event);
+
 	bool changed = false;
-	
+
+	const int old_selstart = selstart_;
+	const int old_selend = selend_;
+
 	//Sanity check: verify that selection start and end are within text
 	//boundaries
 	if(is_selection() && !(size_t(selstart_) <= text_.size() && size_t(selend_) <= text_.size())) {
@@ -378,7 +394,7 @@ void textbox::handle_event(const SDL_Event& event)
 
 		if(grabmouse_)
 			selend_ = cursor_;
-		
+
 		update_text_cache(false);
 
 		if(!grabmouse_ && mousebuttons & SDL_BUTTON(1)) {
@@ -400,7 +416,7 @@ void textbox::handle_event(const SDL_Event& event)
 	if(focus() == false) {
 		if (event.type == SDL_MOUSEMOTION && point_in_rect(mousex, mousey, loc))
 			events::focus_handler(this);
-		   
+
 		return;
 	}
 
@@ -411,28 +427,28 @@ void textbox::handle_event(const SDL_Event& event)
 
 	const SDL_keysym& key = reinterpret_cast<const SDL_KeyboardEvent&>(event).keysym;
 	const SDLMod modifiers = SDL_GetModState();
-	
+
 	const int c = key.sym;
-	int old_cursor = cursor_;
-	
-	if(c == SDLK_LEFT && cursor_ > 0) 
+	const int old_cursor = cursor_;
+
+	if(c == SDLK_LEFT && cursor_ > 0)
 		--cursor_;
 
 	if(c == SDLK_RIGHT && cursor_ < text_.size())
 		++cursor_;
 
-	if(c == SDLK_END)
+	if(c == SDLK_END || (c == SDLK_e && (modifiers & KMOD_CTRL)))
 		cursor_ = text_.size();
-	
-	if(c == SDLK_HOME)
+
+	if(c == SDLK_HOME || (c == SDLK_a && (modifiers & KMOD_CTRL)))
 		cursor_ = 0;
 
 	if((old_cursor != cursor_) && (modifiers & KMOD_SHIFT)) {
-		if(selstart_ == -1) 
+		if(selstart_ == -1)
 			selstart_ = old_cursor;
 		selend_ = cursor_;
-	} 
-	
+	}
+
 	if(c == SDLK_BACKSPACE) {
 		changed = true;
 		if(is_selection()) {
@@ -443,6 +459,12 @@ void textbox::handle_event(const SDL_Event& event)
 		}
 	}
 
+	if(c == SDLK_u && (modifiers & KMOD_CTRL)) {
+		changed = true;
+		cursor_ = 0;
+		text_.resize(0);
+	}
+
 	if(c == SDLK_DELETE && !text_.empty()) {
 		changed = true;
 		if(is_selection()) {
@@ -461,10 +483,11 @@ void textbox::handle_event(const SDL_Event& event)
 
 	//movement characters may have a "Unicode" field on some platforms, so ignore it.
 	if(!(c == SDLK_UP || c == SDLK_DOWN || c == SDLK_LEFT || c == SDLK_RIGHT ||
-	   c == SDLK_DELETE || c == SDLK_BACKSPACE || c == SDLK_END || c == SDLK_HOME)) {
+	   c == SDLK_DELETE || c == SDLK_BACKSPACE || c == SDLK_END || c == SDLK_HOME ||
+	   c == SDLK_PAGEUP || c == SDLK_PAGEDOWN)) {
 		if(character != 0)
 			LOG_STREAM(info, display) << "Char: " << character << ", c = " << c << "\n";
-	
+
 		if(event.key.keysym.mod & KMOD_CTRL) {
 			switch(c) {
 			case SDLK_v:
@@ -473,7 +496,13 @@ void textbox::handle_event(const SDL_Event& event)
 				if(is_selection())
 					erase_selection();
 
-				wide_string s = utils::string_to_wstring(copy_from_clipboard());
+				std::string str = copy_from_clipboard();
+
+				//cut off anything after the first newline
+				str.erase(std::find_if(str.begin(),str.end(),utils::isnewline),str.end());
+
+				wide_string s = utils::string_to_wstring(str);
+
 				if(text_.size() < max_size_) {
 					if(s.size() + text_.size() > max_size_) {
 						s.resize(max_size_ - text_.size());
@@ -481,6 +510,7 @@ void textbox::handle_event(const SDL_Event& event)
 					text_.insert(text_.begin()+cursor_, s.begin(), s.end());
 					cursor_ += s.size();
 				}
+
 				}
 
 				break;
@@ -493,18 +523,18 @@ void textbox::handle_event(const SDL_Event& event)
 				wide_string ws = wide_string(text_.begin() + beg, text_.begin() + end);
 				std::string s = utils::wstring_to_string(ws);
 				copy_to_clipboard(s);
-				} 
+				}
 				break;
 			}
 		} else {
 			if(character >= 32 && character != 127) {
 				changed = true;
-				if(is_selection()) 
+				if(is_selection())
 					erase_selection();
 
 				if(text_.size() + 1 <= max_size_) {
 					text_.insert(text_.begin()+cursor_,character);
-					++cursor_;		
+					++cursor_;
 				}
 			}
 		}
@@ -518,9 +548,11 @@ void textbox::handle_event(const SDL_Event& event)
 	show_cursor_ = true;
 	show_cursor_at_ = SDL_GetTicks();
 
-	update_text_cache(changed);
+	if(changed || old_cursor != cursor_ || old_selstart != selstart_ || old_selend != selend_) {
+		text_image_ = NULL;
+	}
+
 	set_dirty(true);
-	draw();
 }
 
 } //end namespace gui
